/*
 * cmd_layoutview.c - show flash layout
 *
 * Licensed under the GPL-2 or later.
 */
 
#include <common.h>
#include <command.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <gpt_parse.h>


#include <asm/arch/sys_flash_layout.h>
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
  #include <mmc.h>
#endif

#define MAX_COMPONENT_NUM	64

struct compontent_info
{
	unsigned int offset_lo;  
	unsigned int offset_hi;
	unsigned int align_size_lo;
	unsigned int align_size_hi;
	char comp_name[32];
};

struct compontent_info components[MAX_COMPONENT_NUM] = {0};
unsigned int cur_component_index = 0;
unsigned long long free_space_low_addr = 0;
unsigned long long free_space_high_addr = 0;

unsigned long long get_flash_size(void)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)	
	struct mmc *mmc;
	int cur_dev = -1;

	if (get_mmc_num() > 0)
		cur_dev = 0;
	
	mmc = find_mmc_device(cur_dev);
	if (!mmc) {
		printf("no mmc device at slot %x\n", cur_dev);
		return 0;
	}

	return mmc->capacity;
#else
	return 0;
#endif
}

static unsigned long long uint_to_ull(unsigned int hi, unsigned lo)
{
	return ((unsigned long long)((unsigned long long)hi<<32)+(unsigned long long)lo);
}

static void fill_component_info(unsigned long long offset, unsigned long long align_size, char *name)
{
	if(cur_component_index>=MAX_COMPONENT_NUM){
		printf("component number beyound max value!\r\n");
		return;
	}
	
	components[cur_component_index].offset_lo = (unsigned int)(offset & 0xFFFFFFFF);
	components[cur_component_index].offset_hi = (unsigned int)((offset >> 32) & 0xFFFFFFFF);
	
	components[cur_component_index].align_size_lo = (unsigned int)(align_size & 0xFFFFFFFF);
	components[cur_component_index].align_size_hi = (unsigned int)((align_size >> 32) & 0xFFFFFFFF);

	snprintf(components[cur_component_index].comp_name, 32, "%s", name);
	
	cur_component_index++;
}

static int boot_area_fill_component_info(void)
{

	fill_component_info(0x0, SYS_PARAM_BOOTCODE_SIZE, "bootcode");

	fill_component_info(SYS_PARAM_FACTORY_RO_START, SYS_PARAM_FACTORY_RO_SIZE, "factory ro");
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)	
	fill_component_info(SYS_PARAM_FACTORY_EEPROM_START, SYS_PARAM_FACTORY_EEPROM_SIZE, "customer Zone");
#endif
	fill_component_info(SYS_PARAM_FACTORY_START, SYS_PARAM_FACTORY_SIZE, "factory");

#ifdef Config_Board_TV002_V001
	fill_component_info(SYS_PARAM_SECURE_STORE_START, SYS_PARAM_SECURE_STORE_SIZE, "take over");
#else
	fill_component_info(SYS_PARAM_SECURE_STORE_START, SYS_PARAM_SECURE_STORE_SIZE, "secure store");
#endif

#if defined(CONFIG_SYS_RTK_EMMC_FLASH)	
	fill_component_info(SYS_PARAM_RESERVED_START, SYS_PARAM_RESERVED_SIZE, "reserved");
#endif

	fill_component_info(SYS_PARAM_FW_TABLE_START, 0x8000, "fw table");
	
	return 0;
}

static int fw_area_fill_component_info(unsigned char *fw_desc_table_base )
{
	unsigned int loop = 0, fw_entry_num = 0;
	unsigned int version = 0;
	unsigned int unit_len = 0;
	unsigned int offset = 0;
#if defined(Config_Vulcan3_PCB_TRUE)
	unsigned int audio_file_num = 0;
#else
	unsigned int video_file_num = 0, audio_file_num = 0;
#endif
	char file_name[32] = {0};

	fw_desc_table_v1_t *fw_desc_table_v1 = NULL;
	fw_desc_entry_v1_t *fw_entry = NULL;

	fw_desc_table_v1 = (fw_desc_table_v1_t *)fw_desc_table_base;

	version = fw_desc_table_v1->version;
	/* find fw_entry structure according to version */
	switch (version)
	{
		case FW_DESC_TABLE_V1_T_VERSION_1:
			unit_len = sizeof(fw_desc_entry_v1_t);
			break;

		case FW_DESC_TABLE_V1_T_VERSION_11:
			unit_len = sizeof(fw_desc_entry_v11_t);
			break;

		case FW_DESC_TABLE_V1_T_VERSION_21:
			unit_len = sizeof(fw_desc_entry_v21_t);
			break;

		default:
			return -1;
	}

	fw_entry_num = fw_desc_table_v1->fw_list_len / unit_len;
	offset = sizeof(fw_desc_table_v1_t) + fw_desc_table_v1->part_list_len;

	for(loop=0; loop < fw_entry_num; loop++){
		fw_entry = (fw_desc_entry_v1_t *)((char *)fw_desc_table_v1+offset+
					unit_len*loop);
		
		if((fw_entry->offset+ fw_entry->paddings)>free_space_low_addr)
			free_space_low_addr = fw_entry->offset+ fw_entry->paddings;
		
		switch(fw_entry->type){
			case FW_TYPE_KERNEL:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "linuxKernel");
				break;
			case FW_TYPE_AUDIO:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "audioKernel");
				break;
			case FW_TYPE_AUDIO_2:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "audio2Kernel");
				break;
			case FW_TYPE_VIDEO:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "videoKernel");
				break;
			case FW_TYPE_VIDEO_2:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "video2Kernel");
				break;
			case FW_TYPE_KCPU:
				if(fw_entry->reserved_1[0] == 0x1)
					fill_component_info(fw_entry->offset, fw_entry->paddings, "tee");
				else
					fill_component_info(fw_entry->offset, fw_entry->paddings, "kcpuKernel");
				break;
			case FW_TYPE_ECPU:
				fill_component_info(fw_entry->offset, fw_entry->paddings, "emcu");
				break;
			case FW_TYPE_AUDIO_FILE:
				if(!audio_file_num){
					fill_component_info(fw_entry->offset, fw_entry->paddings, "audioFile");
					audio_file_num++;
				}else{
					snprintf(file_name, sizeof(file_name), "%s%d", "audioFile", audio_file_num);
					fill_component_info(fw_entry->offset, fw_entry->paddings, file_name);
					audio_file_num++;
				}
				break;
			case FW_TYPE_VIDEO_FILE:
			case FW_TYPE_IMAGE_FILE:
#if defined(Config_Vulcan3_PCB_TRUE)
			case FW_TYPE_VIDEO_RAW_FILE_2K:
					fill_component_info(fw_entry->offset, fw_entry->paddings, "videoFile_2k");
				break;
			case FW_TYPE_VIDEO_RAW_FILE_4K:
					fill_component_info(fw_entry->offset, fw_entry->paddings, "videoFile_4k");
				break;
#else
			case FW_TYPE_VIDEO_RAW_FILE:
				if(!video_file_num){
					fill_component_info(fw_entry->offset, fw_entry->paddings, "videoFile");
					video_file_num++;
				}else{
					snprintf(file_name, sizeof(file_name), "%s%d", "videoFile", video_file_num);
					fill_component_info(fw_entry->offset, fw_entry->paddings, file_name);
					video_file_num++;
				}
				break;
#endif
			default:
				break;
		}
	}

	return 0;
}

static int part_area_fill_component_info(void)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	int ret = 0;
	unsigned int part_num = 0;
	unsigned int loop = 0;
	gpt_entry entry;
	char partname[256] = {0};
	unsigned long long start = 0, end = 0;

	ret = get_gpt_entry_num(&part_num);
	if(ret < 0){
		return -1;
	}

	for(loop = 0; loop < part_num; loop ++){
		get_gpt_entry_by_index(&entry, loop);
		
		start = get_partition_starting_by_gpt_entry(entry);
		end = get_partition_ending_by_gpt_entry(entry);
		get_partition_partname_by_gpt_entry(entry, partname);

		if(free_space_high_addr == 0)
			free_space_high_addr = start;
		
		fill_component_info(start, end-start, partname);
	}

	return 0;
#else
	printf("Not support!\r\n");
	return -1;
#endif
}

static int free_space_fill_component_info(void)
{
	fill_component_info(free_space_low_addr, (free_space_high_addr - free_space_low_addr), "FREE SPACE");
	return 0;
}

static int fw_table_is_invalidate(unsigned char* fw_desc_table_base)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	unsigned int fw_desc_table_blk = 0;
	fw_desc_table_v1_t fw_desc_table_v1;

	fw_desc_table_blk = SYS_PARAM_FW_TABLE_START/EMMC_BLOCK_SIZE;
	
	/* copy Firmware Description Table Header from flash */
	if (!rtk_eMMC_read(fw_desc_table_blk, sizeof(fw_desc_table_v1_t), (uint *)fw_desc_table_base)){
		printf("[ERR] %s:Read fw_desc_table_v1_t error! (0x%x, 0x%x, 0x%x)\n",
			__FUNCTION__, fw_desc_table_blk, sizeof(fw_desc_table_v1_t), (unsigned int)fw_desc_table_base);

		return -1;
	}else{
		memcpy(&fw_desc_table_v1, (void*)fw_desc_table_base, sizeof(fw_desc_table_v1_t));
	}

	// check signature
	if (strncmp((const char *)fw_desc_table_v1.signature,
			FIRMWARE_DESCRIPTION_TABLE_SIGNATURE,
			sizeof(fw_desc_table_v1.signature)) != 0){
		//printf("[ERR] %s:Signature(%s) error!\n", __FUNCTION__, fw_desc_table_v1.signature);
		printf("[Warning] Your platform un-upgrade install.img!\r\n");
		return -1;
	}

	/* copy Firmware Description/Partition/Fw_entry Tables from flash */
	if (!rtk_eMMC_read(fw_desc_table_blk, fw_desc_table_v1.paddings, (uint *)fw_desc_table_base))
	{
		printf("[ERR] %s:Read all fw tables error! (0x%x, 0x%x, 0x%x)\n",
			__FUNCTION__, fw_desc_table_blk, fw_desc_table_v1.paddings, (unsigned int)fw_desc_table_base);

		return -1;
	}

	return 0;
#else
	printf("Not support!\r\n");
	return -1;
#endif
}


static int layout_fill_component_info(void)
{
	unsigned int fw_desc_table_base = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
		
	// fill bootcode info
	boot_area_fill_component_info();

	if(fw_table_is_invalidate((unsigned char *)fw_desc_table_base)<0){
		free_space_low_addr = SYS_PARAM_FW_TABLE_START+ 0x8000;
		free_space_high_addr = get_flash_size();
	}else{
		// fill fw info
		fw_area_fill_component_info((unsigned char *)fw_desc_table_base);

		// fill part info
		part_area_fill_component_info();
	}

	free_space_fill_component_info();
	
	return 0;
}

static int components_sort_by_offset(void)
{
	struct compontent_info tmp_comp;
	int i, j;

	for(i=0; i<cur_component_index; i++){
		for(j=0; j<(cur_component_index-1-i); j++){
			unsigned long long cur_comp_offset = uint_to_ull(components[j].offset_hi,  components[j].offset_lo);
			unsigned long long post_comp_offset = uint_to_ull(components[j+1].offset_hi,  components[j+1].offset_lo);

			if(cur_comp_offset < post_comp_offset){
				tmp_comp = components[j];
				components[j] = components[j+1];
				components[j+1] = tmp_comp;
			}
		}
	}

	return 0;
}

static int layout_show(void)
{
	unsigned int loop = 0;
	struct compontent_info comp, prev_comp;
	unsigned int first_flag = 1;
	
	printf("\r\n[layout view]\r\n");
	for(loop = 0 ; loop <  cur_component_index; loop++)
	{
		unsigned long long align_size = 0;
		unsigned int align_size_kb = 0;;
		
		comp = components[loop];

		align_size = uint_to_ull(comp.align_size_hi, comp.align_size_lo);
		align_size_kb = (unsigned int)(align_size >> 10);

		if(align_size == 0)
			continue;
		
		if (first_flag) {
			unsigned long long max_value = uint_to_ull(comp.offset_hi, comp.offset_lo) + \
				uint_to_ull(comp.align_size_hi, comp.align_size_lo);
			
			printf("	 +-----------------------------+ 0x%08x%08x\r\n",
				(unsigned int)((max_value >> 32) & 0xFFFFFFFF), (unsigned int)(max_value & 0xFFFFFFFF));
			
			first_flag = 0;
		}else{
			printf("	 +-----------------------------+ 0x%08x%08x\r\n",prev_comp.offset_hi, prev_comp.offset_lo);
		}
		
		printf("	 |%12s (%6uKBytes)	|\r\n",comp.comp_name,align_size_kb);
		
		prev_comp = comp;
	}
	printf("	 +-----------------------------+ 0x%016x\r\n",0);
	printf("\r\n");
	return 0;
}

static int layout_view_emmc(void)
{
	// layout fill
	layout_fill_component_info();

	//layout sort
	components_sort_by_offset();
	
	//layout show
	layout_show();
	
	return 0;
}

static int layout_view_nand(void)
{
	// TODO
	printf("Not support!\r\n");
	return 0;
}

static int layout_view_spi(void)
{
	// TODO
	printf("Not support!\r\n");
	return 0;
}

static int do_layout_view(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;

	if((argc != 2) && (strcmp(argv[1], "view") != 0))
		return CMD_RET_USAGE;
	else{
		// clean global var
		memset(&components, 0x00, sizeof(struct compontent_info)*MAX_COMPONENT_NUM);
		cur_component_index = 0;
		free_space_low_addr = 0;
		free_space_high_addr = 0;
	}
	
	if (boot_flash_type == BOOT_EMMC)
		ret = layout_view_emmc();
	else if (boot_flash_type == BOOT_NAND)
		ret = layout_view_nand();
	else
		ret = layout_view_spi();

	if(ret < 0)
		return CMD_RET_FAILURE;
	else
		return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	layout, 2, 1, do_layout_view,
	"show flash layout",
	"\n"
	"layout view - display flash layout info\n"
);

