#include <linux/string.h>
#include <platform_lib/board/pcb_mgr.h>
#include <platform_lib/board/pcb.h>
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <linux/compiler.h>
#include <asm/io.h>
#include "AudioAmp.h"
#include <drivers/i2c/rtk_i2c_lib.h>
#include <drivers/i2c/rtk_g2c_lib.h>


void AudioAmp_InitACL1310(void)
{
    //***** ACL1310 I2S init *****//
    int regAddr =0 , regval = 0;
    regAddr = 0xb80008f4;
    regval = rtd_inl(regAddr);
    regval = (regval|0x04a40000);
    rtd_outl(regAddr, regval);

    regAddr = 0xb80008c0;
    regval = rtd_inl(regAddr);
    regval = (regval|0x00000900);
    rtd_outl(regAddr, regval);

    regAddr = 0xb8000c4c;
    regval = rtd_inl(regAddr);
    regval = (regval|0x20000000);
    rtd_outl(regAddr, regval);

#if defined(CONFIG_BOARD_298X_TV013_BGA459_EMMC)   //Haier 2984 use 0xb80008b8:bit[21:17]
 //   regAddr = 0xb80008b8;
 //   regval = rtd_inl(regAddr);
 //   regval = (regval|0x004c0000);
 //   rtd_outl(regAddr, regval);
#else
    regAddr = 0xb80008b8;
    regval = rtd_inl(regAddr);
    regval = (regval|0x004c0000);
    rtd_outl(regAddr, regval);

#endif

    regAddr = 0xb8000c2c;
    regval = rtd_inl(regAddr);
    regval = (regval|0x00800000);
    rtd_outl(regAddr, regval);

    regAddr = 0xb8060248;
    regval = rtd_inl(regAddr);
    regval = (regval|0x00f80000);
    rtd_outl(regAddr, regval);
    
    regAddr = 0xb8061100;
    regval = rtd_inl(regAddr);
    regval = (regval|0x00008000);
    rtd_outl(regAddr, regval);
    
    regAddr = 0xb8061108;
    regval = rtd_inl(regAddr);
    regval = (regval|0x00008000);
    rtd_outl(regAddr, regval);

}


//*************************************************************
// PUBLIC FUNCTIONS
//*************************************************************
#define END_FLAG 	(0xff)
#define AMP_DEBUG 	(0)

typedef struct
{
    unsigned char bAddr;
    unsigned char bLength;
    unsigned char bArray[2];
}ALC1310_REG;//markchen add for 1310

static ALC1310_REG Alc1310_DefautTbl[] = 
{
    //init
    {0x05,0x02,{0x40,0x3e}},					// 
    {0x02,0x02,{0x80,0x00}},					//
    {0x09,0x02,{0x80,0x00}},					//
    //DC
    {0x6a,0x02,{0x00,0xdc}},					//
    {0x6c,0x02,{0x00,0x24}},					//
    //D9
    {0x14,0x02,{0x00,0xd9}},					//
    {0x50,0x02,{0xa4,0x10}},					//

    //DRC
    {0x09,0x02,{0xC8,0x00}},//DRC Enable
    {0x0a,0x02,{0x05,0x00}},					//
    {0x0c,0x02,{0x04,0x02}},					//
    {0x0e,0x02,{0x06,0x00}},					//

    /* down layer register and value */
    {0x6a,0x02,{0x00,0x9d}},					//
    {0x6c,0x02,{0x46,0x00}},					//

    {0x6a,0x02,{0x00,0x9e}},					//
    {0x6c,0x02,{0x26,0x26}},					//

    //b0
    {0x6a,0x02,{0x00,0xb0}},					
    {0x6c,0x02,{0x88,0x03}},					//Silence detection:level is -80dB

    {0x13,0x02,{0x00,0x0c}},					//unmute
    {0x13,0x02,{0x00,0x0c}},					//unmute

    //{0x12,0x02,{0xff,0xff}},					//L/R Volume
    //{0x12,0x02,{0x00,0x00}},					//L/R Volume	


    //BQ1 BQ1Ty= 4 BQ1Fs=44100 BQ1Fc=100BQ1Ga=1 BQ1Fb=120
    {0x6a,0x02,{0x00,0x00}}, 
    {0x6c,0x02,{0x02,0x00}},
                            
    {0x6a,0x02,{0x00,0x01}},
    {0x6c,0x02,{0x87,0x8E}},
                            
    {0x6a,0x02,{0x00,0x02}},
    {0x6c,0x02,{0x1C,0x09}},

    {0x6a,0x02,{0x00,0x03}},
    {0x6c,0x02,{0xD4,0xC5}},
		                        
    {0x6a,0x02,{0x00,0x04}},
    {0x6c,0x02,{0x01,0xF6}},
                            
    {0x6a,0x02,{0x00,0x05}},
    {0x6c,0x02,{0x45,0x96}},
                            
    {0x6a,0x02,{0x00,0x06}},
    {0x6c,0x02,{0x03,0xF7}},
                            
    {0x6a,0x02,{0x00,0x07}},
    {0x6c,0x02,{0x37,0xBD}},
                            
    {0x6a,0x02,{0x00,0x08}},
    {0x6c,0x02,{0x1E,0x08}},
		                        
    {0x6a,0x02,{0x00,0x09}},
    {0x6c,0x02,{0xAD,0xE1}},


    //BQ2 BQ2Ty= 4 BQ2Fs=44100 BQ2Fc=6000 BQ2Ga=0.5 BQ2Fb=1500
    {0x6a,0x02,{0x00,0xa}}, 
    {0x6c,0x02,{0x02,0x02}},
                            
    {0x6a,0x02,{0x00,0x0b}},
    {0x6c,0x02,{0x9D,0x55}},
                            
    {0x6a,0x02,{0x00,0x0c}},
    {0x6c,0x02,{0x1D,0x9C}},
                            
    {0x6a,0x02,{0x00,0x0d}},
    {0x6c,0x02,{0xF6,0x0A}},
                            
    {0x6a,0x02,{0x00,0x0e}},
    {0x6c,0x02,{0x01,0xA2}},
                            
    {0x6a,0x02,{0x00,0x0f}},
    {0x6c,0x02,{0xFE,0xEA}},
                            
    {0x6a,0x02,{0x00,0x10}},
    {0x6c,0x02,{0x02,0x66}},
                            
    {0x6a,0x02,{0x00,0x11}},
    {0x6c,0x02,{0x28,0xC3}},
                            
    {0x6a,0x02,{0x00,0x12}},
    {0x6c,0x02,{0x1E,0x58}},
                            
    {0x6a,0x02,{0x00,0x13}},
    {0x6c,0x02,{0x40,0x03}},


    //BQ3 BQ3Ty= 4 BQ3Fs=44100 BQ3Fc=11000 BQ3Ga=1.5 BQ3Fb=14000
    {0x6a,0x02,{0x00,0x14}},
    {0x6c,0x02,{0x02,0x25}},
                            
    {0x6a,0x02,{0x00,0x15}},
    {0x6c,0x02,{0x88,0x62}},
                            
    {0x6a,0x02,{0x00,0x16}},
    {0x6c,0x02,{0x1F,0xFD}},
                            
    {0x6a,0x02,{0x00,0x17}},
    {0x6c,0x02,{0xEC,0x29}},
                            
    {0x6a,0x02,{0x00,0x18}},
    {0x6c,0x02,{0x00,0x47}},
                            
    {0x6a,0x02,{0x00,0x19}},
    {0x6c,0x02,{0x09,0xFB}},
                            
    {0x6a,0x02,{0x00,0x1a}},
    {0x6c,0x02,{0x00,0x02}},
                            
    {0x6a,0x02,{0x00,0x1b}},
    {0x6c,0x02,{0x3A,0xD3}},
                            
    {0x6a,0x02,{0x00,0x1c}},
    {0x6c,0x02,{0x1F,0x8E}},
                            
    {0x6a,0x02,{0x00,0x1d}},
    {0x6c,0x02,{0x38,0x7E}},

    {0x04,0x02,{0xc7,0x00}},


    {END_FLAG ,0x01,{0x00}},
};

void AudioAmp_InitAudioOut2I2S_1st(void)
{
    unsigned char data[8];
    unsigned char regaddr;
    unsigned char u8Index;

    I2C_Init();
    I2C_SetSpeed( AMP_I2C , 100 );

    printf("\n AMP_I2C = %d \n", AMP_I2C );

    for(u8Index = 0; ; u8Index ++)
    {
        regaddr = Alc1310_DefautTbl[u8Index].bAddr;
        data[0] = Alc1310_DefautTbl[u8Index].bAddr;
        if(regaddr == END_FLAG)
        {
            break;
        }
        data[1] = Alc1310_DefautTbl[u8Index].bArray[0];
        data[2] = Alc1310_DefautTbl[u8Index].bArray[1];
#if AMP_DEBUG
        printf("\nwrite addr:0x%x----data[1]=0x%x----data[2]=0x%x\n",regaddr,data[1],data[2]);
#endif
        if (I2C_Write_EX(AMP_I2C, AMP_BOOTCODE_USE_THIS_I2C_ADDRESS,  3, data ) < 0)
            printf("program ACL1310 failed\n");

#if AMP_DEBUG
        unsigned char readArray[2] = {0,0};
        if (I2C_Read_EX(AMP_I2C,AMP_BOOTCODE_USE_THIS_I2C_ADDRESS, 1,  &regaddr, 2, readArray)!= S_OK)
        {
            printf("\n i2c read fail \n"  );
        }
        printf("\nread addr:0x%x----data[0]=0x%x----data[1]=0x%x\n",regaddr,readArray[0],readArray[1]);
#endif

    }


}

void AudioAmp_InitAudioOut2I2S_2nd(void)
{
#if 0
    unsigned long long value=0;
    if (pcb_mgr_get_enum_info_byname("AMP_I2C", &value) < 0)
    {
        printf("no AMP_I2C\n");
        m_I2C_MASTER = 0;
    }
    else
    {
        m_I2C_MASTER = value;
        printf("AMP_I2C=%lld\n", m_I2C_MASTER);
    }
    if (m_I2C_MASTER <= PCB_I2C_3)
        I2C_Init();
    else
        G2C_Init(m_I2C_MASTER);

    if (pcb_mgr_get_enum_info_byname("AMP_DEVICE_ADDR", &value) < 0)
    {
        printf("no AMP_DEVICE_ADDR\n");
        m_AMP_DEVICE_ADDR = 0;
    }
    else
    {
        m_AMP_DEVICE_ADDR = value;
    }

    if (pcb_mgr_get_enum_info_byname("PIN_AMP_MUTE", &value) < 0)
    {
        printf("no PIN_AMP_MUTE\n");
        m_PIN_AMP_MUTE = 0;
    }
    else
    {
        m_PIN_AMP_MUTE = value;
    }

#ifdef Board_PANEL_SLOWBOOT
	udelay(300000);
#endif

    AudioAmp_InitACL1310();
#endif
}

