/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"
#include "adv4_t720_check_0_a.h"
#include "adv4_t720_mem_0.h"

int Check_adv4_t720_check_0_a (struct t_adv4_t720_mem_0 *p);

int RunMaliTest_adv4_t720_part0 (void);
int RunMaliTest_adv4_t720_part1 (void);
int RunMaliTest_adv4_t720_part2 (void);
void load_page_table (void);

static volatile struct t_adv4_t720_mem_0 *malidata_0;
static int mali_step;

int RunMaliTest_adv4_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_adv4_t720_part0();
    case 1  : return RunMaliTest_adv4_t720_part1();
    case 2  : return RunMaliTest_adv4_t720_part2();
    default : return 0;
  };
};

int RunMaliTest_adv4_t720_part0 (void) {
  int res = 0;

  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;

  printf("RUNNING TEST: adv4\n");
  printf("  Purpose: Tests correct operation of the MMU interrupt signal\n");
  printf("  A simple test is run which forces an MMU interrupt by corrupting the page table and causing a page fault.\n");

  // Get current configuration to allow testing all registers
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("Selected configuration:\n");
  printf("  Bus width: %d\n", axi_width);
  printf("  Number of shader cores: %d\n", num_cores);
  printf("  L2 cache size: %d kB\n", ((1<<l2_size)/1024));

  Mali_Reset();

  load_page_table();
  
  mali_step++;
  if (res == 0) {
   return 254;
  }
  return res;
}

int RunMaliTest_adv4_t720_part1 (void) {
  int res = 0;
  int have_interrupts = 0;
  unsigned int ** my_addr;
  /* Check for any interrupt */
  have_interrupts = Mali_InterruptCheck(0xFFFFFFFF,0xFFFFFFFF);
  if (!have_interrupts) {
   return 255;
  }

  Mali_JobPartDone();
  Mali_CheckReg(32, 0, JOB_IRQ_STATUS, 0x00000002);
  res |= Check_adv4_t720_check_0_a(malidata_0);

  Mali_MaskAllInterrupts();
  mali_step++;
  
  printf ("Now corrupt the page table and reload\n");
  my_addr = (unsigned int **) adv4_t720_mem_0.ttb;
  printf ("Corrupting page table: Old data at address %x is %x\n", my_addr, *my_addr);
  *my_addr = (unsigned int *)((char *)adv4_t720_mem_0.ttb + 0x001000 + 0);
  printf ("Corrupting page table: New corrupt data at %x is %x\n", my_addr, *my_addr);
  load_page_table ();
  
  if (res == 0) {
   return 255;
  }
  return res;
}

void load_page_table (void)
{
  
  Mali_WrReg(0x20,0x0, JOB_IRQ_CLEAR       , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, JOB_IRQ_MASK        , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, SHADER_PWRON_LO     , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, TILER_PWRON_LO      , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, JS1_CONFIG_NEXT     , 0x00001000);
  Mali_WrReg(0x20,0x0, JS1_HEAD_NEXT_LO    , 0x00001000);
  Mali_WrReg(0x20,0x0, JS1_AFFINITY_NEXT_LO, 0xFFFFFFFF);
  (struct t_adv4_t720_mem_0 *)malidata_0 = Mali_LdMem(&adv4_t720_mem_0,sizeof(struct t_adv4_t720_mem_0),(int)adv4_t720_mem_0.ttb);
  
  /* execute command */
  Mali_WrReg(0x20,0x0,JS1_COMMAND_NEXT,0x00000001);
}

int RunMaliTest_adv4_t720_part2 (void) {
  int res = 0;
  int mmu_irq_rawstat = Mali_RdReg(0x20,0, MMU_IRQ_RAWSTAT);
  int mmu_irq_status = Mali_RdReg(0x20,0,MMU_IRQ_STATUS);   /* There should have been a page fault interrupt for address space 0  */
  
  mali_step++;

  /* Check for it in the MMU RAWSTAT register                           */
  if ( (mmu_irq_rawstat & 0x00000001) != 0) 
  {
   printf ("MMU page fault interrupt for address space 0 was set correctly to %x\n", mmu_irq_rawstat);
  }
  else
  {
   printf ("MMU page fault interrupt for address space 0 was NOT set correctly, expected 0x00000001 got %x  mmu_irq_status=%x\n", mmu_irq_rawstat, mmu_irq_status );
   res = 1;
  }
  
  /* mask  interrupts so the test can complete */
  Mali_MaskAllInterrupts();
  return res;
}

int Check_adv4_t720_check_0_a (struct t_adv4_t720_mem_0 *p) {
  struct t_CheckData_adv4_t720_check_0_a *r = &CheckData_adv4_t720_check_0_a;
  int res = 0;
  Mali_Message("Performing data check of 32 bytes\n");

  res |= Mali_MemCmp(p->data_00001000, 0x00001000, r->data_00001000, 0, 0x008);
  /* Return -1 if a compare fail */
  if (res) { return -1; };
  Mali_Message("Check okay \n");
  return 0;
};
