/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011-2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"


int RunMaliTest_adv7_t720_part0 (void);

static volatile struct t_adv3_t720_mem_0 *malidata_0;
static int t_adv3_t720__int_count;

static int mali_step;


//=================================================================
int RunMaliTest_adv7_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) 
  {
    case 0  : return RunMaliTest_adv7_t720_part0();
    default : return 0;
  };
};

//=================================================================
int RunMaliTest_adv7_t720_part0 (void) {
  int res = 0;
  int timeout;
  int flush_complete;
  int gpu_irq;

  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;
  int active, barrier_enabled;

  printf("adv7 : RUNNING TEST: adv7 - part 0 \n");
  printf("adv7 :   Purpose: Test that the AXI4 barrier support has been correctly configured.\n");
  printf("adv7 :            AXI4 Barriers (when enabled) are issued on a cache flush.\n");
  printf("adv7 :            This test checks that a cache flush completes as expected.\n");

  // Get current configuration to allow testing all registers
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);
  
  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("adv7 : Selected configuration:\n");
  printf("adv7 :   Bus width                           : %d\n", axi_width);
  printf("adv7 :   Number of shader cores              : %d\n", num_cores);
  printf("adv7 :   L2 cache size                       : %d kB\n", ((1<<l2_size)/1024));


  Mali_Reset();

  res |= Mali_test_reg(0, 0, 0x000, 0xffff0000, 0xffff0000, GPU_ID_VALUE, 1, "GPU_ID");           
  res |= Mali_test_reg(0, 0, 0x100, 0xffffffff, 0xffffffff, core_bitmap,1, "SHADER_PRESENT_LO");
  res |= Mali_test_reg(0, 0, 0x110, 0xffffffff, 0xffffffff, 0x00000001, 1, "TILER_PRESENT_LO"); 
  if (num_cores >= 6)
  {
    res |= Mali_test_reg(0, 0, 0x120, 0xffffffff, 0xffffffff, 0x00000011, 1, "L2_PRESENT_LO");    
  }
  else
  {
    res |= Mali_test_reg(0, 0, 0x120, 0xffffffff, 0xffffffff, 0x00000001, 1, "L2_PRESENT_LO");    
  }

  res |= Mali_test_reg(0, 0, 0x140, 0xffffffff, 0xffffffff, 0x00000000, 1, "SHADER_READY_LO");  
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 0x00000000, 1, "TILER_READY_LO");   
  res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 0x00000000, 1, "L2_READY_LO");      


  // --------------------------------------------------------
  printf("adv7 : POWERING ON \n");
  for(i=0; i<num_cores; i++) 
  {   
    printf("adv7 : Power ON  Shader Core #%d\n", i);
    Mali_WrReg(0, 0, SHADER_PWRON_LO, (1<<i)); 
    while( Mali_RdReg(0, 0, SHADER_PWRTRANS_LO) ) {}
  }
  
  printf("adv7 : Power ON  Tiler\n", i);
  Mali_WrReg(0, 0, TILER_PWRON_LO, 1); 
  while( Mali_RdReg(0, 0, 0x210) ) {}
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 1, 1, "TILER_READY_LO"); 

  printf("adv7 : Power ON  L2\n", i);
  if (num_cores>=6)
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 17); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 17, 1, "L2_READY_LO");
  }
  else
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 1); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 1, 1, "L2_READY_LO");
  }


  // =======================================================================
  printf("adv7 : L2_MMU_CONFIG = 0x%x\n", Mali_RdReg(0x20,0,L2_MMU_CONFIG));
  barrier_enabled = 0;
  
  printf("adv7 : Flush the caches\n", i);
  GPU_flush();
  
  // The flush should always complete since the AXI bus should not hang!
  res = res | GPU_check_flush_complete_timeout(barrier_enabled);
  
  mali_step++;
  return res;
}

//=================================================================
int GPU_flush()
{
  int gpu_status = Mali_RdReg(0x20,0,GPU_IRQ_STATUS);
  //printf ("adv7 : Starting GPU flush \n");
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("adv7 : GPU_STATUS = 0x%x\n", gpu_status);
  printf ("adv7 : Starting flush operation....\n");
  Mali_WrReg(0, 0, GPU_COMMAND, 8); 
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("adv7 : GPU_STATUS = 0x%x\n", gpu_status);
  return gpu_status; 
}

//=================================================================
int GPU_check_flush_complete_timeout (int barrier_enabled)
{

  int timeout = 100;
  int result  = 0;
  int flush_complete;
  int gpu_irq;
  
  printf("adv7 : Looking for the cache flush complete signal - with a timeout = %d \n", timeout);
  gpu_irq = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  flush_complete = (gpu_irq & ( 1<<17)) == (1<<17);
  
  while ( !flush_complete && (timeout > 0) )
  {
    // Add these for debug 
    //printf ("adv7 : timeout = %d \n", timeout );
    //printf ("adv7 : flushed = %d \n", flush_complete );
    //printf ("adv7 : gpu_irq = 0x%x \n", gpu_irq );
    
    gpu_irq = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
    flush_complete = ((gpu_irq & ( 1<<17)) == (1<<17));
    timeout--;
  }
  
  if (flush_complete)
  {
    printf ("adv7 : SUCCESS : Flush completed as expected. timeout = %d \n", timeout );
  }
  else if (timeout == 0)
  {
    printf ("adv7 : FAILURE : Cache Flush timed out! timeout = %d \n", timeout );
    printf ("adv7 :           AXI Barrier causes hang.\n");
    printf ("adv7 :           If you are on an AXI3 system then tie off BARRIERDISABLE=1.\n");
    printf ("adv7 :           If you are on an AXI4 system that supports barriers there is an issue with integration of barriers.\n");
    result = -2;
  }
  else
  {
    printf ("adv7 : FAILURE : Unexpected error!  flush_complete = %d;  timeout = %d \n", flush_complete, timeout );
    result = -2;
  }  
  
  return result;
}

//=================================================================
/* Mali_test_reg
   Test the register.
   access: 0=RW, 1=RO
*/
int Mali_test_reg(  int unit, 
                    int core, 
                    int regnum, 
                    int read_mask, 
                    int write_mask, 
                    int check_value, 
                    int access,
                    char * reg_name) 
{
  int value = Mali_RdReg(unit, core, regnum);
  if( value != check_value ) 
  {
    printf("adv7 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
           (unit<<28)+(core<<16)+regnum, value, check_value, reg_name);
    return -1;
  }
  if( access == 0 ) 
  {
    Mali_WrReg(unit, core, regnum, (0xffffffff & write_mask));
    value = Mali_RdReg(unit, core, regnum) & read_mask;
    if( value != (0xffffffff & write_mask & read_mask) ) 
    {
      printf("adv7 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             (unit<<28)+(core<<16)+regnum, value, (0xffffffff & write_mask & read_mask), reg_name);
      return -2;
    }
    Mali_WrReg(unit, core, regnum, (0x12345678 & write_mask));
    value = Mali_RdReg(unit, core, regnum) & read_mask;
    if( value != (0x12345678 & write_mask & read_mask) ) 
    {
      printf("adv7 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             (unit<<28)+(core<<16)+regnum, value, (0x12345678 & write_mask & read_mask), reg_name);
      return -2;
    }
  }
  printf("adv7 : Register %08x: Success!  for  %s\n", (unit<<28)+(core<<16)+regnum, reg_name);
  return 0;
}

