//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2015, 2017 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//      Release Information : r0p0-00eac0
//
//-----------------------------------------------------------------------------
//


//#include <stdio.h>
#include "IntHandler.h"
#include <stdint.h>
#include "platform.h"

/******************************************************************************
 * Consts / defines 
 ******************************************************************************/
#define PRIMARY_CORE_ID	0	/* Make it for core#0 only. If needed redefine. */

/******************************************************************************
 * GIC distributor and CPU interface register layouts 
 ******************************************************************************/
typedef volatile unsigned int devreg_t;

typedef struct 
{
    /* 0x000 */ devreg_t ctrl;              /* control */
    /* 0x004 */ const devreg_t typer;       /* controller_type; */
    /* 0x008 */ const devreg_t iidr;        /* Distributor Implementer Identification Register */
    /* 0x00C */ const char padding1[0x74];  /* reserved 'n' IMPLEMENTATION DEFINED */
    /* 0x080 */ const devreg_t igroup[0x10];/* Interrupt Group Registers */
    /* 0x0C0 */ const char padding3[0x40];  /* unknwn */       
    /* 0x100 */ devreg_t isenable[0x20];    /* Interrupt Set-enable Registers */
    /* 0x180 */ devreg_t icenable[0x20];    /* Interrupt Clear-enable Registers */
    /* 0x200 */ devreg_t ispend[0x20];     /* Interrupt Set-pending Registers */
    /* 0x280 */ devreg_t icpend[0x20];     /* Interrupt Clear-pending Registers */
    /* 0x300 */ devreg_t isactive[0x20];    /* GICv2 Interrupt Set-active Registers */
    /* 0x380 */ devreg_t icactive[0x20];    /* Interrupt Clear-active Registers */
    /* 0x400 */ devreg_t ipriority[0x80];   /* Interrupt Priority Registers */
    /* 0x600 */ const char padding4[0x200]; /* reserved */
    /* 0x800 */ devreg_t itargets[0x80];    /* Interrupt Processor Targets Registers */
    /* 0xA00 */ const char padding5[0x200]; /* reserved */
    /* 0xC00 */ devreg_t icfg[0x20];        /* Interrupt Configuration Registers */
    /* 0xC80 */ const char padding6[0x80];  /* unknwn*/ 
    /* 0xD00 */ devreg_t ppisr;             /* Private Peripheral Interrupt Status Register, */
    /* 0xD04 */ devreg_t spisr[0xf];         /* Shared Peripheral Interrupt Status Registers, */
    /* 0xd40 */ const char padding7[0xc0];  
    /* 0xE00 */ devreg_t nsacr[0x40];       /* Non-secure Access Control Registers, optional */    
    /* 0xF00 */ devreg_t sgir;              /* Software Generated Interrupt Register */
    /* 0xF04 */ const char padding8[0xC];   /* Reserved */
    /* 0xF10 */ devreg_t cpendsgir[0x4];    /* SGI Clear-Pending Registers */
    /* 0xF20 */ devreg_t spendsgir[0x4];    /* SGI Set-Pending Registers */
    /* 0xF30 */ const char padding9[0xA0];  /* reserved */
    /* 0xFD0 */ devreg_t pidr4;             /* Peripheral ID 4 Register */
    /* 0xFD4 */ devreg_t pidr5;             /* Peripheral ID 5 Register */
    /* 0xFD8 */ devreg_t pidr6;             /* Peripheral ID 6 Register */
    /* 0xFDC */ devreg_t pidr7;             /* Peripheral ID 7 Register */
    /* 0xFE0 */ devreg_t pidr0;             /* Peripheral ID 0 Register */
    /* 0xFE4 */ devreg_t pidr1;             /* Peripheral ID 1 Register */
    /* 0xFE8 */ devreg_t pidr2;             /* Peripheral ID 2 Register */
    /* 0xFEC */ devreg_t pidr3;             /* Peripheral ID 3 Register */
    /* 0xFF0 */ devreg_t cidr0;             /* Component ID 0 Register */
    /* 0xFF4 */ devreg_t cidr1;             /* Component ID 1 Register */
    /* 0xFF8 */ devreg_t cidr2;             /* Component ID 2 Register */
    /* 0xFFC */ devreg_t cidr3;             /* Component ID 3 Register */

} gic400_id ;

typedef struct 
{
    /* 0x00 */  devreg_t ctrl;          /* CPU Interface Control Register */
    /* 0x04 */  devreg_t pmr;           /* Interrupt Priority Mask Register */
    /* 0x08 */  devreg_t bpr;           /* Binary Point Register */
    /* 0x0c */  const devreg_t iar;     /* Interrupt Acknowledge Register */
    /* 0x10 */  devreg_t eoir;          /* End of Interrupt Register */
    /* 0x14 */  const devreg_t  rpr;    /* Running Priority Register */
    /* 0x18 */  const devreg_t  hppir;  /* Highest Priority Pending Interrupt Register */
    /* 0x1C */  devreg_t abpr;          /* Aliased Binary Point Register*/
    /* 0x20 */  const devreg_t  aiar;   /* Aliased Interrupt Acknowledge Register */
    /* 0x24 */  devreg_t aeoir;         /* Aliased End of Interrupt Register */
    /* 0x28 */  const devreg_t ahppir;  /* AliasedHighest Priority Pending Interrupt Register */
    /* 0x2C */  const char padding1[0x14];  /* reserved */
    /* 0x40 */  const char impldef1[0x90];  /* IMPLEMENTATION DEFINED registers */
    /* 0xD0 */  devreg_t apr[0x4];      /* Active Priorities Registers */
    /* 0xE0 */  devreg_t nsapr[0x4];    /* Non-secuire Active Priorities Registers */
    /* 0xF0 */  const char padding10[0xC];   /* reserved */
    /* 0xFC */  const devreg_t iidr;        /* CPU Interface Identification Register */
    /* 0x100 */ const char padding2[0xf00]; /* ????? */
    /* 0x1000 */const devreg_t dir;     /*Deactivate Interrupt Register*/

} gic400_ci ;

/******************************************************************************
 * GIC Distributor and CPU Interface APIs instantiation 
 ******************************************************************************/
static gic400_id *const gic400d = ( gic400_id *)GIC_INT_DISTR;
static gic400_ci *const gic400c = ( gic400_ci *)GIC_CPU_INTF;

void int_enable(const int irq) 
{

	const int word = irq / 32;
	const int bit = 1 << (irq % 32);

	gic400d->isenable[word] = bit;
}

void int_disable (const int irq)
{
	const int word = irq / 32;
	const int bit = 1 << (irq % 32);

	/* clear enable interrupt */
    gic400d->icenable[word] = bit;
}


void (*wrapped_irq_handler_0)();

void irq_handler(void);


void init_and_install_gic(void) {
    unsigned int i, max_irq;

    /* Disable the whole thing */
    gic400d->ctrl = 0x00000000;

    max_irq = gic400d->typer & 0x1F;
    max_irq = (max_irq + 1) * 32;

    /* Disable all interrupts */
    for (i=0; i < max_irq / 32; i++)
    {
        gic400d->icenable[i] = 0xffffffff;
    }

    /* Clear all interrupts */
    for (i=1; i < max_irq / 32; i++)
    {
        gic400d->icpend[i] = 0xffffffff;
    }

    /* Reset interrupt priorities */
    for (i=0; i < max_irq / 4; i++)
    {
        gic400d->ipriority[i] = 0xa0a0a0a0;
    }


	/* NOTE: According to ARM DDI 0471A, 
		chapter 3.3 Distributor register summary,
		SGIs and PPIs are hardcoded and read-only,
		so operations below makes no effect. */
    /* For 16 SGIs allow all CPUs to be recepients. */
    for (i=0; i < max_irq / 4; i++)
    {
        gic400d->itargets[i] = 0x01010101;
    }

//	/* For 16 PPIIs allow all CPUs to be recepients. */
//    for (i=4; i < 8; i++)
//    {
//        gic400d->itargets[i] = 0xffffffff;
//    }
//
//	/* For the rest SPIs makes scheduler core recepient only. */
//    for (i=8; i < max_irq / 4; i++)
//    {
//		gic400d->itargets[i] = ( ((1 << PRIMARY_CORE_ID) << 24) | 
//								 ((1 << PRIMARY_CORE_ID) << 16) |
//								 ((1 << PRIMARY_CORE_ID) << 8) |
//								 ((1 << PRIMARY_CORE_ID) << 0) );
//
//    }

    /* Set correct interrupt configuration (level sensitive, 1-N) */
    for (i=0; i < max_irq / 16; i++)
    {
        gic400d->icfg[i] = 0x55555555;
    }

    /* Distributor Enable interrupts */
    gic400d->ctrl = 0x00000003;


    /* Interrupt CPU Interface
     Disable interrupts */
    gic400c->ctrl = 0x00000000;

    /* Set priority mask to enable all interrupts. */
    gic400c->pmr = 0xF0;

	/* Set binary point register */
    gic400c->bpr = 0x00000007;

    /* Enable interrupts */
    gic400c->ctrl = 0x00000003;

    // Finally install the GIC as the main interrupt handler
    wrapped_irq_handler_0 = &irq_handler;        
 
}

#define MAX_NO_OF_INTERRUPTS_IMPLEMENTED 224
#define SPURIOUS_INTERRUPT 1023
#define INTERRUPT_MASK 0x000003FF  
#define CPUID_SHIFT    10

static tHandler *IntHandlers[MAX_NO_OF_INTERRUPTS_IMPLEMENTED];

void irq_handler(void) {
    int source, interrupt, raw_interrupt;

    /* service all pending interrupts */
    while (1)
    {
      /* Get the highest priority interrupt */
      raw_interrupt = gic400c->iar;
      source        = raw_interrupt >> CPUID_SHIFT;
      interrupt     = raw_interrupt & INTERRUPT_MASK;

      if (interrupt == SPURIOUS_INTERRUPT) {
        break;
      }

      /* Call the handler function */
      if (IntHandlers[interrupt]) {
		printf ("Calling Interrupt handler for interrupt no:%d\n", interrupt);
        IntHandlers[interrupt](interrupt, source);
      } else {
		printf("No Interrupt Handler found\n");
	  }

      /* Clear the interrupt */
      gic400c->eoir = raw_interrupt;
    }
};


void interrupt_handler_set(int interrupt, tHandler handler)
{
  if (interrupt < 0 || interrupt > MAX_NO_OF_INTERRUPTS_IMPLEMENTED) {
    return;
  }
  printf ("interrupt_handler_set called for interrupt no:%d\n", interrupt);
  IntHandlers[interrupt] = handler;
}
