#ifndef _HDMI_TX_STRUCT_DEF_H_INCLUDED_
#define _HDMI_TX_STRUCT_DEF_H_INCLUDED_

#define VSIF_VIDEO_FORMAT_BIT 5
#define VSIF_3D_FORMAT_BIT 4

typedef struct {
	unsigned int blank;
	unsigned int active;
	unsigned int sync;
	unsigned int front;
	unsigned int back;
} cea_timing_h_param_st;

typedef struct {
	unsigned int space;
	unsigned int blank;
	unsigned int act_video;
	unsigned int active;
	unsigned int sync;
	unsigned int front;
	unsigned int back;
} cea_timing_v_param_st;

typedef struct {
	unsigned int synci;
	unsigned int blanki;
	unsigned int backi;
	unsigned int fronti;
} cea_timing_v_i_param_st;

typedef struct {
	unsigned char  vfch_num;
	unsigned char  fp_swen;
	unsigned char  fp;
	unsigned char  pp_swen;
	unsigned char  swen;
	unsigned char  default_ph;
} deep_color_param_st;

typedef struct {
	unsigned char  dly_nor_h;
	unsigned char  dly_nor_v;
	unsigned char  dly_disp_h;
	unsigned char  dly_disp_v;
} disp_timing_param_st;

typedef struct {
	unsigned int video_code;
	cea_timing_h_param_st h;
	cea_timing_v_param_st v;
	cea_timing_v_i_param_st vi;
} hdmi_tx_timing_gen_st;

typedef struct {
	deep_color_param_st dpc;
	disp_timing_param_st disp;
} hdmi_tx_timing_dpc_disp_st;

typedef struct {
	unsigned int tv_ncode;
	unsigned int tv_fcode;
	unsigned int tv_pre_div;
	unsigned int tmds_div;
	unsigned int pix_div;	
} pll_disp_param_st;

typedef struct {
	unsigned int tx_m1;
	unsigned int tx_m2;
	unsigned int tx_kvco;
	unsigned int mac_div;	
} pll_hdmi_tx_param_st;

typedef struct {
	pll_disp_param_st disp;
	//pll_ddsa_param_st *ddsa
	pll_hdmi_tx_param_st hdmi;
} pll_div_param_st;

typedef struct {	
	unsigned int tv_cp;
	unsigned int tv_cs;
	unsigned int tv_icp;
	unsigned int tv_rs;	
} pll_disp_analog_st;

typedef struct {	
	unsigned int tx_pll_icp;
	unsigned int tx_pll_rs;
	unsigned int tx_pll_cp;
	unsigned int tx_pll_cs;
} pll_hdmi_analog_st;

typedef struct {
	pll_disp_analog_st disp;
	pll_hdmi_analog_st hdmi;
	//pll_others_analog_st others;
} pll_analog_param_st;

typedef struct {
	unsigned int hpos;
	unsigned int hs_st;
	unsigned int hs_end;
	unsigned int hpos_st;
	unsigned int hpos_end;
} tve_timing_h_param_st;

typedef struct {
	unsigned int vpos;
	unsigned int st_pos1v;
	unsigned int st_pos1h;
	unsigned int end_pos1v;
	unsigned int end_pos1h;
	unsigned int st_pos2v;
	unsigned int st_pos2h;
	unsigned int end_pos2v;
	unsigned int end_pos2h;
} tve_timing_v_param_st;

typedef struct {
	tve_timing_h_param_st h;
	tve_timing_v_param_st v;
	deep_color_param_st dpc;
	disp_timing_param_st disp;
	bool interlace;
} hdmi_tx_tve_timing_gen_st;

typedef struct {
	unsigned char v_max;
	unsigned char h_max;
} hdmi_tx_max_packet_num_st;

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	unsigned char rt;
	unsigned char ct;
	bool enlarge_en;
} hdmi_tx_swing_r_st;

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	unsigned char rt;
	unsigned char ct;
	bool enlarge_en;
} hdmi_tx_swing_g_st;

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	unsigned char rt;
	unsigned char ct;
	bool enlarge_en;
} hdmi_tx_swing_b_st;

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	bool ct_en;
	unsigned char ct;
	unsigned char delay_ctr;
} hdmi_tx_swing_clk_st;

typedef struct {
	hdmi_tx_swing_r_st r;
	hdmi_tx_swing_g_st g;
	hdmi_tx_swing_b_st b;
	hdmi_tx_swing_clk_st clk;
} hdmi_tx_swing_st;

#if 0
typedef struct {
	unsigned int ibias;
	unsigned int amp;
	unsigned int driver;
	unsigned int slew;
	unsigned int rt_sel;
	unsigned int emph;
	bool en_emph;	
} hdmi_tx_swing_st;

typedef struct {
	unsigned int ibias;
	unsigned int amp;
	unsigned int driver;
	unsigned int slew;
	unsigned int rt_sel;
	unsigned int emph;
	bool en_emph;
	unsigned int mhl_vcm;
} mhl_tx_swing_st;
#endif

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	unsigned char rt;
	unsigned char ct;
	bool enlarge_en;
} mhl_tx_swing_b_st;

typedef struct {
	bool emph_en;
	unsigned char emph;
	unsigned char driver;
	unsigned char slew;
	bool ct_en;
	unsigned char ct;
	unsigned char delay_ctr;
} mhl_tx_swing_clk_st;

typedef struct {
	mhl_tx_swing_b_st b;
	mhl_tx_swing_clk_st clk;
} mhl_tx_swing_st;

enum hdmi_tx_vsif_video_format {
	HDMI_TX_NO_ADDITIONAL_VIDEO_FORMAT = 0x0,
	HDMI_TX_EXTENDED_RESOLUTION_4K2K_FORMAT = 0x1,
	HDMI_TX_EXTENDED_RESOLUTION_3D_FORMAT = 0x2,
};

enum hdmi_tx_vsif_4k2k_vic {
	HDMI_TX_2160P30_VIC = 0x1,
	HDMI_TX_2160P24_VIC = 0x3,
};

#endif // _HDMI_TX_STRUCT_DEF_H_INCLUDED_
