#include "ifd_driver.h"

#ifndef _RTD_TYPES_H
#include <rtd_types_qc.h>
#endif
//#include "rtk_i2c_lib.h"

void drvif_scaleri2c_tuner_init(void) 
{
	IFD_QC_FUNC();
	I2C_Init();
	return;
}

#ifndef IFD_I2C_DRIVER
extern void I2C_UnInit(void);
#endif

UINT8 drvif_scaleri2c_tuner_write(I2C_MODULE *pI2CModule, UINT8 *pWriteArray) {
	
	UINT8 device_id_local;
	UINT16 addr_local;
	UINT16 data_len=7;
	UINT8 data[16];
	//int i2c_flag=0;

	UINT8 i, j;
	UINT8 retry = 3;
	UINT8 result = 0;
     
tuner_write_retry:
	
	device_id_local = pI2CModule->device_id>>1;

	while(retry) {
		retry --;

		//init address
		if((pI2CModule->addrtype==0)||(pI2CModule->addrtype==3))
			addr_local = 0;//No Address
		else
			addr_local = pI2CModule->slave_addr;

		for (i = 0; i<pI2CModule->len;)
		{
			#if !defined(CONFIG_I2C_PRE_COMMAND_ENABLE)
			ROS_ENTER_CRITICAL();
			#endif

			if(pI2CModule->addrtype==0)
			{
				if((pI2CModule->len-i)>7)
					data_len = 7;
				else
					data_len = (pI2CModule->len-i);

				for (j=0 ; j<data_len; j++) 
				{
					data[j]=pWriteArray[i+j];
				}
				result = I2C_Write_EX(pI2CModule->master_id, device_id_local, data_len, data/*, i2c_flag*/);
			}
			else if(pI2CModule->addrtype==1)
			{
				if((pI2CModule->len-i)>6)
					data_len = 6;
				else
					data_len = (pI2CModule->len-i);

				data[0]=addr_local&0xff;
				for (j=0 ; j<data_len; j++) data[j+1]=pWriteArray[i+j];
				//for (j=0 ; j<data_len; j++) IFD_DEBUG_MSG(IFD_MSG_TUNER, "@@@I2C write@@@data[%x]=%x\n", j+data[0], data[j+1]);
				udelay(10000);
				result = I2C_Write_EX(pI2CModule->master_id, device_id_local, 1+data_len, data/*, i2c_flag*/);
			}
			else if(pI2CModule->addrtype==2){
				if((pI2CModule->len-i)>5)
					data_len = 5;
				else
					data_len = (pI2CModule->len-i);

				data[0]=(addr_local>>8)&0xff;
				data[1]=addr_local&0xff;
				for (j=0 ; j<data_len; j++) data[j+2]=pWriteArray[i+j];
				result = I2C_Write_EX(pI2CModule->master_id, device_id_local, 2+data_len, data/*, i2c_flag*/);
			}
			else if(pI2CModule->addrtype==3)
			{
				if((pI2CModule->len-i)>9)
					data_len = 9;
				else
					data_len = (pI2CModule->len-i);

				for (j=0 ; j<data_len; j++) data[j]=pWriteArray[i+j];
				result = I2C_Write_EX(pI2CModule->master_id, device_id_local, data_len, data/*, i2c_flag*/);
				udelay(10000);
			}

			#if !defined(CONFIG_I2C_PRE_COMMAND_ENABLE)
			ROS_EXIT_CRITICAL();
			#endif
			if (result == 1) {
					IFD_DEBUG_MSG(IFD_MSG_TUNER, "tuner write error : I2C error\n");
					#ifndef IFD_I2C_DRIVER
					I2C_UnInit();
					#endif
					goto tuner_write_retry;
			}

			 i+=data_len;
			addr_local+=data_len;
		}

		return _TRUE;
	}

	return _FALSE;

}


UINT8 drvif_scaleri2c_tuner_read(I2C_MODULE *pI2CModule, UINT8 *pReadArray) {
	
	UINT8 device_id_local;
	UINT16 addressbyte_num;
	UINT16 address_local;
	UINT8 address_byte[2];
	UINT16 databyte_num;
	//int i2c_flag=0;

	UINT8 i, result;
	UINT8 retry = 0;

	//UINT8 j;

	device_id_local = pI2CModule->device_id>>1;
	address_local = pI2CModule->slave_addr;

	for (i = 0; i<pI2CModule->len;) {

		if(pI2CModule->addrtype==0)
		{
			databyte_num = ((pI2CModule->len - i) < 8) ? (pI2CModule->len - i) : 8;
			address_byte[0] = 0;
			address_byte[1] = 0;
		}
		else if(pI2CModule->addrtype==1)
		{
			databyte_num = ((pI2CModule->len - i) < 8) ? (pI2CModule->len - i) : 8;
			address_byte[0] = address_local & 0xFF;
			address_byte[1] = 0;
		}
		else if(pI2CModule->addrtype==3)
		{
			databyte_num = ((pI2CModule->len - i) < 20) ? (pI2CModule->len - i) : 20;
			address_byte[0] = address_local & 0xFF;
			address_byte[1] = 0;
		}
		else
		{
			databyte_num = ((pI2CModule->len - i) < 8) ? (pI2CModule->len - i) : 8;
			address_byte[0] = address_local >> 8;
			address_byte[1] = address_local & 0xFF;
		}
		if(pI2CModule->addrtype<3)
			addressbyte_num = pI2CModule->addrtype;
		else
			addressbyte_num = 0;
		retry = 0;
		do {
			#if !defined(CONFIG_I2C_PRE_COMMAND_ENABLE)
			ROS_ENTER_CRITICAL();
			#endif
			
			//IFD_DEBUG_MSG(IFD_MSG_TUNER, "addressbyte_num=%d, address_byte=%x, databyte_num=%d,\n", addressbyte_num, address_byte[0], databyte_num);
			result = I2C_Read_EX(pI2CModule->master_id,device_id_local, addressbyte_num, &address_byte[0],databyte_num, &pReadArray[address_local-pI2CModule->slave_addr]/*, i2c_flag*/);//by Lee, 0715
			//for (j=0 ; j<databyte_num; j++) IFD_DEBUG_MSG(IFD_MSG_TUNER, "@@@I2C read@@@data[%x]=%x\n", j+address_byte[0], pReadArray[j+i]);//by LEE
			udelay(10000);
			if(result ==1) {
   				IFD_DEBUG_MSG(IFD_MSG_TUNER, "tuner read error : I2C error-->Retry %d\n", retry);
				I2C_UnInit();
			}
			#if !defined(CONFIG_I2C_PRE_COMMAND_ENABLE)
			ROS_EXIT_CRITICAL();
			#endif
			retry++;
		}while((result== 1) && (retry<= 3));

		if (retry>=4)
		{
			IFD_DEBUG_MSG(IFD_MSG_TUNER, "tuner read error\n");
			return _FALSE;
		}
		address_local+=databyte_num;
		i+=databyte_num;
	}

	return _TRUE;
}

