//------------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2013-2018 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORINTH-MP090-r4p0-00rel0
//
//------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// Description:
//
//   Data is loaded from the data pool after cleaning and invalidating the
//   caches to force a cache miss.
//
//   This file contains the function that implements the test code plus the
//   literal data pools that the test code uses.
//-------------------------------------------------------------------------------

#define REG_UART0_BASE          0x18062300
// Enable UAL syntax
.syntax unified

                .section .text, "ax", %progbits
                .code 32
                .eabi_attribute Tag_ABI_align_preserved, 1

.equ ITERATIONS, 20

//------------------------------------------------------------------------------
// PMU events
//------------------------------------------------------------------------------

.equ PMU_DCACHE_MISS_CNTR,       0x0  // Event counter to use
.equ PMU_EVENT_L1D_CACHE_REFILL, 0x3  // Event type for this event


//-------------------------------------------------------------------------------
// Description:
//
//   Data is loaded from the data pool after cleaning and invalidating the
//   caches to force a cache miss.
//
//-------------------------------------------------------------------------------

.global Cache_miss_test
.type   Cache_miss_test, %function

Cache_miss_test:
                //-----------------------------------------
                // Preamble
                //-----------------------------------------

                // This function only uses registers that can be used as
                // temporary registers in the AArch32 procedure call standard.
                // Therefore no registers need to be stored on the stack.
                // The link register does not need to be saved because this
                // is a leaf function.

                mov     r12, #ITERATIONS

                //-----------------------------------------
                // Set up performance monitors
                //-----------------------------------------

                // Permit PMU counting at EL3
                mrc     p15, 0, r0, c1, c3, 1   // Read SDCR
                orr     r0, r0, #1<<17          // Set SPME
                mcr     p15, 0, r0, c1, c3, 1   // Write SDCR

                // Clear all counters
                mrc     p15, 0, r0, c9, c12, 0  // Read PMCR
                bic     r0, r0, #0xF            // Clear PMCR.{D,C,P,E}
                mcr     p15, 0, r0, c9, c12, 0  // Write PMCR
                isb

                // Set up counter for D cache miss
                mov     r1, #PMU_DCACHE_MISS_CNTR
                mcr     p15, 0, r1, c9, c12, 5  // Write PMSELR
                mov     r1, #PMU_EVENT_L1D_CACHE_REFILL
                mcr     p15, 0, r1, c9, c13, 1  // Write PM0EVTYPER

                // Clear overflow status and enable
                ldr     r1, =0x80000001
                mcr     p15, 0, r1, c9, c12, 3  // Write PMOVSR
                mcr     p15, 0, r1, c9, c12, 1  // Write PMCNTENSET

                // Reset and enable counters
                orr     r0, r0, #0x7            // Set PMCR.{C,P,E}
                mcr     p15, 0, r0, c9, c12, 0  // Write PMCR
                isb


                //-----------------------------------------
                // Main test code
                //-----------------------------------------

                // Init memory position for second load
                // in the loop to always miss. The load
                // will access peripheral memory (non-cacheable)
//                movw    r0, #:lower16:data_pool1
//                movt    r0, #:upper16:data_pool1
                adrl    r0, data_pool1
                movw    r1, #0x0400
                movt    r1, #0x1300

                str     r1, [r0]
                dsb     sy

//loop:           movw    r0, #:lower16:data_pool1
//                movt    r0, #:upper16:data_pool1
//                movw    r1, #:lower16:data_pool2  // this address should be already invalid
//                movt    r1, #:upper16:data_pool2
//                movw    r2, #:lower16:data_pool3
//                movt    r2, #:upper16:data_pool3
cache_loop:           adrl    r0, data_pool1
                adrl    r1, data_pool2  // this address should be already invalid
                adrl    r2, data_pool3

                dsb     sy
                mcr     p15, 0, r0, c7, c14, 1  // DCCIMVAC
                mcr     p15, 0, r1, c7, c14, 1
                mcr     p15, 0, r2, c7, c14, 1
                dsb     sy

                ldr     r1, [r0]
                ldr     r4, [r1]
                ldr     r3, [r2]

                subs    r12, r12, #0x1
                bne     cache_loop


                //-----------------------------------------
                // Loops complete - read event counters
                //-----------------------------------------

                // Ensure all earlier loads have completed
                dsb     sy
                isb

                // Disable counters
                mrc     p15, 0, r0, c9, c12, 0  // Read PMCR
                bic     r0, r0, #0x1
                mcr     p15, 0, r0, c9, c12, 0  // Write PMCR

                // Read event counter into r0 to form the return value
                mov     r0, #PMU_DCACHE_MISS_CNTR
                mcr     p15, 0, r0, c9, c12, 5  // Write PMSELR
                mrc     p15, 0, r0, c9, c13, 2  // Read PMXEVCNTR into r0

                cmp     r0, #(ITERATIONS*2)     // Non-cacheable load is not counted
                ldreq   r1, _pass_msg
                ldrne   r1, _fail_msg
                // Print test end message
                ldr     r0, UART0_BASE
//                mov     r5, #'A'
//                strb    r5, [r0]
print_loop:     ldrb    r2, [r1], #1
                cmp     r2, #0
                beq     print_end
                strb    r2, [r0]
                b       print_loop

print_end:      mov     r2, #0x04
                strb    r2, [r0]
                dsb     sy
                mov     pc, lr
//                pop	{r4, r5-r10,pc}

//end:            mov     r2, #0x04       // EOT character
//                strb    r2, [r0]
//                dsb     sy
end_wfi:        wfi
                b       end_wfi


//-------------------------------------------------------------------------------
// Data pool
//-------------------------------------------------------------------------------

//                .section .data, "aw", %progbits

// The messages to print.
                .balign 4
UART0_BASE:     .word 0x1801B100

_pass_msg:      .word pass_message
_fail_msg:      .word fail_message

                .balign 4
pass_message:   .asciz "\n ** TEST PASSED **\t\n"

                .balign 4
fail_message:   .asciz "\n ** TEST FAILED **\t\n"

                .balign 1024

data_pool1:     .word data_pool2
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555
                .word 0xAAAAAAAA
                .word 0x55555555

                .balign 4096

data_pool2:     .word data_pool3
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000
                .word 0xFFFFFFFF
                .word 0x00000000

                .balign 4096

data_pool3:     .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA
                .word 0xDEADCAFE
                .word 0xEAEAEAEA

                .end

