//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2013-2014 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
//      SVN Information
//
//      Checked In          : $Date: 2014-07-25 10:06:39 +0100 (Fri, 25 Jul 2014) $
//
//      Revision            : $Revision: 285790 $
//
//      Release Information : CORTEXA53-r0p4-00rel0
//
//-------------------------------------------------------------------------------
// Description:
//
//   Maximum power test
//-------------------------------------------------------------------------------
/* If Porting new IC, remember to verify the following #define */
#define REG_UART0_BASE			0x18062300
#define SCPU_CORE_status_ananke_0_reg   0x1805B83C
#define NUM_OF_SECONDARY_CORES      4       //Merlin6 has A55 penta cores.

// Enable UAL syntax
.syntax unified

                .section testcode, "ax", %progbits

                .global test_start


//-------------------------------------------------------------------------------
// FIQ handler
//
//   This test uses a FIQ to release CPUs from WFI.  The FIQ handler simply
//   clears the FIQ using the trickbox and returns.
//-------------------------------------------------------------------------------

// Make fiq_handler visible to linker so that it can override the default handler
.global fiq_handler

fiq_handler:
                mrc     p15, 0, r8, c0, c0, 5   // Read MPIDR
                tst     r8, #0xFF00
                bne     fiq_loop

                ldr     r8, =0xFF3FFF00         // Trickbox base
                str     r8, [r8, #0x10]         // Clear FIQ (nb. write value ignored)

                // Poll the ISR register until the FIQ has cleared
fiq_loop:       mrc     p15, 0, r9, c12, c1, 0  // r9 == ISR
                tst     r9, #1<<6
                bne     fiq_loop

                // Return using ERET.  This test uses WFI spin code (a WFI
                // instruction followed by a branch back to the WFI) to force
                // the CPUs to remain idle until the test breaks them out of the
                // WFI spin.  For FIQ exceptions the LR has an offset of +4
                // applied, which means that this return will take us to the
                // instruction following the WFI spin branch and hence break us
                // out of the WFI spin.
                eret

                .balign 8
_slock:           .word   0
//-------------------------------------------------------------------------------
// Main test code
// input args:
// r0: the iteration count
// r1: flag, bit map of enum {SYNC_OF_ALL_CORE, TBD } 
//-------------------------------------------------------------------------------
.global _maxpower
_maxpower:
		push {r0-r12, lr}
		mov r11, r0
		mov r14, r1
	
_init_maxpower:
                // Initialise the register bank
                mov     r0,  #0
                mov     r1,  #0
                mov     r2,  #0
                mov     r3,  #0
                mov     r4,  #0
                mov     r5,  #0
                mov     r6,  #0
                mov     r7,  #0
                mov     r8,  #0
                mov     r9,  #0
                mov     r10, #0


                // Enable NEON and initialize the register bank if NEON
                // is present.  Test this by attempting to enable CP10 and
                // CP11 in the CPACR and read back the value to see if
                // bit[20] is set.
                mov     r0, #(0xFF << 20)
                mcr     p15, 0, r0, c1, c0, 2   // Write CPACR
                mrc     p15, 0, r0, c1, c0, 2   // Read CPACR
                ubfx    r0, r0, #20, #1         // Get bit[20]
                cmp     r0, #1
                bne     dacr_setup               // Skip if no NEON

                mrc     p15, 0, r1, c1, c0, 2
                orr     r1, r1, #(0xF << 20)
                mcr     p15, 0, r1, c1, c0, 2
                mov     r1, #(0x1 << 30)
                isb
                vmsr    fpexc, r1

                mov             r1, #0
                mov             r2, #0
                vmov.f64        d0, r1, r2
                vmov.f64        d1, d0
                vmov.f64        d2, d0
                vmov.f64        d3, d0
                vmov.f64        d4, d0
                vmov.f64        d5, d0
                vmov.f64        d6, d0
                vmov.f64        d7, d0
                vmov.f64        d8, d0
                vmov.f64        d9, d0
                vmov.f64        d10, d0
                vmov.f64        d11, d0
                vmov.f64        d12, d0
                vmov.f64        d13, d0
                vmov.f64        d14, d0
                vmov.f64        d15, d0
                vmov.f64        d16, d0
                vmov.f64        d17, d0
                vmov.f64        d18, d0
                vmov.f64        d19, d0
                vmov.f64        d20, d0
                vmov.f64        d21, d0
                vmov.f64        d22, d0
                vmov.f64        d23, d0
                vmov.f64        d24, d0
                vmov.f64        d25, d0
                vmov.f64        d26, d0
                vmov.f64        d27, d0
                vmov.f64        d28, d0
                vmov.f64        d29, d0
                vmov.f64        d30, d0
                vmov.f64        d31, d0

dacr_setup:	
                // Set all domains as manager
                ldr     r1, =0xFFFFFFFF
                mcr     p15, 0, r1, c3, c0, 0   // DACR

                // Enable caches and MMU
                mrc     p15, 0, r1, c1, c0, 0   // SCTLR
                orr     r1, r1, #(0x1 << 2)     // C bit (data cache)
                orr     r1, r1, #(0x1 << 12)    // I bit (instruction cache)
                orr     r1, r1, #0x1            // M bit (MMU)
                mcr     p15, 0, r1, c1, c0, 0

                // Ensure all writes to system registers have taken place
                dsb
                isb
	
                mrc p15, 0, r12, c1, c0, 0  //Read SCTLR into Rt
                bic  r12, r12, #2 // disable alignment check
                mcr p15, 0, r12, c1, c0, 0  //Write Rt to SCTLR
                mrc     p15, 0, r12, c0, c0, 5  // Read MPIDR
                ands    r12, r12, #0xFF00         // r12 == CPU number
                bne     setup

setup:

                // Mask FIQ interrupts, so that the interrupt will wake the core
                // from WFI, but not cause an exception
                cpsid   f

                // Enable invasive debug by setting the DBGDSCR.HDBGen bit
                // prior to enabling watchpoints and breakpoints.  Enabling
                // watchpoints and breakpoints increases power consumption
                // because the comparators have to compare addresses against
                // the watchpoints/breakpoint registers.
                mrc     p14, 0, r0, c0, c2, 2   // r0 == DBGDSCR
                orr     r0, r0, #(0x1 << 14)    // Set HDBGen
                mcr     p14, 0, r0, c0, c2, 2   // Write DBGDSCR

                // Clear the OS lock to enable programming of the breakpoint
                // and watchpoint registers (writing any value other than
                // 0xC5ACCE55 clears the lock.)
                mov     r0, #0
                mcr     p14, 0, r0, c1, c0, 4   // Write DBGOSLAR

                // Set breakpoint and watchpoint value registers
                ldr     r0, =bkpt0
                mcr     p14, 0, r0, c0, c0, 4   // Write DBGBVR0
                ldr     r0, =bkpt1
                mcr     p14, 0, r0, c0, c1, 4   // Write DBGBVR1
                ldr     r0, =bkpt2
                mcr     p14, 0, r0, c0, c2, 4   // Write DBGBVR2
                ldr     r0, =bkpt3
                mcr     p14, 0, r0, c0, c3, 4   // Write DBGBVR3
                ldr     r0, =bkpt4
                mcr     p14, 0, r0, c0, c4, 4   // Write DBGBVR4
                ldr     r0, =bkpt5
                mcr     p14, 0, r0, c0, c5, 4   // Write DBGBVR5
                ldr     r0, =wpt0
                mcr     p14, 0, r0, c0, c0, 6   // Write DBGWVR0
                ldr     r0, =wpt1
                mcr     p14, 0, r0, c0, c1, 6   // Write DBGWVR1
                ldr     r0, =wpt2
                mcr     p14, 0, r0, c0, c2, 6   // Write DBGWVR2
                ldr     r0, =wpt3
                mcr     p14, 0, r0, c0, c3, 6   // Write DBGWVR3

                // Set breakpoint and watchpoint enable registers to enable
                // in all states.  The value written to the DBGBCRn registers
                // is:
                //   Bit[13]   (HMC) - 1'b1    - State matching (match all states)
                //   Bits[8:5] (BAS) - 4'b1111 - Match A32/A64 instructions
                //   Bits[2:1] (PMC) - 2'b11   - State matching (match all states)
                //   Bit[0]    (E)   - 1'b1    - Enable this breakpoint
                ldr     r0, =0x21E7             // DBGBCRn value
                mcr     p14, 0, r0, c0, c0, 5   // Write DBGBCR0
                mcr     p14, 0, r0, c0, c1, 5   // Write DBGBCR1
                mcr     p14, 0, r0, c0, c2, 5   // Write DBGBCR2
                mcr     p14, 0, r0, c0, c3, 5   // Write DBGBCR3
                mcr     p14, 0, r0, c0, c4, 5   // Write DBGBCR4
                mcr     p14, 0, r0, c0, c5, 5   // Write DBGBCR5

                // The value written to the DBGWCRn registers is:
                //   Bit[13]    (HMC) - 1'b1  - State matching (match all states)
                //   Bits[12:5] (BAS) - 8'hFF - Watch all byte lanes
                //   Bits[2:1]  (PAC) - 2'b11 - State matching (match all states)
                //   Bit[0]     (E)   - 1'b1  - Enable this watchpoint
                ldr     r0, =0x3FE7             // DBGWCRn value
                mcr     p14, 0, r0, c0, c0, 7   // Write DBGWCR0
                mcr     p14, 0, r0, c0, c1, 7   // Write DBGWCR1
                mcr     p14, 0, r0, c0, c2, 7   // Write DBGWCR2
                mcr     p14, 0, r0, c0, c3, 7   // Write DBGWCR3

                // Set initial register values
                ldr     r0, =0x00000000
                ldr     r1, =0xFFFFFFFD
                ldr     r2, =0xFFFFFFFC
                ldr     r3, =0x00000006
                ldr     r4, =0x00000000
                ldr     r5, =0xFFFFFFFD
                ldr     r6, =0xFFFFFFFC
                ldr     r7, =0x00000006

                vmov.i8 q1, #0x66  // q1 = 0x66666666
                vmov.i8 q2, #0xAA  // q2 = 0xAAAAAAAA
                vmov.i8 q3, #0x66  // q3 = 0x66666666
                vmov.i8 q4, #0xAA  // q4 = 0xAAAAAAAA
                vmov.i8 q5, #0x66  // q5 = 0x66666666
                vmov.i8 q6, #0xAA  // q6 = 0xAAAAAAAA
                vmov.i8 q7, #0x66  // q7 = 0x66666666
                vmov.i8 q8, #0xAA  // q8 = 0xAAAAAAAA
                vmov.i8 q9, #0x66  // q9 = 0x66666666

                // Now CPU0 programs the trickbox to schedule another FIQ and
                // all CPUs enter WFI.  The CPUs will then all wake and execute
                // the power loop at the same time.
                cmp     r12, #0
                bne     finish_setup
finish_setup:
                // Finish setting up registers before entering WFI
                mov     r9,  #0x007
                mov     r10, #0x005
                mov     r12, #0x001

                b       outer_loop

//-------------------------------------------------------------------------------
// Main power loop
//
// The intent is for the add/sub instructions to run along behind and recreate
// r0, r2 and r4 using the shifter to create the +0x3 or -0xA
//-------------------------------------------------------------------------------

                // Align the loop to an 8-byte boundary so that two instructions
                // will be in the same instruction cache data bank.  This allows
                // us to dual-issue all pairs of instructions in the loop for
                // maximum power in the pipeline.
                .balign 8
outer_loop:
		//preload cache line, we want the burst off wfe.
		pli restore_set
		pli power_loop

		tst	r14, #1    //test if need to sync with other core
                moveq     r9,  #0x007
                moveq    r10, #0x005
                moveq     r12, #0x001
		beq	power_loop

	
                mrc     p15, 0, r12, c0, c0, 5  // Read MPIDR
                ands     r12, r12, #0xFF00      // r12 == CPU number, check value of Affinity level 1
                                                // Merlin6 consists of A55 ARM cores.
		beq 	core0_wait


		// write protect with lock of ldrex/strex
		ldr 	r9,  =_slock 
1:		ldrex	r10, [r9] 
		add 	r10, r10, #1
		strex	r12, r10, [r9]
		teq	r12, #0
		bne	1b

		//unlock
		clrex
	
stop_123:	wfe // wait for core 0 , event 2
		ldrex r10, [r9]
		cmp r10, #0
		bne stop_123

		b core123_ready

core0_wait:	
		//ldr r9, =0x1805B83C
		ldr r9, =SCPU_CORE_status_ananke_0_reg
test_wfe_core4:	ldr r10, [r9]  //core4 for Merlin6
		ubfx r10, r10, #8, #1
		teq r10, #0
		bne test_wfe_core4

		ldr r9, =SCPU_CORE_status_ananke_0_reg
test_wfe_core123:	ldr r10, [r9]  //core1-3 for Merlin6
		ubfx r10, r10, #1, #3
		teq r10, #0
		bne test_wfe_core123

		// write protect with lock by ldrex/strex
		ldr 	r9,  =_slock 
1:		ldrex	r10, [r9]
		cmp	r10, #NUM_OF_SECONDARY_CORES     // Number of secondary cores
		bne	1b
		mov	r12, #0
		strex	r10, r12, [r9]
		teq	r10, #0
		bne	1b

		//unlock
		clrex

		// release core 1,2,3,4 prepare to enter power loop.
		sev
		dsb
		isb

	
core123_ready:
setup_sync_event:	
		mov r12, #(1<<2)
		MRRC p15, 0, r9, r10, c14
		and r9, r9, #1
		orr r12, r12, r9, lsl #3
		MCR p15, 0, r12, c14, c1, 0
	
restore_set:
                mov     r9,  #0x007
                mov     r10, #0x005
                mov     r12, #0x001

sync_core0123_event:
		dsb
	        wfe  // wait for core 0 , event 2


        
power_loop:
                // Group 1
                vfma.f32  q1, q2, q3           // Dual Issue Pair 0, Instruction 0
                ldr       r8, [r0,+r1,LSL #1]! // Dual Issue Pair 0, Instruction 1

                ldr       r8, [r2,+r3]!        // Dual Issue Pair 1, Instruction 0
                add       r0, r1, r9, LSR r12  // Dual Issue Pair 1, Instruction 1

                sub       r2, r3, r10, LSL r12 // Dual Issue Pair 2, Instruction 0
                ldr       r8, [r4,+r5,LSL #1]! // Dual Issue Pair 2, Instruction 1

                ldr       r8, [r6,+r7]         // Dual Issue Pair 3, Instruction 0
                add       r4, r5, r9, LSR r12  // Dual Issue Pair 3, Instruction 1

                // Group 2
                vfma.f32  q4, q5, q6           // Dual Issue Pair 0, Instruction 0
                ldr       r8, [r0,+r1,LSL #1]! // Dual Issue Pair 0, Instruction 1

                ldr       r8, [r2,+r3]!        // Dual Issue Pair 1, Instruction 0
                add       r0, r1, r9, LSR r12  // Dual Issue Pair 1, Instruction 1

                sub       r2, r3, r10, LSL r12 // Dual Issue Pair 2, Instruction 0
                ldr       r8, [r4,+r5,LSL #1]! // Dual Issue Pair 2, Instruction 1

                ldr       r8, [r6,+r7]         // Dual Issue Pair 3, Instruction 0
                add       r4, r5, r9, LSR r12  // Dual Issue Pair 3, Instruction 1

                // Group 3
                vfma.f32  q7, q8, q9           // Dual Issue Pair 0, Instruction 0
                ldr       r8, [r0,+r1,LSL #1]! // Dual Issue Pair 0, Instruction 1

                ldr       r8, [r2,+r3]!        // Dual Issue Pair 1, Instruction 0
                add       r0, r1, r9, LSR r12  // Dual Issue Pair 1, Instruction 1

                sub       r2, r3, r10, LSL r12 // Dual Issue Pair 2, Instruction 0
                ldr       r8, [r4,+r5,LSL #1]! // Dual Issue Pair 2, Instruction 1

                ldr       r8, [r6,+r7]         // Dual Issue Pair 3, Instruction 0
                add       r4, r5, r9, LSR r12  // Dual Issue Pair 3, Instruction 1

                // Group 4
                vfma.f32  q4, q5, q6           // Dual Issue Pair 0, Instruction 0
                ldr       r8, [r0,+r1,LSL #1]! // Dual Issue Pair 0, Instruction 1

                ldr       r8, [r2,+r3]!        // Dual Issue Pair 1, Instruction 0
                add       r0, r1, r9, LSR r12  // Dual Issue Pair 1, Instruction 1

                sub       r2, r3, r10, LSL r12 // Dual Issue Pair 2, Instruction 0
                ldr       r8, [r4,+r5,LSL #1]! // Dual Issue Pair 2, Instruction 1

                ldr       r8, [r6,+r7]         // Dual Issue Pair 3, Instruction 0
                add       r4, r5, r9, LSR r12  // Dual Issue Pair 3, Instruction 1

                // End of loop check
                subs    r11, r11, #1
                bne     power_loop
#if 0
                // Outer loop check
                // Poll the ISR register until the FIQ has cleared
fiq_loop2:      mrc     p15, 0, r8, c12, c1, 0  // r8 == ISR
                tst     r8, #1<<6
                bne     fiq_loop2

                ldr     r8, =0xFF3FFF00         // Trickbox base
                mov     r11, #0x400             // FIQ schedule value
                .ifdef BIGEND
                rev     r11, r11
                .endif
                str     r11, [r8, #0x8]         // Store to FIQ counter

                subs    r14, r14, #1
                bne     outer_loop
#endif
//-------------------------------------------------------------------------------
// End of test
//-------------------------------------------------------------------------------

                // All CPUs except CPU0 enter WFI, while CPU0 ends the test
#if 0
                mrc     p15, 0, r0, c0, c0, 5   // Read MPIDR
                ands    r0, r0, #0xFF           // r0 == CPU number
                bne     end_wfi
                ldr     r0, =pass_msg
                bl      print
#endif
                mov r9,#0x00000000      // Switch on the VFP and NEON hardware
                vmsr FPEXC,r9           // Set EN bit in FPEXC
                
end:
                mrc p15, 0, r12, c1, c0, 0  //Read SCTLR into Rt
                and r12, r12, #0xfffffffD  // enable alignment check
                mcr p15, 0, r12, c1, c0, 0  //Write Rt to SCTLR
                //b    end
                //bx      lr                      // Return on NULL character

		//disable timer event
		MRC p15, 0, r12, c14, c1, 0
		bic r12, #(1<<2)
		MCR p15, 0, r12, c14, c1, 0

		pop {r0-r12, pc}

end_wfi:        wfe
                b       end_wfi

//-------------------------------------------------------------------------------
// Subroutines
//-------------------------------------------------------------------------------

                // Routine to print a string at the address in r0.
.type print, %function
print:         ldr     r1, =0x18062300         // Tube address
               // ldr     r0, =0xFF3FFF00 // Tube address
                ldrb    r2, [r0], #0x1
                cmp     r2, #0x0
                strbne  r2, [r1]
                bne     print
                mov     r2, #'\n'
                strb    r2, [r1]
                bx      lr                      // Return on NULL character


//-------------------------------------------------------------------------------
// Breakpoint and watchpoint labels
//-------------------------------------------------------------------------------

// These labels are used to set the breakpoint and watchpoint registers.
// There is no real code at these labels and they are never executed: the
// breakpoints and watchpoints are set purely to increase power consumption.
bkpt0:          .word   0
bkpt1:          .word   0
bkpt2:          .word   0
bkpt3:          .word   0
bkpt4:          .word   0
bkpt5:          .word   0

// Align watchpoints to double-word boundaries so that all byte address select
// bits can be set in the watchpoint control register
                .balign 8
wpt0:           .word   0
                .word   0
wpt1:           .word   0
                .word   0
wpt2:           .word   0
                .word   0
wpt3:           .word   0
                .word   0

//-------------------------------------------------------------------------------
// Strings
//-------------------------------------------------------------------------------
                .balign 4
pass_msg:       .asciz "** TEST PASSED OK **\n"

                .end

