#include <qc_verify_common.h>
#include <qc_verify.h>
#include <qc_verify_def.h>
#if defined(CONFIG_SYS_RTK_NAND_FLASH)
 #include <linux/mtd/nand.h>
 #include <nand.h>
#endif

#define VE_QC_GO_ADDRESS  0x01200000 //0x00108000
#define EMMC_TMP_BUFFER   0x01200000
#define EMMC_BLOCK_SIZE             (512UL)
#define EMMC_BLOCK_SIZE_SHIFT       (9)
#define EMMC_QC_VE_ENCODE_PATTERN_ADDR		0x04200000  										// offset 66MB
#define EMMC_QC_VE_ENCODE_PATTERN_BLK_ADDR	(EMMC_QC_VE_ENCODE_PATTERN_ADDR>>EMMC_BLOCK_SIZE_SHIFT)		//
#define NAND_QC_VE_ENCODE_BLK_ADDR		456
#define BLOCK_QC_VE_ENCODE			0x92
#define NAND_TMP_BUFFER				0x01200000

static unsigned int align_to_boundary(unsigned int size, unsigned int block_size)
{
	return (size % block_size ? size / block_size + 1 : size / block_size);
}

int emmc_get_qc_ve_encode_buf(unsigned int ve_buf)
{
	int ret = 0;
	char cmd[256] = {0};
	unsigned int *qc_ptr;
	unsigned int data_size;
	unsigned int block_no,block_cnt;

	/* read qc_ve_encode_pattern header */
	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", EMMC_TMP_BUFFER, EMMC_QC_VE_ENCODE_PATTERN_BLK_ADDR, 1);
	printf("run cmd => %s\n", cmd);
	if ((ret = run_command(cmd, 0))) {
		printf("command error\n");
		ret = -1;
		return ret;
	}
	qc_ptr = (unsigned int *)EMMC_TMP_BUFFER;
	if(*qc_ptr++ != 0x12345678)
	{
		printf("error magic no.\nno qc_ve_encode_pattern on flash\n");
		ret = -1;
		return ret;
	}

	data_size = *qc_ptr;
	block_cnt = align_to_boundary(data_size,EMMC_BLOCK_SIZE);
	block_no = EMMC_QC_VE_ENCODE_PATTERN_BLK_ADDR + 1; // body = header +1
	printf("qc_ve_encode_pattern body: block_no = 0x%x, block_cnt = 0x%x", block_no, block_cnt);

	/* read qc_ve_encode_pattern body */
	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", ve_buf, block_no, block_cnt);
	if ((ret = run_command(cmd, 0))) {
		printf("command error\n");
		ret = -1;
	}
	return ret;
}

int get_nand_qc_ve_encode_buf(unsigned int qc_buf)
{
	int ret = 0;
#ifdef CONFIG_SYS_RTK_NAND_FLASH
	int blk_addr = 0,hdr_size = 0;
	unsigned int start_addr = 0,*qc_ptr = NULL,buf_size = 0;
	nand_info_t *nand =&nand_info[0]; 
	/*read header content*/
	blk_addr = nand->scan_block_tag(nand, NAND_QC_VE_ENCODE_BLK_ADDR,20,0,BLOCK_QC_VE_ENCODE);
	if(blk_addr == 0){
		printf("[ERR] %s %d: Scan block-tag failed!!!\n", __FUNCTION__,__LINE__);
		return -1;
	}else{
		printf("[OK] %s %d:Scan %d block-tag 0x%x ok!!\n", __FUNCTION__,__LINE__,
			NAND_QC_VE_ENCODE_BLK_ADDR,BLOCK_QC_VE_ENCODE);
	}
	hdr_size = ((512+nand->oobblock-1)/nand->oobblock)*nand->oobblock;
	start_addr = blk_addr* (nand->erasesize);
	if((ret = nand->rescue_image_read(nand, 
		(unsigned char *)NAND_TMP_BUFFER, start_addr, 
			hdr_size, 0, BLOCK_QC_VE_ENCODE)) < 0){
		printf("[ERR] %s: %d get image content err!!\n", __FUNCTION__,__LINE__);
		return ret;
	}
	
	/*read buffer content*/
	qc_ptr = (unsigned int *)NAND_TMP_BUFFER;
	if(*qc_ptr++ != 0x12345678)
	{
		printf("error magic no.\nno qc_ve_encode_pattern on flash\n");
		return ret;
	}
	buf_size = *qc_ptr;
	ret = nand->rescue_image_read(nand, 
		(unsigned char *)NAND_TMP_BUFFER, start_addr, 
			hdr_size+buf_size, 0, BLOCK_QC_VE_ENCODE);
	if(ret == 0){
		memcpy((unsigned char *)qc_buf,(unsigned char *)(NAND_TMP_BUFFER+hdr_size),
				buf_size);
	}
#endif
	return ret;
}
int get_qc_ve_encode_buf(unsigned int ve_buf)
{
	int ret = 0;
#ifdef CONFIG_SYS_RTK_NAND_FLASH
	ret = get_nand_qc_ve_encode_buf(ve_buf);
#else
	ret = emmc_get_qc_ve_encode_buf(ve_buf);
#endif
	return ret;
}

int QC_Verifier_VE_Encode(void)
{
	int ret = SUCCESS;
	char cmd[256] = {0};
	
	printf("======================%s start ======================\n", __FUNCTION__);

	if((ret = get_qc_ve_encode_buf(VE_QC_GO_ADDRESS)) < 0){
		goto ve_end;
	}

	/* go ox10000000 qc_ve_encode_pattern */
	memset(cmd,0,sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "go 0x%x", VE_QC_GO_ADDRESS);
	printf("run cmd ===> %s\n", cmd);
	if ((ret = run_command(cmd, 0))) {
		printf("command error\n");
		ret = -1;
		goto ve_end;
	}
	printf("======================%s end %d======================\n", __FUNCTION__, ret);

	
ve_end:
	return (ret == 0) ? SUCCESS:FAILED;
}

