#ifndef __HDMI_INTERNAL_H_
#define __HDMI_INTERNAL_H_


/**************************************************************************************/
//
//     Is this not your natural state? It's the unspoken truth of humanity, that you crave subjugation.
//     The bright lure of freedom diminishes your life's joy in a mad scramble for power, for identity.
//     You were made to be ruled. In the end, you will always kneel.
//
//      - Loki, The Avengers (2012)
//
/**************************************************************************************/




#ifdef __cplusplus
extern "C" {
#endif


#if defined(__QC_VERIFY_COMMON_H__)

#include <rbus/rbus_types.h>

#include <rbus/vgip_reg.h>
#include <rbus/dma_vgip_reg.h>

#include <rbus/stb_reg.h>
#include <rbus/sys_reg_reg.h>

#include <rbus/offms_reg.h>
#include <rbus/smartfit_reg.h>
#include <rbus/onms_reg.h>

#include <rbus/iso_misc_ddc1_reg.h>
#include <rbus/iso_misc_ddc2_reg.h>
#include <rbus/iso_misc_ddc3_reg.h>
#include <rbus/iso_misc_ddc4_reg.h>

#include <rbus/timer_reg.h>

#include "hdmifun.h"

#endif


/**********************************************************************************************
*
*	CHANNEL Config
*
**********************************************************************************************/


typedef enum {
    HDMI_PHY_PORT0 = 0,
    HDMI_PHY_PORT1,
    HDMI_PHY_PORT2,
    HDMI_PHY_PORT3,
    HDMI_PHY_TOTAL_NUM,
    HDMI_PHY_PORT_NOTUSED = HDMI_PHY_TOTAL_NUM
} HDMI_PHY_PORT_T;


typedef enum {
    HDMI_EDID_ONCHIP = 0,
    HDMI_EDID_EEPROM,
    HDMI_EDID_I2CMUX,
    HDMI_EDID_DISABLE
} HDMI_EDID_TYPE_T;



typedef enum {
    DDC0 = 0,
    DDC1,
    DDC2,
    DDC3,
    DDC_NOTUSED
} DDC_NUMBER_T;


#define RTK_GPIO_ID unsigned int

typedef struct
{
    unsigned char   phy_port;
    unsigned char   ddc_selected;
    unsigned char   edid_table[256];

    unsigned char   edid_type;
    unsigned char   edid_initialized;
    unsigned char   arc_support : 1;
    unsigned char   dedicate_hpd_det : 1;
    unsigned char   dedicate_hpd_det_sel : 2;    // (valid when dedicate_hpd_det=1)
    unsigned char   reserved : 4;

    RTK_GPIO_ID     hpd;        // GPIO for HPD    (valid when dedicate_hpd_det=0)
    RTK_GPIO_ID     det_5v;     // GPIO for 5V det (valid when dedicate_hpd_det=0)

} HDMI_CHANNEL_T;



typedef struct
{
    unsigned char   phy_port;
    unsigned char   ddc_selected;
    unsigned char   edid_type;
    unsigned char   arc_support : 1;
    unsigned char   dedicate_hpd_det : 1;
    unsigned char   dedicate_hpd_det_sel : 2;     // (valid when dedicate_hpd_det=1)
    unsigned char   reserved : 4;

    RTK_GPIO_ID     hpd;        // GPIO for HPD    (valid when dedicate_hpd_det=0)
    RTK_GPIO_ID     det_5v;     // GPIO for 5V det (valid when dedicate_hpd_det=0)

} HDMI_CHANNEL_T_PCB_INFO;





/**********************************************************************************************
*
*	Marco or Definitions
*
**********************************************************************************************/

#define USE_H5X_PTG					1
#define CBUS_NOT_BYPASS			1    // 1: for normal flow   0 : for MHL CTS  TEST
#define VALID_MIN_CLOCK			116
#define HDMI2p0					1
#define HDMI2p1					1
#define HDMI_OLD_CLK_DETECT			0
#define HDCP22_PATCH_DEFAULT_EN		1
#define MEAS_RETRY_LIMIT			6
#define HDMI_MEASURE_BY_SMART_FIT		1
#define BITERR_PATCH				1
#define AUDIO_OVER_192K			0
#define BIST_PLUG_CABLE				0
#define BIST_DFE_SCAN				1
#define BIST_PHY_SCAN				1
#define CONFIG_POWER_SAVING_MODE 1
#define MEASURE_FROM_TMDS_CLOCK 0          // 0: PIXEL CLOCK  1:TMDS CLOCK
#define HDMI_FRL_TRANS_DET 0
#define HDMI_DFE_BRSWAP			1
//USER:LewisLee DATE:2016/08/20
//just for short time 1080P 100Hz, 120Hz test
//It need webos modify FW
#define HDMI_120HZ_SHORT_TIME_PATCH		0 // 1

//USER:LewisLee DATE:2015/11/09
//Some Tx device, change timing have transient garbage
//clock chnage later than data change
//So need enable AV mute to fix it
//PS. It should be turn on 0xb8028300[15] is more better
#define HDMI_VIDEO_WD_ENABLE		1 // Video WatchDog function enable

//USER:LewisLee Date:2016/09/30
//to fix colorimetry NG case
#define HDMI_AVI_INFO_COLORIMETRY_ENABLE				1
#define HDMI_AVI_INFO_COLORIMETRY_DEBOUNCE_CNT		25

//USER:LewisLee Date:2016/10/08
//Fix Tx send stable clock at first, and check HDCP later
#define HDMI_CHECK_HDCP_ENABLE				1
#define HDMI_HDCP_DEBOUNCE_CNT				25

//USER:LewisLee DATE:2016/10/25
//modify clk_debounce_count threshold value
//to prevent HDCP handshake too fast
//set phy disable HDCP 1.4 will let picture unstable
#define HDMI_PHY_CLOCK_DYNAMIC_CNT_ENABLE	1

#define DISP_DEB_CNT_APPLETV				2

#define TMDS_ERR_DET_SHORT_PERIOD		1

//WOSQRTK-10354, PC40000
//Video card : AMD radeon R9 390 / Intel HD 4600 
//OC D5 8GB Tri-X NITRO - SAPPHIRE
#define DISP_DEB_CNT_PC40000				2

//USER:LewisLee DATE:2016/12/22
//WOSQRTK-10319, [STB-14] KT UHD STB (IC1000)
//STB DC off/on, will send AV mute after display, let it flicker
#define HDMI_TOGGLE_HPD_Z0_ENABLE		1
#define HDMI_START_TOGGLE_HPD_Z0_MS		5000
#define HDMI_TOGGLE_HPD_Z0_MS			400 // some times NG at 100 ~ 200, 500 ms is good
#define HDMI_DC_ON_DELAY_MS				5000 //OK, 3000ms NG

//USER:LewisLee DATE:2016/12/22
//WOSQRTK-10334, [STB-20] IC1100 - KT olleh GiGA UHD tv STB
//STR TV and STB DC off/on together, It will keep to send AVMute, can not display
#define HDMI_STR_TOGGLE_HPD_ENABLE		1
#define HDMI_STR_TOGGLE_HPD_MS			400

#if defined(CONFIG_HDMI_DRV_HPD)
#define HDMI_STR_HPD_SPEED_UP_ENABLE			1
#define  CEC_CEC_CR0_reg                                                         0xB8061E00
#define  CEC_CEC_CR0_get_cec_mode(data)                                          ((0xC0000000&(data))>>30)
#else
#define HDMI_STR_HPD_SPEED_UP_ENABLE			0
#endif

//USER:LewisLee DATE:2016/12/26
//for Tx output stable clock => unstbale => stable
//let output flicker issue
#define HDMI_DISPLAY_DELAY_ENABLE		1

#define HDMI_DISPLAY_DELAY_LHB_755_MS		600
#define HDMI_DISPLAY_DELAY_LHB_755_MS_2		200

#define HDMI_DISPLAY_DELAY_ROKU_ULTRA_4640R_MS		1500 // large 1500ms OK
#define HDMI_DISPLAY_DELAY_SAMSUNG_GX_HC630CH_MS		2000 // large 1500ms OK
#define HDMI_DISPLAY_DELAY_KT_STB_KAO_CT1100_MS		500 // large 500ms OK

//USER:LewisLee DATE:2017/01/04
//for Roku Ultra 4640R, if TV DC off/on, some times it will HDCP 2.2 handshake error
//and can not output
//If DC off/on, some times it will output HDCP 1.4
#define HDCP_STATUS_CHECK_ENABLE		1
#define HDCP_STATUS_CHECK_TOGGLE_MS	10000

#define HDCP_STATUS_CHECK_ROKU_ULTRA_4640R_HPD_MS	600

#define HPD_DEFAULT_LOW_MS	100
#define HPD_ROKU_ULTRA_4640R_LOW_MS 1000

//USER:Lewislee DATE:2017/07/07
//modify Interlace/progressive mode change detect
#define HDMI_INT_PRO_CHG_FLAG_ENABLE		1

#define ERRDET_LIMIT_COUNT	10


// 0xB800D090[16]
#define HDMI_GCP_ACKG_PACKET_PARSER_FIRST_BYTE   0
#define HDMI_GCP_ACKG_PACKET_PARSER_ALL                  1
#define HDMI_GCP_ACKG_PACKET_HEADER_PARSER_MODE  HDMI_GCP_ACKG_PACKET_PARSER_ALL

// 0xB800D024[19:18]
// Apply Value = Setting value + 3
// Valid range = [1,3]
#define HDMI_PK_GB_NUM  1

// 0xB800D024[17:16]
// Apply Value = Setting value + 3
// Valid range = [1,3]
#define HDMI_VD_GB_NUM  1

// 0xB800D024[15:13]
// Apply Value = Setting value + 1
// Valid range = [3,7]
#define HDMI_PK_PRE_NUM  3

// 0xB800D024[12:10]
// Apply Value = Setting value + 1
// Valid range = [3,7]
#define HDMI_VD_PRE_NUM  3

//#define DEBUG_REG_ADDR			0xb800d01c    // TMDS_PWDCTL
#define DEBUG_REG_ADDR			0xb800d508    // TMDS_PWDCTL

#define DEBUG_27_BYPASS_PHY_ISR	_BIT27
#define DEBUG_26_BYPASS_DETECT	_BIT26
#define DEBUG_25_BYPASS_CHECK		_BIT25
#define DEBUG_24_FORCE_6G		_BIT24

#define DEBUG_23_BITERR_DET		_BIT23
#define DEBUG_22_PHY_SCAN_TEST	_BIT22

#define DEBUG_21_FORCE_NO_INFOPKT	_BIT21
#define DEBUG_20_FORCE_NO_DRMPKT	_BIT20
#define DEBUG_19_BYPASS_PACKET_DET	_BIT19

#define DEBUG_21_QC_PORT3		_BIT21
#define DEBUG_20_QC_PORT2		_BIT20
#define DEBUG_19_QC_PORT1		_BIT19
#define DEBUG_18_QC_PORT0		_BIT18

#define DEBUG_17_PHY_DBG_MSG		_BIT17

#define DEBUG_17_DUMP_REG		_BIT17
#define DEBUG_16_PRINT_DRM	_BIT16
#define DEBUG_15_PRINT_AVI		_BIT15

#define DEBUG_16_FORCE_LTP_BYPASS		_BIT16
#define DEBUG_15_FORCE_FRL_12G_4_LANE	_BIT15
#define DEBUG_14_FORCE_FRL_10G_4_LANE	_BIT14
#define DEBUG_13_FORCE_FRL_8G_4_LANE		_BIT13
#define DEBUG_12_FORCE_FRL_6G_4_LANE		_BIT12
#define DEBUG_11_FORCE_FRL_6G_3_LANE		_BIT11
#define DEBUG_10_FORCE_FRL_3G_3_LANE		_BIT10



#define DBG_FW_ADDR			0xb800d508

#define  DEBUG_31_PLUG_CABLE_TEST	_BIT31
#define  DEBUG_30_DFE_SCAN_TEST		_BIT30
#define  DEBUG_29_BYPASS_AUDIO_DET	_BIT29
/* DBG_FW_ADDR[28:0] forbidden to  use*/


#define HDMI_ABS(x, y)	 	((x > y) ? (x-y) : (y-x))
#define HDMI_CONST		const


extern const char* _hdmi_video_fsm_str(unsigned int fsm);
extern const char* _hdmi_audio_fsm_str(unsigned int fsm);
extern const char* _hdmi_color_space_str(unsigned char color_space);
extern const char* _hdmi_color_depth_str(unsigned char depth);


#define SET_H_VIDEO_FSM(port, fsm)	(hdmi_rx[port].video_fsm = fsm)
#define SET_H_AUDIO_FSM(port, fsm)	(hdmi_rx[port].audio_fsm = fsm)

#define SET_H_INTERLACE(port, interlace)	(hdmi_rx[port].timing_t.is_interlace = interlace)
#define SET_H_MODE(port, m)		(hdmi_rx[port].timing_t.mode = m)
#define SET_H_COLOR_DEPTH(port, color)	(hdmi_rx[port].timing_t.colordepth= color)
#define SET_H_COLOR_SPACE(port, color)	(hdmi_rx[port].timing_t.colorspace = color)
#define SET_H_PIXEL_REPEAT(port, pr)	(hdmi_rx[port].timing_t.pixel_repeat = pr)
#define SET_H_VIDEO_FORMAT(port, fmt)	(hdmi_rx[port].timing_t.hvf = fmt)
#define SET_H_3DFORMAT(port, fmt) 	(hdmi_rx[port].timing_t.h3dformat = fmt)

#define SET_AUDIO_TYPE(port, type)	(hdmi_rx[port].audio_type = type)

#define GET_H_VIDEO_FSM(port)		(hdmi_rx[port].video_fsm)
#define GET_H_AUDIO_FSM(port)		(hdmi_rx[port].audio_fsm)

#define GET_H_INTERLACE(port)		(hdmi_rx[port].timing_t.is_interlace)
#define GET_H_MODE(port)		(hdmi_rx[port].timing_t.mode)
#define GET_H_COLOR_DEPTH(port)	(hdmi_rx[port].timing_t.colordepth)
#define GET_H_COLOR_SPACE(port)	(hdmi_rx[port].timing_t.colorspace)
#define GET_H_PIXEL_REPEAT(port)	(hdmi_rx[port].timing_t.pixel_repeat)
#define GET_H_VIDEO_FORMAT(port) 	(hdmi_rx[port].timing_t.hvf)
#define GET_H_3DFORMAT(port) 		(hdmi_rx[port].timing_t.h3dformat)

#define GET_AUDIO_TYPE(port)		(hdmi_rx[port].audio_type)


#define GET_VGIP_CHNx_CTRL_ADDR()		(vgip_ch ? SUB_VGIP_VGIP_CHN2_CTRL_reg : VGIP_VGIP_CHN1_CTRL_reg)
#define GET_VGIP_CHNx_STATUS_ADDR()		(vgip_ch ? SUB_VGIP_VGIP_CHN2_STATUS_reg : VGIP_VGIP_CHN1_STATUS_reg)
#define GET_VGIP_CHNx_V_CAP_ADDR()  		(vgip_ch ? SUB_VGIP_VGIP_CHN2_ACT_VSTA_Length_reg : VGIP_VGIP_CHN1_ACT_VSTA_Length_reg)
#define GET_VGIP_CHNx_H_CAP_ADDR()  		(vgip_ch ? SUB_VGIP_VGIP_CHN2_ACT_HSTA_Width_reg : VGIP_VGIP_CHN1_ACT_HSTA_Width_reg)
#define GET_VGIP_CHNx_DELAY_ADDR()  		(vgip_ch ? SUB_VGIP_VGIP_CHN2_DELAY_reg : VGIP_VGIP_CHN1_DELAY_reg)



#define GET_SFT_AUTO_FIELD_VADDR()				(SMARTFIT_AUTO_FIELD_reg)
#define GET_SFT_AUTO_H_BOUNDARY_VADDR()			(SMARTFIT_AUTO_H_BOUNDARY_reg)
#define GET_SFT_AUTO_V_BOUNDARY_VADDR()			(SMARTFIT_AUTO_V_BOUNDARY_reg)
#define GET_SFT_AUTO_ADJ_VADDR()					(SMARTFIT_AUTO_ADJ_reg)
#define GET_SFT_AUTO_RESULT_VSTA_END_VADDR() 	(SMARTFIT_AUTO_RESULT_VSTA_END_reg)
#define GET_SFT_AUTO_RESULT_HSTA_END_VADDR() 	(SMARTFIT_AUTO_RESULT_HSTA_END_reg)



#define PLL_PIXELCLOCK_X1024_160M    163840 // 160*1024
#define PLL_PIXELCLOCK_X1024_300M    307200 // 300*1024


/**********************************************************************************************
*
*	Enumerations or Structures
*
**********************************************************************************************/

// -------------------------------------  hdmiPhy ----------------------------------------------//


typedef struct {
	unsigned short b;
	unsigned short b_pre;
	unsigned short b_count;
	unsigned short b_debouce_count;
	unsigned char b_change;
	unsigned char avi_info_in;
	unsigned short avi_info_miss_cnt;
	unsigned short LG_patch_timer;
	unsigned short timer;
	short    hotplug_timer;
	unsigned short hotplug_delay_count;
} HDMIRX_IOCTL_STRUCT_T;


//HD20 PHY structure
typedef struct {
	u_int16_t b_upper;
	u_int16_t b_lower;

	u_int16_t M_code;
	u_int16_t N_code;
	u_int8_t pscaler;
	u_int8_t SEL_D4;
	u_int8_t CMU_KVCO;  //merlin5 remove

	u_int8_t EN_CAP;
	u_int8_t CK_CS;
	u_int8_t CK_Icp;
	u_int8_t Icp_DUL;
	u_int8_t CK_RS;

	u_int8_t CK_LDOD;
	u_int8_t PI_DIV;
	u_int8_t DEMUX;
	u_int8_t RATE_SEL;
	u_int8_t PIX2_SEL;

	u_int8_t PI_ISEL;
	u_int8_t PI_CSEL;
	u_int8_t CDR_bias;
	u_int8_t CDR_KP;
	u_int8_t CDR_KP2;

	u_int8_t CDR_KI;
	u_int8_t CDR_KD;
	u_int8_t EQ_gain;
	u_int8_t EQ_bias;
	u_int8_t EQ_adj;

	u_int8_t PR;
	u_int8_t vsel_ldo;
	u_int8_t CMU_SEL_CP;
	u_int8_t PI_CURRENT;

	char *band_name;
} HDMI_PHY_PARAM_T;

typedef struct {
	u_int16_t fvco_upper;
	u_int16_t fvco_lower;
	u_int8_t CMU_KVCO;   //merlin5 remove
	u_int8_t EN_CAP;
	u_int8_t CCO_BAND_SEL;
	u_int8_t PI_ISEL;
	u_int8_t PI_CSEL;
}HDMI_PHY_PARAM2_T;



//HD21 phy structure

#ifdef HD20_FLD_MODE
typedef struct {
	u_int16_t b_upper;
	u_int16_t b_lower;

	//M N code
	u_int16_t M_code;
	u_int16_t N_code;
	u_int8_t PRE_M;  //pre scaler
	u_int8_t POST_M;  //SEL_D4
	
	u_int8_t MANUAL_SEL;
	u_int8_t DATA_LANE_CK_SRC;

	//CCO
	u_int8_t CMU_KVCO;
	u_int8_t EN_CAP;
	u_int8_t BAND_SEL;
	u_int8_t DELAY_CEL_SEL;
	u_int8_t CAP_EN_PLL_MODE;
	u_int8_t CAP_EN_ACDR_MODE;
	u_int8_t CAP_EN_MANUAL_MODE;
	u_int8_t RES_EN_PLL_MODE;
	u_int8_t RES_EN_ACDR_MODE;
	u_int8_t RES_EN_MANUAL_MODE;
	u_int8_t FINE_I_PLL_MODE;
	u_int8_t FINE_I_ACDR_MODE;
	u_int8_t FINE_I_MANUAL_MODE;
	u_int8_t COURSE_I_PLL_MODE;
	u_int8_t COURSE_I_ACDR_MODE;
	u_int8_t COURSE_I_MANUAL_MODE;
	u_int8_t DM_EN_PLL_MODE;
	u_int8_t DM_EN_ACDR_MODE;
	u_int8_t DM_EN_MANUAL_MODE;
	u_int8_t DM_I_PLL_MODE;
	u_int8_t DM_I_ACDR_MODE;
	u_int8_t DM_I_MANUAL_MODE;


	//CHARGE PUMP
	u_int8_t HALF_RATE_SEL;   //icp double
	u_int8_t CP_OP_EN;
	u_int8_t CP_CAP_EN;
	u_int8_t ICP_PLL_MODE;
	u_int8_t ICP_ACDR_MODE;
	u_int8_t CP_TIME;
	u_int8_t CP_TIME_2;

	//LPF
	u_int8_t VC_VOL_SEL;
	u_int8_t RS_PLL_MODE;
	u_int8_t RS_ACDR_MODE;
	u_int8_t RS_MANUAL_MODE;
	u_int8_t CS_PLL_MODE;
	u_int8_t CS_ACDR_MODE;
	u_int8_t CS_MANUAL_MODE;
	u_int8_t CP_PLL_MODE;
	u_int8_t CP_ACDR_MODE;
	u_int8_t CP_MANUAL_MODE;
	

	u_int8_t PI_DIV;
	u_int8_t DEMUX_RATE_SEL;
	u_int8_t RATE_SEL;
	u_int8_t PIX_RATE_SEL;

	u_int8_t PI_ISEL;
	u_int8_t PI_CSEL;
	u_int8_t KP;
	u_int8_t KI;
	u_int8_t FULL_RATE_SEL;

	char *band_name;

} HDMI21_PHY_TMDS_PARAM_T;
#else
typedef struct {
	u_int16_t b_upper;
	u_int16_t b_lower;

	//M N code
	u_int16_t M_code;
	u_int16_t N_code;
	u_int8_t PRE_M;  //pre scaler
	u_int8_t POST_M;  //SEL_D4
	
	u_int8_t MANUAL_SEL;
	u_int8_t DATA_LANE_CK_SRC;

	//CCO
	u_int8_t CMU_KVCO;
	u_int8_t EN_CAP;
	u_int8_t BAND_SEL;
	u_int8_t DELAY_CEL_SEL;
	u_int8_t CAP_EN_MANUAL_MODE;
	u_int8_t RES_EN_MANUAL_MODE;
	u_int8_t FINE_I_MANUAL_MODE;
	u_int8_t COURSE_I_MANUAL_MODE;
	u_int8_t DM_EN_MANUAL_MODE;
	u_int8_t DM_I_MANUAL_MODE;


	//CHARGE PUMP
	u_int8_t HALF_RATE_SEL;   //icp double
	u_int8_t CP_OP_EN;
	u_int8_t CP_CAP_EN;
	u_int8_t ICP_PLL_MODE;

	//LPF
	u_int8_t VC_VOL_SEL;
	u_int8_t RS_MANUAL_MODE;
	u_int8_t CS_MANUAL_MODE;
	u_int8_t CP_MANUAL_MODE;
	
	
	u_int8_t PI_DIV;
	u_int8_t DEMUX_RATE_SEL;
	u_int8_t RATE_SEL;
	u_int8_t PIX_RATE_SEL;

	u_int8_t PI_ISEL;
	u_int8_t PI_CURRENT_ADJ;
	u_int8_t PI_CSEL;
	u_int8_t KP;
	u_int8_t KI;
	u_int8_t FULL_RATE_SEL;	
	char *band_name;
} HDMI21_PHY_TMDS_PARAM_T;
#endif



typedef struct {
	u_int16_t b_upper;
	u_int16_t b_lower;

	//M N code
	u_int16_t M_code;
	u_int16_t N_code;
	u_int8_t PRE_M;  //pre scaler
	u_int8_t POST_M;  //SEL_D4
	
	u_int8_t MANUAL_SEL;
	u_int8_t DATA_LANE_CK_SRC;

	//CCO
	u_int8_t CMU_KVCO;
	u_int8_t EN_CAP;
	u_int8_t BAND_SEL;
	u_int8_t DELAY_CEL_SEL;
	u_int8_t CAP_EN_PLL_MODE;
	u_int8_t CAP_EN_ACDR_MODE;
	u_int8_t CAP_EN_MANUAL_MODE;
	u_int8_t RES_EN_PLL_MODE;
	u_int8_t RES_EN_ACDR_MODE;
	u_int8_t RES_EN_MANUAL_MODE;
	u_int8_t FINE_I_PLL_MODE;
	u_int8_t FINE_I_ACDR_MODE;
	u_int8_t FINE_I_MANUAL_MODE;
	u_int8_t COURSE_I_PLL_MODE;
	u_int8_t COURSE_I_ACDR_MODE;
	u_int8_t COURSE_I_MANUAL_MODE;
	u_int8_t DM_EN_PLL_MODE;
	u_int8_t DM_EN_ACDR_MODE;
	u_int8_t DM_EN_MANUAL_MODE;
	u_int8_t DM_I_PLL_MODE;
	u_int8_t DM_I_ACDR_MODE;
	u_int8_t DM_I_MANUAL_MODE;


	//CHARGE PUMP
	u_int8_t HALF_RATE_SEL;   //icp double
	u_int8_t CP_OP_EN;
	u_int8_t CP_CAP_EN;
	u_int8_t ICP_PLL_MODE;
	u_int8_t ICP_ACDR_MODE;
	u_int8_t CP_TIME;
	u_int8_t CP_TIME_2;

	//LPF
	u_int8_t VC_VOL_SEL;
	u_int8_t RS_PLL_MODE;
	u_int8_t RS_ACDR_MODE;
	u_int8_t RS_MANUAL_MODE;
	u_int8_t CS_PLL_MODE;
	u_int8_t CS_ACDR_MODE;
	u_int8_t CS_MANUAL_MODE;
	u_int8_t CP_PLL_MODE;
	u_int8_t CP_ACDR_MODE;
	u_int8_t CP_MANUAL_MODE;
	

	u_int8_t PI_DIV;
	u_int8_t DEMUX_RATE_SEL;
	u_int8_t RATE_SEL;
	u_int8_t PIX_RATE_SEL;

	u_int8_t PI_ISEL;
	u_int8_t PI_CSEL;
	u_int8_t KP;
	u_int8_t KI;
	u_int8_t FULL_RATE_SEL;
	char *band_name;
} HDMI21_PHY_FRL_PARAM_T;



typedef struct {
	u_int16_t fvco_upper;
	u_int16_t fvco_lower;
	u_int8_t CMU_KVCO;
	u_int8_t EN_CAP;
	u_int8_t CCO_BAND_SEL;
	u_int8_t PI_ISEL;
	u_int8_t PI_CURRENT_ADJ;
	u_int8_t PI_CSEL;
	u_int8_t FINE_I;
	u_int8_t COURSE_I;
}HDMI21_PHY_TMDS_PARAM2_T;


typedef struct {
	u_int16_t b_upper;
	u_int16_t b_lower;
	u_int8_t RLSEL_LEQ;
	u_int8_t RLSEL_LEQ1_NC1;
	u_int8_t RLSEL_LEQ1_NC2;
	u_int8_t RLSEL_TAP0;
	u_int8_t RSSEL_LEQ2;
	u_int8_t RSSEL_LEQ1_1;
	u_int8_t RSSEL_LEQ1_2;
	u_int8_t RSSEL_TAP0;
	u_int8_t LEQ_CURRETN_ADJ;
	u_int8_t LEQ_ISEL;
	u_int8_t LE_PTAT_ISEL;
	u_int8_t TAP0_ISEL;
	u_int8_t POW_PTAT;
	u_int8_t POW_PTAT_STAGE1;
	u_int8_t DFE_Bleed;
}HDMI21_EQ_PARAM_T;


typedef struct {
	//Phase I
	u_int16_t b_upper;
	u_int16_t b_lower;
	u_int8_t vco_fine_init;
	u_int8_t vco_coarse_init;
	u_int8_t divide_num;
	u_int16_t coarse_lock_up_limit;
	u_int16_t coarse_lock_dn_limit;

	//phase II
	u_int16_t lock_up_limit;
	u_int16_t lock_dn_limit;

}HDMI21_FLD_PARAM_T;

typedef struct {
	unsigned int b_upper;
	unsigned int b_lower;
	unsigned int lock_up_limit; /*FLD upper bound*/
	unsigned int lock_dn_limit; /*FLD lower bound*/
	unsigned char cdr_bw_icp;
	unsigned char lfrs_sel;
	unsigned char vcogain;
	unsigned char hr_pd;
	unsigned char rate_sel;
} HDMI_ACDR_T;




#if 1//#ifdef HDMI_PHY_APPLY_NEW_METHOD
typedef struct
{
	unsigned char ucCDR_Reset;
	unsigned char ucDFE_DFE_Adapt_Start;
	unsigned char ucDFE_DFE_Adapt_3G_ReStart;
	unsigned char ucDFE_DFE_Adapt_6G_ReStart;
}HDMI_PHY_APPLY_PARAM_T;

typedef enum {
	HDMI_PHY_APPLY_NONE = 0,
	HDMI_PHY_APPLY_CDR_RESET,
	HDMI_PHY_APPLY_DFE_ADAPT_START,
	HDMI_PHY_APPLY_3G_DFE_ADAPT_RE_START,
	HDMI_PHY_APPLY_6G_DFE_ADAPT_RE_START,
	HDMI_PHY_APPLY_UNKNOW,
}  HDMI_PHY_APPLY_T;
#endif //#ifdef HDMI_PHY_APPLY_NEW_METHOD


#if 1//#if HDCP_STATUS_CHECK_ENABLE
/*
0x001: unauthenticated
0x002: compute km
0x004: compute L'
0x008: compute ks
0x010: authenticated
0x020: wait for downstream
0x040: assemble receiver ID list
0x080: verify receiver ID list acknowledgement
0x100: content stream management
0x200: content stream management from wait for downstream which has not been done
0x400: content stream management from assemble receiver ID list which has not been done
0x800: content stream management from verify receiver ID list acknowledgement which has not been done
*/

typedef enum {
	HDMI_HDCP_2P2_UNAUTH = 0x001,
	HDMI_HDCP_2P2_COMPUTE_KM = 0x002,
	HDMI_HDCP_2P2_COMPUTE_LP = 0x004,
	HDMI_HDCP_2P2_COMPUTE_KS = 0x008,
	HDMI_HDCP_2P2_AUTHENTICATED = 0x010,
	HDMI_HDCP_2P2_WAIT_FOR_DOWNSTREAM = 0x020,
	HDMI_HDCP_2P2_ASSEMBLE_RX_ID_LIST = 0x040,
	HDMI_HDCP_2P2_VERIFY_RX_ID_LIST_ACK = 0x080,
	HDMI_HDCP_2P2_CONTENT_STREAM_MANAGEMENT = 0x100,
	HDMI_HDCP_2P2_CONTENT_STREAM_MANAGEMENT_2 = 0x200,
	HDMI_HDCP_2P2_CONTENT_STREAM_MANAGEMENT_3 = 0x400,
	HDMI_HDCP_2P2_CONTENT_STREAM_MANAGEMENT_4 = 0x800,
	HDMI_HDCP_2P2_CONTENT_UNKNOW,
}  HDMI_HDCP_2P2_STATUS_T;

typedef enum {
	HDMI_HDCP_TOGGLE_NONE = 0,
	HDMI_HDCP_TOGGLE_INIT,
	HDMI_HDCP_TOGGLE_ENABLE,
	HDMI_HDCP_TOGGLE_EXIT,
	HDMI_HDCP_TOGGLE_UNKNOW,
}  HDMI_HDCP_TOGGLE_STATUS_T;
#endif //#if HDCP_STATUS_CHECK_ENABLE







// -------------------------------------  HdmiHdmi ----------------------------------------------//

typedef enum {

	HDMIRX_DETECT_FAIL = 0,
	HDMIRX_DETECT_SUCCESS,
	HDMIRX_DETECT_AVMUTE,
	HDMIRX_CHECK_MODE,
	HDMIRX_RELEASED,

} HDMIRX_APSTATUS_T;

typedef struct {
	unsigned char SM;
	unsigned char SN;
	unsigned char RatioM;
	unsigned char RatioN;
} VIDEO_DPLL_RATIO_T;



typedef struct {
	unsigned char dpll_M;
	unsigned char dpll_N;
	unsigned char dpll_IP;
	unsigned char dpll_RS;
}VIDEO_DPLL_FIX_T;



typedef struct {
	unsigned char min_M;
	unsigned char max_M;
	unsigned char dpll_IP;
	unsigned char dpll_RS;

} VIDEO_DPLL_RANGE_T;


typedef enum {
	HDMI_SMT_PATH_MAIN_VGIP = 0,
	HDMI_SMT_PATH_SUB_VGIP,
	HDMI_SMT_PATH_DMA_VGIP,
	HDMI_SMT_PATH_UNKNOW,
} HDMI_SMT_PATH;

#if 0

typedef struct
{
    unsigned char   VSIF_TypeCode;
    unsigned char   VSIF_Version;
    unsigned char   Length;
    unsigned char   Reg_ID[3];
    unsigned char   Payload[25];
}HDMI_VSI_T;

typedef struct
{
    unsigned char type_code;
    unsigned char ver;
    unsigned char len;
    unsigned int    S:2;
    unsigned int    B:2;
    unsigned int    A:1;
    unsigned int    Y:3;
    unsigned int    R:4;
    unsigned int    M:2;
    unsigned int    C:2;
    unsigned int    SC:2;
    unsigned int    Q:2;
    unsigned int    EC:3;
    unsigned int    ITC:1;
    unsigned int    VIC:8;
    unsigned int    PR:4;
    unsigned int    CN:2;
    unsigned int    YQ:2;
    unsigned char   ETB07_00;
    unsigned char   ETB15_08;
    unsigned char   SBB07_00;
    unsigned char   SBB15_08;
    unsigned char   ELB07_00;
    unsigned char   ELB15_08;
    unsigned char   SRB07_00;
    unsigned char   SRB15_08;
}HDMI_AVI_T;

typedef struct
{
    unsigned char   type_code;
    unsigned char   ver;
    unsigned char   len;
    unsigned char   VendorName[8];
    unsigned char   ProductDesc[16];
    unsigned char   SourceInfo;
}HDMI_SPD_T;

typedef struct
{
    unsigned char type_code;
    unsigned char ver;
    unsigned char len;
    unsigned int    CC:3;
    unsigned int    F13:1;
    unsigned int    CT:4;
    unsigned int    SS:2;
    unsigned int    SF:3;
    unsigned int    F27_25:3;
    unsigned int    CXT:5;
    unsigned int    F37_35:3;
    unsigned int    CA:8;
    unsigned int    LFEPBL:2;
    unsigned int    F52:1;
    unsigned int    LSV:4;
    unsigned int    DM_INH:1;
    unsigned char   F67_60;
    unsigned char   F77_70;
    unsigned char   F87_80;
    unsigned char   F97_90;
    unsigned char   F107_100;
}HDMI_AUDIO_T;

typedef struct
{
        unsigned char type_code;
        unsigned char ver;
        unsigned char len;
        unsigned char  eEOTFtype;
        unsigned char  eMeta_Desc;
        unsigned short display_primaries_x0;
        unsigned short display_primaries_y0;
        unsigned short display_primaries_x1;
        unsigned short display_primaries_y1;
        unsigned short display_primaries_x2;
        unsigned short display_primaries_y2;
        unsigned short white_point_x;
        unsigned short white_point_y;
        unsigned short max_display_mastering_luminance;
        unsigned short min_display_mastering_luminance;
        unsigned short maximum_content_light_level;
        unsigned short maximum_frame_average_light_level;
		unsigned char  reserved;
}__attribute__((packed)) HDMI_DRM_T;
#endif





typedef struct {

	unsigned int v_total;
	unsigned int h_total;
	unsigned int v_act_len;
	unsigned int h_act_len;
	unsigned int v_act_sta;
	unsigned int h_act_sta;
	unsigned int v_freq;
	unsigned int h_freq;
	unsigned int polarity;
	unsigned int mode_id;
	unsigned int modetable_index;
	unsigned char is_interlace;
	unsigned int IVSyncPulseCount;	//Input VSYNC High Period Measurement Result
	unsigned int IHSyncPulseCount;	//Input HSYNC High Period Measurement Result

	HDMI_DVI_MODE_T 	mode;
	HDMI_COLOR_DEPTH_T colordepth;
	HDMI_COLOR_SPACE_T colorspace;
	HDMI_COLORIMETRY_T  colorimetry;
	unsigned char pixel_repeat;

	HDMI_HVF_E hvf;
	HDMI_3D_T h3dformat;

	unsigned int tmds_clk_b;
	unsigned long pll_pixelclockx1024;
	unsigned char tmds_clk_stable_cnt;

	// only for fram packing
	unsigned int v_active_space1;
	unsigned int v_active_space2;
	unsigned char avi_vic;
	unsigned char run_vrr;
	unsigned char run_dsc;    // 0: do not run dsc flow, 1: run dsc flow
} HDMI_TIMING_T;


typedef struct
{
    unsigned char  vic;
    unsigned short hActive;
    unsigned short vActive;
    unsigned char prog;    // 0: interlace, 1: progressive
    unsigned short hFreq;
    unsigned short vFreq;
    unsigned short hTotal;
    unsigned short vTotal;
    unsigned short hSyncWidth;
    unsigned short hBackPorch;
    unsigned short vSyncWidth;
    unsigned short vBackPorch;
    unsigned char PolarityFlag;
}VIC_TIMING_INFO;

typedef enum _HDMI_PORT_{
	HDMI_PORT0 = 0,
	HDMI_PORT1,
	HDMI_PORT2,
	HDMI_PORT3,
	HDMI_PORT_TOTAL_NUM
}HDMI_PORT_E;


typedef enum {
	MAIN_FSM_HDMI_WAIT_SYNC,
	MAIN_FSM_HDMI_SETUP_VEDIO_PLL,
	MAIN_FSM_HDMI_MEASURE,
	MAIN_FSM_HDMI_MEASURE_ACTIVE_SPACE,
	MAIN_FSM_HDMI_WAIT_READY,
	MAIN_FSM_HDMI_DISPLAY_ON,
	MAIN_FSM_HDMI_VIDEO_READY
} HDMI_MAIN_FSM_T;


typedef enum {
	AUDIO_FSM_AUDIO_START = 0,
	AUDIO_FSM_FREQ_DETECT,
	AUDIO_FSM_AUDIO_WAIT_TO_START,
	AUDIO_FSM_AUDIO_START_OUT,
	AUDIO_FSM_AUDIO_WAIT_PLL_READY,
	AUDIO_FSM_AUDIO_CHECK
} HDMI_AUDIO_FSM_T;


typedef enum {

	AUDIO_FORMAT_LPCM = 0,
	AUDIO_FORMAT_NONLPCM

} HDMI_AUDIO_FORMAT_T;

typedef enum {
	HDMI_MS_ONESHOT = 0,
	HDMI_MS_CONTINUOUS,
	HDMI_MS_DIRECT
} HDMI_MS_T;

typedef enum _HDMI_HDCP_{
	NO_HDCP = 0,
	HDCP14,
	HDCP22,
	HDCP_OFF
}HDMI_HDCP_E;

typedef enum
{
	HDMI_IPCHECK_PROGRESSIVE = 0,
	HDMI_IPCHECK_INTERLACE,
	HDMI_IPCHECK_VSYNC_NO_CHANGED
}HDMI_INT_PRO_CHECK_RESULT;


#if 1//#if HDMI_TOGGLE_HPD_Z0_ENABLE
typedef enum _HDMI_DC_ON_STATE_{
	HDMI_DC_ON_NONE = 0,
	HDMI_DC_ON_ENABLE_DELAY,
	HDMI_DC_ON_START_TIME,
	HDMI_DC_ON_FINISH,
	HDMI_DC_ON_UNKNOW
}HDMI_DC_ON_STATE_E;
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

#if 1//#if HDMI_STR_TOGGLE_HPD_ENABLE
typedef enum _HDMI_STR_HPD_TOGGLE_STATE_{
	HDMI_STR_HPD_TOGGLE_NONE = 0,
	HDMI_STR_HPD_TOGGLE_ENTER,
	HDMI_STR_HPD_TOGGLE_READY,
	HDMI_STR_HPD_TOGGLE_UNKNOW,
}HDMI_STR_HPD_TOGGLE_STATE_E;
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE

#if 1//#if HDMI_DISPLAY_DELAY_ENABLE
typedef enum _HDMI_DISPLAY_DELAY_STATE_{
	HDMI_DISPLAY_DELAY_NONE = 0,
	HDMI_DISPLAY_DELAY_INIT,
	HDMI_DISPLAY_DELAY_START,
	HDMI_DISPLAY_DELAY_FINISH,
}HDMI_DISPLAY_DELAY_STATE_STATE_E;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

#if 1//#if HDMI_DISPLAY_DELAY_ENABLE
typedef enum _HDMI_SPECIFIC_DEVICE_{
	HDMI_SPECIFIC_DEVICE_NONE = 0,
	HDMI_SPECIFIC_DEVICE_SK_STB,
	HDMI_SPECIFIC_DEVICE_KT_STB_IC1000,
	HDMI_SPECIFIC_DEVICE_KT_STB_IC1100,
	HDMI_SPECIFIC_DEVICE_KT_STB_KT600EJ, //[STB-29 GX-KT600EJ]
	HDMI_SPECIFIC_DEVICE_LHB_755, // [HOME-05] LHB755 - LG
	HDMI_SPECIFIC_DEVICE_ROKU_ULTRA_4640R,
	HDMI_SPECIFIC_DEVICE_SAMSUNG_GX_HC630CH,
	HDMI_SPECIFIC_DEVICE_KT_STB_KAO_CT1100,
}HDMI_SPECIFIC_DEVICE_E;
#endif //#if HDMI_DISPLAY_DELAY_ENABLE


typedef struct {
	unsigned char coding_type; // 0=pcm; 1=nonpcm
	unsigned char sr_mismatch_cnt;
	unsigned char track_mode_tgl;
} HDMI_AUDIO_ST;


typedef struct
{
	unsigned char interlace : 1;     // video is progressive or interlace
	unsigned char vs_locked : 7;     // vsync is stable (2 vsync freq is the same)
	unsigned char vs_valid;          // vsync is valid
	unsigned char vs_pol :1;         // vsync polarity : 0 : negative, 1 : positive
	unsigned char vs_pol_stable :7;  // vsync stable count (increase when vs_pol is stable)
	unsigned int last_vs_cnt;          //last  number of vs detected
	unsigned int  vs_cnt;            // number of vs detected
	unsigned char last_interlace_status[3];
	unsigned char is_interlace_chg;

	// for ps measure
	unsigned char ps_measure_update;
	unsigned char ps_measure_stable_cnt;  // number of ps_measure result is stable
	unsigned int  vs2vs_cnt;
	unsigned int  last_pixel2vs_cnt;
} HDMI_VIDEO_ST;
typedef struct {

	unsigned char hdmi_2p1_en;   // 0 : hdmi 2.0 mac, others : hdmi 2.1 mac
	
#ifdef HDMI_FIX_RL6583_2215
	unsigned char hdmi_2p1_fw_pol_check_cnt;
	unsigned char hdmi_2p1_vs_pol;
#endif

	HDMI_PHY_PARAM_T phy_param_t;

	HDMI_TIMING_T timing_t;
	HDMI_AUDIO_ST audio_t;
	HDMI_VIDEO_ST video_t;

	HDMI_AVI_T avi_t;
	HDMI_VSI_T vsi_t;       // HDMI-VSIF (1.4)/  HDMI-Forum VSIF HDMI (2.0)
	HDMI_VSI_T dvsi_t;      // Dolby VSIF
	HDMI_VSI_T hdr10pvsi_t;      // HDR10+, VSIF packet
	
	HDMI_SPD_T spd_t;
	HDMI_AUDIO_T audiopkt_t;
	HDMI_DRM_T drm_t;
	unsigned int  pkt_cnt;
	unsigned char no_avi_cnt;
	unsigned char err_avi_cnt;
	unsigned char no_vsi_cnt;
	unsigned char no_spd_cnt;
	unsigned char err_spd_cnt;
	unsigned char no_audiopkt_cnt;
	unsigned char no_drm_cnt;
	unsigned char no_rsv2_cnt;

	// EMP
	unsigned int  vtem_emp_cnt;
	unsigned int  hdr_emp_cnt;
	unsigned int  vsem_emp_cnt;
	unsigned int  cvtem_emp_cnt;
//	unsigned char vtem_emp[EM_VTEM_INFO_LEN];
//	unsigned char hdr_emp[MAX_EM_HDR_INFO_LEN];
//	unsigned char vsem_emp[EM_VSEM_INFO_LEN];
//	unsigned char cvtem_emp[EM_CVTEM_INFO_LEN];

	HDMI_MAIN_FSM_T video_fsm;
	HDMI_AUDIO_FSM_T audio_fsm;


#if HDMI2p0
	unsigned char scramble_flag;
	unsigned char clock40x_flag;
	unsigned char hdmi2p0_tmds_toggle_flag;
#endif

	unsigned char tx_aksv[5];
	unsigned char hdcp14_auth_count;
	unsigned char hdcp14_ri_count;	
	HDMI_HDCP_E hdcp_authing;	// 0=no hdcp; 1=hdcp1.4; 2=hdcp2.2

	unsigned char hdcp2_reauth   : 1;   // force reauth hdcp2
	unsigned char hdcp2_state    : 3;   // hdcp2 state : please ref HDMI_HDCP2_RX_STATE
	unsigned char hdcp2_auth_count : 4; // number of hdcp2 authen performed
	unsigned long hdcp2_wait_mesage_timout;
	
#if HDCP2_2 && HDCP22_PATCH_DEFAULT_EN
	unsigned char hdcp_init_period;	// Timeout to reset to turn on (hdcp1.4/2.2), while no sync
#endif

	unsigned char cable_conn;	// 5V or not
	unsigned char vs_cnt;

	unsigned char meas_retry_cnt;

#if HDMI2p1
	unsigned char lts;
	unsigned char frl_mode;
	unsigned char lane_num;
#endif

	unsigned char onlinemeasure_error;
	unsigned int fw_char_error[4];
	unsigned int fw_bit_error[4];
	unsigned int fw_bit_disp_error_detected; // 2.0: fw bit error, 2.1: disparity error
	unsigned char wait_hdcp14;
	unsigned char wait_avi;
	unsigned char wait_spd;

	//----------------------------------
	// error detector
	//----------------------------------
	unsigned int  hpd_recover_cnt;
	unsigned int  reset_phy_recover_cnt;
	unsigned long err_msg_display_timeout;

	// TMDS Async fifo error detection
	unsigned int  rgb_detect;
	unsigned int  rgb_async_fifo_error;
	unsigned int  rgb_async_fifo_error_timeout;

	// TMDS Channel fifo error detection
	unsigned int  rgb_de_align : 4;
	unsigned int  rgb_de_align_cnt : 28;
	unsigned int  ch_fifo_error;
	unsigned int  ch_fifo_error_tomeout;

	// BCH error detection
	unsigned int  bch_err_detect : 4;
	unsigned int  bch_err_cnt_continue : 28;
	unsigned int  bch_err_cnt;
	unsigned int  bch_1bit_err_cnt;    // for debugging

	// Packet detection
	unsigned int  pkt_cnt_prev;        // for no packet detection
	unsigned int  no_pkt_continue;     // number of frame that not receiving packet (reseted when packet received)
	unsigned int  no_pkt_and_vsync_unstable_continue;

	unsigned char tmds_clk_stable_cnt;
	unsigned char tmds_clk_stable_cnt_thd;
} HDMI_PORT_INFO_T;



typedef struct {
	unsigned char spd_vn_name[8];
	unsigned char psd_pd_desc[16];
} HDMI_POOR_DEV_T;



typedef struct{
	unsigned int h_act;
	unsigned int v_act;
	unsigned int h_tol;
	unsigned int v_tol;
}HDMI_TIMING_TABLE_T;




typedef enum {
	ACKG = 0,
	SPD_INFO,
	DRM_INFO,
	FVS,
	GM,
	VS_INFO,
	MPEG_INFO,
	ISRC2,
	ISRC1,
	ACP,
	AUDIO_INFO,
	AVI_INFO,
	ALL_PKT
} HDMI_PACKET_CONTENT_CLR;

typedef enum {
	HDMI_RX_VAR_INIT_SKIP_CASE1,
	HDMI_RX_VAR_INIT_SKIP_MAX
} HDMI_RX_VAR_INIT_SKIP;




/////////////////////////HDMI 2.1/////////////////////////////////
typedef enum {
	HDMI_3LANE= 3,
	HDMI_4LANE,
}HDMI_LANE_COUNT;

typedef enum {
	MODE_TMDS,
	MODE_FRL_3G_3_LANE,
	MODE_FRL_6G_3_LANE,
	MODE_FRL_6G_4_LANE,
	MODE_FRL_8G_4_LANE,
	MODE_FRL_10G_4_LANE,
	MODE_FRL_12G_4_LANE,
	
}HDMI_FRL_MODE_T;



typedef enum {
    init_state,
    comma_seq_search_state,
    comma_seq_lock_state
} data_align_state;

typedef enum {
	LTS_1,
	LTS_2,
	LTS_3,
	LTS_4,
	LTS_P,
	LTS_L,
}LT_STATE;


#if 0
typedef struct 
{
	unsigned char state;
	unsigned int  debounce_cnt;
	unsigned int timeout;
}lane_align_state;
#endif


/**********************************************************************************************
*
*	Variables
*
**********************************************************************************************/
extern const VIDEO_DPLL_RATIO_T dpll_ratio[];

extern const HDMI_ACTIVE_SPACE_TABLE_T hdmi_active_space_table[];










/**********************************************************************************************
*
*	External Funtion Declarations
*
**********************************************************************************************/

/*
#include <rtk_gpio.h>

typedef struct 
{
    unsigned char   ddc_channel;
    RTK_GPIO_ID     hpd;
    RTK_GPIO_ID     det_5v;
}HDMI_PORT_CONFIG;
*/
// -------------------------------------  hdmiHdmi ----------------------------------------------//
extern void newbase_hdmi_init(void);
extern void newbase_hdmi_uninit(void);
extern void newbase_hdmi_init_source(unsigned char port);
//extern void newbase_hdmi_init_source_ex(unsigned char port, HDMI_PORT_CONFIG* p_config);
extern void newbase_hdmi_release_source(unsigned char port);
extern void newbase_hdmi_disconnect_source(unsigned char port);

void newbase_hdmi_port_select(unsigned char port);

extern void newbase_hdmi_port_var_init_all(void);
extern void newbase_hdmi_port_var_init(unsigned char port);
extern void newbase_hdmi_port_var_init_skip(unsigned char port, HDMI_RX_VAR_INIT_SKIP Skip_Case, unsigned char ucRecover);

extern unsigned char newbase_hdmi_detect_mode(unsigned char port);
extern unsigned char newbase_hdmi_check_condition_change(unsigned char port);
extern unsigned char newbase_hdmi_wait_sync(unsigned char port, unsigned char frl_mode);
extern unsigned char newbase_hdmi_setup_video_pll(unsigned char port, unsigned char frl_mode);
extern unsigned char newbase_hdmi_measure(unsigned char port, unsigned char frl_mode);
extern unsigned char newbase_hdmi_display_on(unsigned char port);
extern unsigned char newbase_hdmi_video_ready(unsigned char port);
extern unsigned char newbase_hdmi_check_mode(void);



extern void newbase_hdmi_reset_video_state(unsigned char port);
extern unsigned char newbase_hdmi_get_video_state(unsigned char port);
extern void newbase_hdmi_reset_meas_counter(unsigned char port);

extern void newbase_hdmi_check_crc_0(unsigned char nport);


extern void newbase_hdcp_init_timer(unsigned char port);
extern void newbase_hdcp_handler(unsigned char port);
extern unsigned char newbase_hdcp_get_auth_mode(unsigned char port);

extern void newbase_hdmi2p0_check_tmds_config(unsigned char port);
extern unsigned char newbase_hdmi2p0_get_scramble_flag(unsigned char port);
extern unsigned char newbase_hdmi2p0_get_clock40x_flag(unsigned char port);
extern void newbase_hdmi2p0_detect_config(unsigned char port);
extern void newbase_hdmi2p0_reset_scdc_toggle(unsigned char port);
extern void newbase_hdmi2p0_inc_scdc_toggle(unsigned char port);




extern void newbase_hdmi_set_current_display_port(unsigned char port);
extern unsigned char newbase_hdmi_get_current_display_port(void);

//extern void _lock_timing_detect_sem(void);
//extern void _unlock_timing_detect_sem(void);

extern unsigned char newbase_hdmi_get_timing(unsigned char port, HDMI_TIMING_T *ptiming);
extern unsigned char newbase_hdmi_get_hvstart(unsigned char port, HDMI_TIMING_T *ptiming, unsigned char vsc_ch);
extern unsigned char newbase_hdmi_get_interlace_conti_mode(unsigned char nport);
extern unsigned char newbase_hdmi_get_is_interlace(unsigned char port);
extern unsigned char newbase_hdmi_get_hdmi_mode_reg(unsigned char port);
extern unsigned char newbase_hdmi_get_hdmi_mode(unsigned char port);
extern unsigned char newbase_hdmi_get_colordepth(unsigned char port);
extern unsigned char newbase_hdmi_get_colorspace_reg(unsigned char port);
extern unsigned char newbase_hdmi_get_colorspace(unsigned char port);
extern unsigned char newbase_hdmi_get_video_format_reg(unsigned char port);
extern unsigned char newbase_hdmi_get_video_format(unsigned char port);
extern unsigned char newbase_hdmi_get_3d_structure_reg(unsigned char port);
extern unsigned char newbase_hdmi_get_3d_structure(unsigned char port);
extern unsigned char newbase_hdmi_get_colorimetry(unsigned char port);
extern unsigned char newbase_hdmi_get_color_range(unsigned char port);
extern unsigned int newbase_hdmi_get_tmds_clockx10(unsigned char port);

// Configurations
extern void newbase_hdmi_save_pcbinfo(unsigned char port, const HDMI_CHANNEL_T_PCB_INFO *pcb);

// HDP & 5V DET related
extern unsigned char newbase_hdmi_get_5v_state(unsigned char port);
extern unsigned char newbase_hdmi_get_5v_stateNew(unsigned char port);

extern void newbase_hdmi_set_hpd(unsigned char nport, unsigned char high, unsigned char controllable);
extern void newbase_hdmi_set_hpdNew(unsigned char nport, unsigned char high, unsigned char controllable);

extern void newbase_hdmi_hpd_delay_work_init(void);
extern void newbase_hdmi_check_connection_state(unsigned char port);


// EDID related
extern void newbase_hdmi_edid_enable(unsigned char port, unsigned char enable);
extern void newbase_hdmi_read_edid(unsigned char port, unsigned char *edid, unsigned int len);
extern void newbase_hdmi_write_edid(unsigned char port, unsigned char *edid, unsigned int len);
extern void newbase_hdmi_save_edid(unsigned char port, const unsigned char *edid, const unsigned int len);
extern int  newbase_hdmi_edid_initialized(unsigned char port);

// PHY related
extern unsigned char newbase_hdmi_phy_port_status(unsigned char port);

// HDCP related

extern void newbase_hdmi_read_aksv(unsigned char port, unsigned char *pAKSV);
extern void newbase_hdmi_read_hdcp14_key(unsigned char *bksv, unsigned char *hdcp_key);
extern void newbase_hdmi_write_hdcp14_key(unsigned char *bksv, unsigned char *hdcp_key);
extern void newbase_hdmi_save_hdcp(const HDCP_KEY_T *key);



// Info frame related
//extern void _lock_info_pack_sem(void);
//extern void _unlock_info_pack_sem(void);
//extern unsigned char hdmi_get_infoframe_thread_stop(void);
//extern void hdmi_set_infoframe_thread_stop(unsigned char ucStop);
//extern void hdmi_set_infoframe_thread_stop_nonseamphore(unsigned char ucStop);
extern void hdmi_infoframe(void);

extern void newbase_hdmi_reset_all_infoframe(unsigned char port);
extern void newbase_hdmi_update_infoframe(unsigned char nport);
extern unsigned char newbase_hdmi_get_hdmi_mode_infoframe(unsigned char port);
extern unsigned char newbase_hdmi_get_vs_infoframe(unsigned char port, HDMI_VSI_T *p_vsi_t);
extern unsigned char newbase_hdmi_get_avi_infoframe(unsigned char port, HDMI_AVI_T *p_avt_t);
extern unsigned char newbase_hdmi_get_spd_infoframe(unsigned char port, HDMI_SPD_T *p_spd_t);
extern unsigned char newbase_hdmi_get_audio_infoframe(unsigned char port, HDMI_AUDIO_T *p_audio_t);
extern unsigned char newbase_hdmi_get_drm_infoframe(unsigned char port, HDMI_DRM_T *p_drm_t);
extern unsigned char newbase_hdmi_check_stable_before_display(void);
extern unsigned char newbase_hdmi_watchdog_apply(unsigned char Enable, unsigned char Condition);

extern void newbase_hdmi_set_info(unsigned char port);

#if HDCP2_2 // HDCP2_2
extern void  newbase_msg_hander(unsigned char port);
extern void newbase_hdmi_hdcp2_clear_status(unsigned char port);
#endif


unsigned char newbase_hdmi_is_dispsrc(unsigned char disp);

extern void newbase_hdmi_check_data_align(unsigned char nport,unsigned char frl_mode, unsigned lane_mode);
extern void newbase_set_frl_mode(unsigned char port, unsigned char frl_mode);
extern void newbase_set_lane_num(unsigned char port, unsigned char lane_num);
extern unsigned char newbase_get_lane_num(unsigned char port);
extern unsigned char newbase_get_frl_mode(unsigned char port);
extern void newbase_hdmi_frl_start(unsigned char nport,unsigned char frl_mode);
extern unsigned int newbase_hdmi_link_training(unsigned char port);
extern unsigned char lib_scdc_get_frl_mode(unsigned char nport);
extern void lib_hdmi_mac_hd20_tmds_input_eanbale(unsigned char nport, unsigned char enable);
extern void lib_hdmi_mac_set_hde_enable(unsigned char nport, unsigned char enable);
extern void lib_hdmi_mac_release(unsigned char nport, unsigned char frl_mode);
extern unsigned char lib_hdmi_get_is_interlace(unsigned char port, HDMI_MS_T one_shot_mode);//Need update
extern unsigned char lib_hdmi_get_is_interlace_change(unsigned char nport);
extern unsigned char lib_hdmi_set_video_pll(unsigned char nport, unsigned int b_clk, unsigned char frl_mode,unsigned char cd, unsigned char is_interlace, unsigned char is_420);
extern unsigned char lib_hdmi_is_video_output(unsigned char nport);
extern void lib_hdmi_bch_error_clr(unsigned char nport);
extern void lib_hdmi_wdm_enable(unsigned char nport, unsigned char enable);
extern unsigned char lib_hdmi_is_wdm_enable(unsigned char nport);
extern void lib_hdmi_auto_avmute_enable(unsigned char nport, unsigned char enable);
extern unsigned char lib_hdmi_is_auto_avmute_enable(unsigned char nport);
extern void lib_hdmi2p0_scdc_update_flag(unsigned char port);
#if HDMI_TOGGLE_HPD_Z0_ENABLE
extern void lib_hdmi_set_toggle_hpd_z0_flag(unsigned char nport, unsigned char uc_flag);
extern unsigned char lib_hdmi_get_toggle_hpd_z0_flag(unsigned char nport);
extern unsigned char lib_hdmi_toggle_hpd_z0_check(unsigned char nport, unsigned char ucInit);
extern void lib_hdmi_set_dc_on_delay_state(unsigned char nport, HDMI_DC_ON_STATE_E state);
extern HDMI_DC_ON_STATE_E lib_hdmi_get_dc_on_delay_state(unsigned char nport);
extern unsigned char lib_hdmi_dc_on_delay_check(unsigned char nport, unsigned char ucInit);
#endif //#if HDMI_TOGGLE_HPD_Z0_ENABLE

#if HDMI_STR_TOGGLE_HPD_ENABLE
void lib_hdmi_set_str_toggle_hpd_state(unsigned char nport, HDMI_STR_HPD_TOGGLE_STATE_E state);
HDMI_STR_HPD_TOGGLE_STATE_E lib_hdmi_get_str_toggle_hpd_state(unsigned char nport);
#endif //#if HDMI_STR_TOGGLE_HPD_ENABLE

#if HDMI_DISPLAY_DELAY_ENABLE
extern void lib_hdmi_set_delay_display_enable(unsigned char nport, unsigned char ucEnable);
extern unsigned char lib_hdmi_get_delay_display_enable(unsigned char nport);
extern void lib_hdmi_set_delay_display_state(unsigned char nport, HDMI_DISPLAY_DELAY_STATE_STATE_E state);
extern HDMI_DISPLAY_DELAY_STATE_STATE_E lib_hdmi_get_delay_display_state(unsigned char nport);
extern unsigned char lib_hdmi_delay_check_before_display(unsigned char nport, unsigned char ucInit);
extern unsigned int lib_hdmi_get_delay_display_time(unsigned char nport);
extern unsigned char lib_hdmi_delay_display_action(unsigned char port);
#endif //#if HDMI_DISPLAY_DELAY_ENABLE

void lib_hdmi_set_specific_device(unsigned char nport, HDMI_SPECIFIC_DEVICE_E device);
HDMI_SPECIFIC_DEVICE_E lib_hdmi_get_specific_device(unsigned char nport);

extern void lib_hdmi_misc_variable_initial(unsigned char port);

#if HDCP_STATUS_CHECK_ENABLE
extern void lib_hdmi_set_hdcp_check_toggle_state(unsigned char nport, HDMI_HDCP_TOGGLE_STATUS_T status);
extern HDMI_HDCP_TOGGLE_STATUS_T lib_hdmi_get_hdcp_check_toggle_state(unsigned char nport);
extern unsigned char lib_hdmi_hdcp_status_check(unsigned char nport, unsigned char IsInit);
extern unsigned char lib_hdmi_hdcp_toggle_action(unsigned char nport);
#endif //#if HDCP_STATUS_CHECK_ENABLE

extern void lib_hdmi_packet_contnet_clear(unsigned nport, unsigned char pkt_type);

extern void lib_hdmi_2p1_path_sel(unsigned char nport, unsigned char frl_mode, unsigned lane_mode);
//------------------------------------------------------------------
// Phy FIFO
//------------------------------------------------------------------
extern void lib_hdmi_hd21_phy_afifo_enable(unsigned char nport, unsigned char enable);
//------------------------------------------------------------------
// CH FIFO
//------------------------------------------------------------------

//------------------------------------------------------------------
// Data Align
//------------------------------------------------------------------

extern void lib_hdmi_hd21_force_realign(unsigned char nport);

//------------------------------------------------------------------
// Common
//------------------------------------------------------------------

extern unsigned char lib_hdmi_is_hdmi_21_available(unsigned char nport);

//extern void lib_hdmi_hd21_set_lane_mode_fw(unsigned char nport, HDMI_FRL_MODE_T frl_mode);


extern int lib_hdm_hdr_emp_sram_sel(unsigned char nport, unsigned char hd21);

#include "hdmi_mac.h"
#include "hdmi_mac_2p1.h"
#include "hdmi_phy.h"
//#include "hdmi_phy_2p1.h"
#include "hdmi_hdcp.h"


// -------------------------------------  hdmiMs ----------------------------------------------//
#define OFFMS_WAIT_TIME_OUT_CNT 30    // max count of check measure done 
#define OFFMS_CHECK_MSDONE_EVERY_MS    5    // Total timeout = OFFMS_WAIT_TIME_OUT_CNT x OFFMS_CHECK_MSDONE_EVERY_MS

typedef enum
{
	OFFMS_ONE_SHOT = 0,
	OFFMS_CONTINUOUS_MODE = 1
}OFFMS_MSMODE;
extern void set_no_polarity_inverse(unsigned char TorF);//allow vgip to set h v polarity inverse or not
extern unsigned char lib_hdmims_measure(HDMI_TIMING_T *tm, unsigned char frl_mode, unsigned char is_HV_Mode);
extern unsigned char lib_hdmims_get_hvstart(HDMI_TIMING_T *tm, unsigned char vsc_ch);
extern unsigned char lib_hdmims_active_correction(unsigned int *h_active, unsigned int *v_active);
extern unsigned char newbase_hdmims_measure_video(unsigned char port, HDMI_TIMING_T *tm, unsigned char frl_mode);
extern void newbase_hdmims_measure_init(void);
extern unsigned char lib_hdmims_calc_freq(HDMI_TIMING_T *tm);
extern unsigned char lib_hdmims_offline_measure(HDMI_TIMING_T *tm, unsigned char frl_mode, OFFMS_MSMODE ms_mode);
extern unsigned char lib_hdmims_offline_measure_demode(HDMI_TIMING_T *tm, unsigned char frl_mode);
extern unsigned char lib_hdmims_offline_measure_native(HDMI_TIMING_T *tm, unsigned char frl_mode);
extern unsigned char lib_hdmims_online1_get_ms_src_sel(void);
extern void lib_hdmims_online1_set_ms_src_sel(unsigned char ms_src_sel);
extern unsigned char lib_hdmims_online1_measure(HDMI_TIMING_T *tm);
extern unsigned char lib_hdmims_online3_get_ms_src_sel(void);
extern void lib_hdmims_online3_set_ms_src_sel(unsigned char ms_src_sel);
extern unsigned char lib_hdmims_online3_measure(HDMI_TIMING_T *tm);
extern unsigned char lib_hdmims_online3_demode_measure(HDMI_TIMING_T *tm);
extern void lib_hdmims_onms1_watchdog_int_en(unsigned char enable);
extern void lib_hdmims_onms3_watchdog_int_en(unsigned char enable);
extern void lib_hdmims_offms_config_before_startms(OFFMS_MSMODE ms_mode);
// ---------------- audio -------------------------------------------//
#include "hdmi_audio.h"

// -------------------------------------  hdmi_ddc ----------------------------------------------//
extern void lib_hdmiddc_crt_on(void);
extern void lib_hdmiddc_sync_reset(unsigned char port);
extern unsigned char lib_hdmiddc_write(unsigned char port, unsigned char* EDID, unsigned int len);
extern unsigned char lib_hdmiddc_read(unsigned char port, unsigned char *edid_tbl, unsigned int len);
extern void lib_hdmiddc_enable(unsigned char port, unsigned char enable);
extern void lib_hdmiddc_set_sda_delay(unsigned char port, unsigned char delay_sel);
extern void lib_hdmiddc_sda2gpio(unsigned char port);
extern void lib_hdmiddc_gpio2sda(unsigned char port);
extern void lib_hdmiddc_sda_dir_out(unsigned char port, unsigned char dir_o);
extern void lib_hdmiddc_sync_reset(unsigned char port);
//------------------------------------------------------------------
// BCH Error control
//------------------------------------------------------------------

//------------------------------------------------------------------
// CED
//------------------------------------------------------------------
extern void lib_hdmi_fw_char_err_rst(unsigned char nport, unsigned char rst);

//------------------------------------------------------------------
// Video Data Error Detection
//------------------------------------------------------------------

//------------------------------------------------------------------
// Deep Color
//------------------------------------------------------------------


#ifdef __cplusplus
}
#endif

#endif

