#ifndef __HDMI_SCDC_H__
#define __HDMI_SCDC_H__

#define SCDC_SINK_VERSION                    0x01
#define SCDC_SOURCE_VERSION                  0x02
#define SCDC_UPDATE_FLAGS                    0x10
#define SCDC_TMDS_CONFIGURATION              0x20 
#define SCDC_TMDS_SCRAMBLER_STATUS           0x21   
#define SCDC_CONFIGURATION                   0x30

#define SCDC_CONFIGURATION_1		     0x31

#define SCDC_SOURCE_TEST_CONFIGURATION       0x35

#define SCDC_STATUS_FLAGS				0x40
#define SCDC_STATUS_LN01_LTP_REQ			0x41
#define SCDC_STATUS_LN23_LTP_REQ			0x42

#define SCDC_TEST_READ_REQUEST               0xC0





// Update Flags (0x10)
#define SCDC_UPDATE_FLAGS_STATUS_UPDATE      (0x1)
#define SCDC_UPDATE_FLAGS_CED_UPDATE         (0x1<<1)
#define SCDC_UPDATE_FLAGS_RR_TEST            (0x1<<2)
#define SCDC_UPDATE_FLAGS_SOURCE_TEST_UPDATE (0x1<<3)
#define SCDC_UPDATE_FLAGS_FRL_START          (0x1<<4)
#define SCDC_UPDATE_FLAGS_FLT_UPDATE         (0x1<<5)
#define SCDC_UPDATE_FLAGS_RSED_UPDATE        (0x1<<6)   
   
// SCDC_TMDS_CONFIGURATION (0x20)
#define SCDC_SCAMBLING_ENABLE                (0x1)
#define SCDC_TMDS_BIT_CLOCK_RATIO            (0x1<<1)  
    
// SCDC_SCRAMBLER_STATUS
#define SCDC_SCAMBLING_STATUS                (0x1)  
   
// SCDC_CONFIGURATION   
#define SCDC_RR_ENABLE                       (0x1)  
   


//------------------------------------------------------------------------------
extern void lib_hdmi_scdc_enable(unsigned char nport, unsigned char enable);
extern void lib_hdmi_scdc_data_sel(unsigned char nport, unsigned char hdmi2p1);
extern void lib_hdmi_scdc_reset(unsigned char nport, unsigned char rst);
extern void lib_hdmi_scdc_reset_toggle(unsigned char nport);
extern void lib_hdmi_scdc_write(unsigned char nport, unsigned char addr,unsigned char value);
extern void lib_hdmi_scdc_write_mask(unsigned char nport, unsigned char addr, unsigned char mask, unsigned char value);
extern unsigned char lib_hdmi_scdc_read(unsigned char nport, unsigned char addr);
extern void lib_hdmi_scdc_ced_enable(unsigned char nport, unsigned char enable);
extern unsigned char lib_hdmi_scdc_is_sink_config_31_updated(unsigned char nport);
extern void lib_hdmi_scdc_clr_sink_config_31_update_status(unsigned char nport);


#define lib_hdmi_scdc_get_source_version(nport)         lib_hdmi_scdc_read(nport,  SCDC_SOURCE_VERSION)
#define lib_hdmi_scdc_get_update_flags(nport)           lib_hdmi_scdc_read(nport,  SCDC_UPDATE_FLAGS)
#define lib_hdmi_scdc_get_tmds_config(nport)            lib_hdmi_scdc_read(nport,  SCDC_TMDS_CONFIGURATION)
#define lib_hdmi_scdc_get_tmds_scrambler_status(nport)  lib_hdmi_scdc_read(nport,  SCDC_TMDS_SCRAMBLER_STATUS)
#define lib_hdmi_scdc_get_test_read_request(nport)      lib_hdmi_scdc_read(nport,  SCDC_TEST_READ_REQUEST)

//0x31
#define lib_hdmi_scdc_get_config_1(nport)      lib_hdmi_scdc_read(nport,  SCDC_CONFIGURATION_1)


#define lib_hdmi_scdc_get_status_flags(nport)	      lib_hdmi_scdc_read(nport,SCDC_STATUS_FLAGS)
#define lib_hdmi_scdc_get_ln01_req(nport)			lib_hdmi_scdc_read(nport,SCDC_STATUS_LN01_LTP_REQ)
#define lib_hdmi_scdc_get_ln23_req(nport)			lib_hdmi_scdc_read(nport,SCDC_STATUS_LN23_LTP_REQ)


#define lib_hdmi_scdc_set_sink_version(nport, ver)      lib_hdmi_scdc_write(nport, SCDC_SINK_VERSION, ver)
#define lib_hdmi_scdc_set_update_flags(nport, val)      lib_hdmi_scdc_write(nport, SCDC_UPDATE_FLAGS, val)

#define lib_hdmi_scdc_set_status_flags(nport,val)	      lib_hdmi_scdc_write(nport,SCDC_STATUS_FLAGS,val)
#define lib_hdmi_scdc_set_ln01_req(nport,val)			lib_hdmi_scdc_write(nport,SCDC_STATUS_LN01_LTP_REQ,val)
#define lib_hdmi_scdc_set_ln23_req(nport,val)			lib_hdmi_scdc_write(nport,SCDC_STATUS_LN23_LTP_REQ,val)

//------------------------------------------------------------------------------


//0x10
#define Status_Update_mask			0x01
#define CED_Update_mask			0x02		
#define RR_Test_mask				0x04
#define Source_Test_Update_mask	0x08
#define FRL_start_mask				0x10
#define FLT_update_mask				0x20
#define RSED_Update_mask			0x40

//0x11


//0x31
#define FRL_Rate_mask			0x0f
#define FFE_Levels_mask			0xf0


//0x40
#define Clock_Detected_mask 	0x01
#define Ch0_Ln0_Locked_mask 	0x02
#define Ch1_Ln1_Locked_mask 	0x04
#define Ch2_Ln2_Locked_mask 	0x08
#define Lane3_Locked_mask		0x10
#define FLT_ready_mask			0x40
#define DSC_DecodeFail_mask	0x80

//0x41
#define Ln0_LTP_req_mask		0x0f	
#define Ln1_LTP_req_mask		0xf0

//0x42
#define Ln2_LTP_req_mask		0x0f
#define Ln3_LTP_req_mask		0xf0


#if 0
#ifdef HDMI_SCDC_DBG_EN                        
    #define HDMI_SCDC_DBG(fmt, args...)        rtd_printk(KERN_NOTICE, "[HDMI][SCDC] DBG, ", fmt, ##args)
#else
    #define HDMI_SCDC_DBG(fmt, args...)
#endif

#define HDMI_SCDC_INFO(fmt, args...)          rtd_printk(KERN_INFO,   "[HDMI][SCDC] Info,", fmt, ##args)       
#define HDMI_SCDC_WARN(fmt, args...)          rtd_printk(KERN_EMERG,  "[HDMI][SCDC] Warn, ", fmt, ##args) 
#endif

#endif //__HDMI_SCDC_H__
