#include <common.h>
#include <linux/ctype.h>
#include <drivers/amp/rtk_amp_interface.h>
#include <platform_lib/board/pcb.h>



extern       struct amp_controller rtk_amp;

extern int fgets(char *s, int n);
extern char* strtrim(char *s);
extern void rtk_amp_i2c_init(void);
extern void rtk_amp_init(void);
char rtk_amp_model[16];

/* Strip whitespace chars off end of given string, in place. Return s. */
static char *rtrim(char *s)
{
    char *p = s + strlen(s);
    while (p > s && isspace(*--p)){
        *p = '\0';
    }
    return s;
}

static char *del_ch(char *s, char c){
    static char new_str[16];
    int i = 0,j = 0;
    while(s[i] != '\0'){
        if((s[i] != c)&&(s[i] != '\n')){
            new_str[j++] = s[i];
        }
        i++;
    }
    new_str[j] = '\0';
    return new_str;
}

int rtk_amp_boot_ini_exist(void)
{
    return (strlen(rtk_amp_model) != 0);
}

int rtk_amp_boot_setting_from_factory(void)
{
    int i = 0;
    char value[64]={0};
    char amp_name[16]={0};
    if(strlen(rtk_amp_model) == 0)
        return (-1); //ini file no exist

    for(i = 0; i< getAmpListLength(); ++i){
        strcpy(amp_name ,getAmpName(i));
        if(strcmp(rtrim(amp_name),del_ch(rtk_amp_model,'"')) == 0){
            printf("\n===================================================\n");
            printf("you have selected [x]  AMP_IC=%s, Slave_addr=%X\n", getAmpName(i), getAmpI2cAddress(i));
            printf("===================================================\n");
            snprintf(value, 64, "%d", getAmpDeviceId(i));
            setenv("amp_select", strim(value));
            snprintf(value, 64, "%x", getAmpI2cAddress(i));
            setenv("amp_select_value", value);
            saveenv();
            break;
        }
    }
    return 0;
}
int do_ampinit (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int ini_file_exit = 0;
    char buf[128]={0};
    int i  = 0, select = -1;
    char value[64]={0};
    const char *current_sel = getenv("amp_select");
    const char *factory_sel = getenv("amp_factory_sel");


    //Check amp info from ini
    ini_file_exit = rtk_amp_boot_ini_exist();

    if(current_sel != NULL)
        select = simple_strtoul(current_sel, NULL, 10);

    printf("===================================================\n");
    if(current_sel != NULL ){
        for( i = 0 ; i < getAmpListLength(); ++i){
            if( select == getAmpDeviceId(i)){
                if(factory_sel != NULL){
                    printf("The current AMP selection is [x] AMP_IC=%s, Slave_addr=%X\n",
                    getAmpName(i), getAmpI2cAddress(i));
                }
                else{
                    printf("The current AMP selection is [%d] AMP_IC=%s, Slave_addr=%X\n", i,
                    getAmpName(i), getAmpI2cAddress(i));
                }
            }
        }
    }
    else
    {
        printf("You don't select any AMP_IC\n");
    }
    printf("===================================================\n");

    printf("Please select the AMP_IC you want AMP_LIST_SIZE = %d :\n" , getAmpListLength() );
    if(ini_file_exit)
        printf("[x] Get from 000BootParam.h:%s", rtk_amp_model);

    for( i = 0; i < getAmpListLength() ; i++)
    {
        if(i == 0)
            printf("[%d] %s\n", i,  getAmpName(i));
        else
            printf("[%d] AMP_IC=%s - Slave_addr=%X\n", i, getAmpName(i), getAmpI2cAddress(i));
    }
    printf("=> ");
    fgets(buf, 127);

    if(buf[0] =='x'){
        setenv("amp_factory_sel",buf);
        rtk_amp_boot_setting_from_factory();
    }
    else{
        select = simple_strtoul((char *)strtrim(buf), NULL, 10);
        if(select < getAmpListLength() && select > 0)
        {
            printf("\n===================================================\n");
            printf("you have selected [%d]  AMP_IC=%s, Slave_addr=%X\n", select, getAmpName(select), getAmpI2cAddress(select));
            printf("===================================================\n");

            memset(value, 0, sizeof(value));
            snprintf(value, 64, "%d", getAmpDeviceId(select));
            setenv("amp_select", strim(value));

            memset(value, 0, sizeof(value));
            snprintf(value, 64, "%x", getAmpI2cAddress(select));
            setenv("amp_select_value", value);

            setenv("amp_factory_sel", NULL);
            saveenv();
        }
        else if((select == 0))
        {
            printf("\nClear amp setting! \n");
            if(ini_file_exit){
                printf("Setting from factory\n");
                buf[0] = 'x';
                setenv("amp_factory_sel", buf);
                rtk_amp_boot_setting_from_factory();
            }
            else{
                setenv("amp_select", NULL);
                setenv("amp_factory_sel", NULL);
                saveenv();
            }
        }
        else
        {
            printf("\n [ERR] wrong input! \n");
        }
    }

    AudioAmp_InitAudioOut2I2S_2nd();

    return 0;
}

U_BOOT_CMD(
        amp, 2, 1,    do_ampinit,
        "Run Amp Speaker Initial Task",
        "\n"
        "    - Run Amp Speaker Initial Task"
);

