/*
 * cmd_rtk_cw.c - interface to One-Time-Programmable memory
 *
 * Copyright (c) 2008-2015 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 */
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <net.h>
#include <drivers/cw/rtk_cw_util.h>
#include <drivers/cw/rtk_cw_reg.h>
static int g_cw_func_int = 0;
typedef enum {
    CW_CMD_HELP,
    CW_CMD_REQUEST_CWS,
    CW_CMD_RELEASE_CWS,
    CW_CMD_RELEASE_CWS_BY_MODULE,
    CW_CMD_SET_CWS,
    CW_CMD_GET_CWS,
    CW_CMD_QUERY_CWS_BY_MODULE,
    CW_CMD_DUMP_CWS_STATUS,
} CW_CMD_ID;

typedef struct {
    CW_CMD_ID      id;
    unsigned long   flags;
    unsigned long start_index;
    char *module_name;
    char *data;
    int len;
} CW_CMD;

static int str_to_long(char *str, long* p_val, unsigned char base)
{
    char *p;
    long int  val;

    base = (base == 8 || base == 10 || base == 16 || base == 32) ? base : 0;
    val = simple_strtoul(str, &p, base);

    if (*p == '\0') {
        *p_val = val;
        return 0;
    }
    return -1;
}







/*******************************************************************************
 * FUNC : do_cw_request_cws
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_request_cws(char *module_name,
                              unsigned char start_index, int len)
{
    int ret ;
    ret = rtk_cw_request_multi_cws(module_name, start_index, len);
    if(ret == 0) {
        printf("Request CWs Sucess\n");
    } else {
        printf("Request CWs failed\n");
    }
}


/*******************************************************************************
 * FUNC : do_cw_release_cws
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_release_cws(char *module_name,
                              unsigned char start_index, int len)
{
    int i = 0;
    for(i = 0; i < len; i++)
        rtk_cw_release_cw(module_name, start_index + i);
    printf("%s,Request CWS sucess\n", module_name);
}

/*******************************************************************************
 * FUNC : do_cw_release_cws_by_module
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_release_cws_by_module(char *module_name)
{
    int ret ;
    ret = rtk_cw_release_cw_by_module(module_name);
    if(ret == 0) {
        printf("Request All CWS of %s Sucess\n", module_name);
    } else {
        printf("Request All CWS of %s failed\n", module_name ? module_name : "");
    }
}

/*******************************************************************************
 * FUNC : do_cw_set_cws
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_set_cws(char *module_name, UINT8 start_index,
                          UINT8 *content, int len)
{
    int ret = -1;
    ret = rtk_cw_set_cw(module_name, start_index, content, len);
    if(ret == 0) {
        printf("Set CWs Sucess\n");
    } else {
        printf("Set CWs Failed\n");
    }
}

/*******************************************************************************
 * FUNC : do_cw_crc_partial_region
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_get_cws(char *module_name, UINT8 start_index, int len)
{
    UINT8 *content;
    int ret = -1;
    content = malloc(len);
    ret = rtk_cw_get_cw(module_name, start_index, content, len);
    if(ret == 0) {
        int i = 0;
        printf("Get CW Sucess,The cw data is:");
        for (i = 0; i < len; i++) {
            if(i % TPK_CW_LENGTH == 0)
                printf("\nCW[%d]:", start_index++);
            printf("%c", content[i]);
        }
        printf("\n");
        //dump_mem(content, len);
    } else {
        printf("Get CW Failed\n");
    }
    free(content);
}

/*******************************************************************************
 * FUNC : do_cw_query_used_cw_by_module
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_query_used_cw_by_module(char *module_name)
{
    UINT8 index_array[128];
    int i = 0;
    int len = rtk_cw_query_used_cw_by_module(module_name, index_array, 128);
    printf("The used cws by module[%s]: ", module_name);
    for (i = 0; i < len; i++) {
        printf("%d ", index_array[i]);
    }
    printf("\n");
}

/*******************************************************************************
 * FUNC : do_cw_dump_cw_status
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_cw_dump_cw_status(void)
{
    char buf[2048];
    int len = rtk_cw_dump_cw_status(buf);
    if(len > 0)
        printf(buf);
}


/*******************************************************************************
 * FUNC : do_help
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static void do_help(void)
{
    printf("                                                    \n");
    printf(" @@@ Realtek cw Tool @@@                           \n");
    printf("                                                    \n");
    printf("Format : cw_util <opcode> module_name=<module_name> start_index=<start_index> data=<data> len=<len>\n");
    printf("<opcode> : operations     \n");
    printf("  -d  : dump cw status             \n");
    printf("  -a  : request one or more cws          \n");
    printf("  -r  : release one or cws          \n");
    printf("  -c  : release cws of special module       \n");
    printf("  -s  : set the value one or more cws    \n");
    printf("  -g  : get the value one or more cws              \n");
    printf("  -q  : query the uesed cws  of special module               \n");
    printf("  -d  : query the uesed cws status of all modules              \n");
    printf("                                     \n");
    printf(" Example: Request a single cw or multi cws                             \n");
    printf("      cw_util -a module_name=<module_name> start_index=<start_index> len=<len>\n");
    printf("                                     \n");
    printf(" Example: Release a single cw or multi cws                             \n");
    printf("      cw_util -r module_name=<module_name> start_index=<start_index> len=<len>\n");
    printf("                                     \n");
    printf(" Example: Release all cws of special module                             \n");
    printf("      cw_util -q module_name=<module_name>\n");
    printf("                                     \n");
    printf(" Example: Set a single cw or multi cws                             \n");
    printf("      cw_util -s module_name=<module_name> start_index=<start_index> data=<data> len=<len>\n");
    printf("                                     \n");
    printf(" Example: Get a single cw or multi cws                             \n");
    printf("      cw_util -g module_name=<module_name> start_index=<start_index> len=<len>\n");
    printf("                                     \n");
    printf(" Example: Query the uesed cws  of special module                             \n");
    printf("      cw_util -q module_name=<module_name>\n");
    printf("                                     \n");
    printf(" Example: Dump cw status                             \n");
    printf("      cw_util -d\n");
    printf("                                     \n");
    printf(" Example: print help                             \n");
    printf("      cw_util -h\n");
}

static int check_cmd_range(CW_CMD *p_cmd)
{
    if((p_cmd->id == CW_CMD_REQUEST_CWS ||
        p_cmd->id == CW_CMD_RELEASE_CWS ||
        p_cmd->id == CW_CMD_RELEASE_CWS_BY_MODULE ||
        p_cmd->id == CW_CMD_SET_CWS ||
        p_cmd->id == CW_CMD_GET_CWS ||
        p_cmd->id == CW_CMD_QUERY_CWS_BY_MODULE) &&
        p_cmd->module_name == NULL) {
        printf("The parameters is not right\n");
        return -1;
    }
    if(p_cmd->id == CW_CMD_SET_CWS && p_cmd->data == NULL) {
        printf("The parameters is not right\n");
        return -1;
    }
    return 0;
}
/*
******************************************************************************
 * FUNC : do_command
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static int do_command(CW_CMD *p_cmd)
{
    if (check_cmd_range(p_cmd) < 0)
        return 0;
    switch (p_cmd->id) {
        case CW_CMD_REQUEST_CWS:
            do_cw_request_cws(p_cmd->module_name, p_cmd->start_index, p_cmd->len);
            break;
        case CW_CMD_RELEASE_CWS:
            do_cw_release_cws(p_cmd->module_name, p_cmd->start_index, p_cmd->len);
            break;
        case CW_CMD_RELEASE_CWS_BY_MODULE:
            do_cw_release_cws_by_module(p_cmd->module_name);
            break;
        case CW_CMD_SET_CWS:
            do_cw_set_cws(p_cmd->module_name, p_cmd->start_index, (UINT8 *)p_cmd->data, p_cmd->len);
            break;
        case CW_CMD_GET_CWS:
            do_cw_get_cws(p_cmd->module_name, p_cmd->start_index, p_cmd->len);
            break;
        case CW_CMD_QUERY_CWS_BY_MODULE:
            do_cw_query_used_cw_by_module(p_cmd->module_name);
            break;
        case CW_CMD_DUMP_CWS_STATUS:
            do_cw_dump_cw_status();
            break;
        case CW_CMD_HELP:
            do_help();
            break;
    }
    return 0;
}

/*
******************************************************************************
 * FUNC : do_command
 * DESC :
 * PARM :
 * RET  :
 *******************************************************************************/
static int parse_command(unsigned char idx, const char* str, CW_CMD *p_cmd)
{

    if(idx == 1) {
        if (strncmp(str, "-a", 2) == 0)
            p_cmd->id = CW_CMD_REQUEST_CWS;
        else if (strncmp(str, "-r", 2) == 0)
            p_cmd->id = CW_CMD_RELEASE_CWS;
        else if (strncmp(str, "-c", 2) == 0)
            p_cmd->id = CW_CMD_RELEASE_CWS_BY_MODULE;
        else if (strncmp(str, "-s", 2) == 0)
            p_cmd->id = CW_CMD_SET_CWS;
        else if (strncmp(str, "-g", 2) == 0)
            p_cmd->id = CW_CMD_GET_CWS;
        else if (strncmp(str, "-q", 2) == 0)
            p_cmd->id = CW_CMD_QUERY_CWS_BY_MODULE;
        else if (strncmp(str, "-d", 2) == 0)
            p_cmd->id = CW_CMD_DUMP_CWS_STATUS;
        else if (strncmp(str, "-h", 2) == 0)
            p_cmd->id = CW_CMD_HELP;
        else
            return -1;
    } else {
        if (strncmp(str, "module_name=", 12) == 0) {
            p_cmd->module_name = (char *)((unsigned int)str + 12);
        } else if (strncmp(str, "start_index=", 12) == 0) {
            if (str_to_long((char *)((unsigned int)str + 12), (long *)(&p_cmd->start_index), 10) == -1) {
                printf("invalid start index- %s\n", str);
                return -1;
            }
        } else if (strncmp(str, "len=", 4) == 0) {
            if (str_to_long((char *)((unsigned int)str + 4), (long *)(&p_cmd->len), 10) == -1) {
                printf("invalid len - %s\n", str);
                return -1;
            }
        } else if (strncmp(str, "data=", 5) == 0) {
            p_cmd->data = (char *)((unsigned int)str + 5);
        } else {
            printf("unknown parameter - %s\n", str);
            return -1;
        }
    }
    return 0;
}

static int do_cw(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{

    CW_CMD cmd = {
        .id    = CW_CMD_HELP,
        .flags = 0,
        .start_index = 0,
        .len = 1,
        .module_name = NULL,
        .data = NULL,
    };

    int i;
    if(!g_cw_func_int) {
        rtk_cw_module_init();
        g_cw_func_int = 1;
    }

    if (argc > 1) {
        for (i = 1; i < argc ; i++) {
            if (parse_command(i, argv[i], &cmd) < 0) {
                cmd.id = CW_CMD_HELP;
                break;
            }
        }
    }

    do_command(&cmd);
    return 1;
}
U_BOOT_CMD(cw_util, 6, 0, do_cw,
           "Realtek CW Tool",
           "Format : cw_util <opcode> module_name=<module_name> start_index=<start_index> data=<data> len=<len>\n"
           "<opcode> : operations     \n"
           " -d  : dump cw status             \n"
           " -a  : request one or more cws          \n"
           " -r   : release one or cws          \n"
           " -c  : release cws of special module       \n"
           " -s  : set the value one or more cws    \n"
           " -g  : get the value one or more cws              \n"
           " -q  : query the uesed cws  of special module               \n"
           " -d  : query the uesed cws status of all modules              \n"
           "                                     \n"
           " Example: Request a single cw or multi cws                             \n"
           " 	cw_util -a module_name=<module_name> start_index=<start_index> len=<len>\n"
           " Example: Release a single cw or multi cws                             \n"
           "      cw_util -r module_name=<module_name> start_index=<start_index> len=<len>\n"
           " Example: Release all cws of special module                             \n"
           "      cw_util -c module_name=<module_name>\n"
           " Example: Set a single cw or multi cws                             \n"
           "      cw_util -s module_name=<module_name> start_index=<start_index> data=<data> len=<len>\n"
           " Example: Get a single cw or multi cws                             \n"
           "      cw_util -g module_name=<module_name> start_index=<start_index> len=<len>\n"
           " Example: Query the uesed cws  of special module                             \n"
           "      cw_util -q module_name=<module_name>\n"
           " Example: Dump cw status                             \n"
           "     cw_util -d\n"
           " Example: print help                             \n"
           "      cw_util -h\n"
          );
