#include <drivers/mcp/rtk_mcp_rsa_hw_fun.h>
#include <common.h>
#include <command.h>
#include <asm/arch/system.h>
#include <rbus/cp_reg.h>

#define REG_RSA_X       CP_RSA_X_0_reg
#define REG_RSA_Y       CP_RSA_Y_0_reg
#define REG_RSA_M       CP_RSA_M_0_reg
#define REG_RSA_A       CP_RSA_A_0_reg
#define REG_RSA_CTRL    CP_RSA_CTRL_reg
#define REG_RSA_NUM     CP_RSA_NUM_reg
#define REG_RSA_CLEAR   CP_RSA_CLEAR_reg
#define RSA_go          CP_RSA_CTRL_rsa_go_mask
#define RSA_X2Y         CP_RSA_CTRL_rsa_xeqy_mask
#define RSA_A2X         CP_RSA_CTRL_rsa_xeqa_mask

#define RSA_CLEAR_A     CP_RSA_CLEAR_rsa_clear_a_mask
#define RSA_CLEAR_M     CP_RSA_CLEAR_rsa_clear_m_mask
#define RSA_CLEAR_Y     CP_RSA_CLEAR_rsa_clear_y_mask
#define RSA_CLEAR_X     CP_RSA_CLEAR_rsa_clear_x_mask

#define RSA_CLEAR_ALL   (RSA_CLEAR_A | RSA_CLEAR_M | RSA_CLEAR_Y | RSA_CLEAR_X)

extern int BI_chkmsbbit (BI *bi);
extern int BI_is_bit_set (BI *bi, int n);

//-----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//For HW RSA utility function===========================================
//-----------------------------------------------------------------------------
//----------------------------------------------------------------------------
#define TYPE_HW_MONT	0
#define TYPE_HW_IMONT	1
static void hwsu_get_data (void *addr, BI *bi, UINT8 type)
{
	int i, j, max_idx;
	unsigned int *reg = (unsigned int *) addr;
	unsigned int *regM = (unsigned int *) (addr + 0x800);

	if (type == TYPE_HW_MONT)
		max_idx = 95; //Max: 96 * 32 = 3072 bit
	else
		max_idx = 12; //Max: 13 * 32 = 416 bit

	/* Skip MSB zero */
	i = max_idx;

	while (1) {
		//printf("i=%d\n",i);
		if (i < 64)
			if (i && (rtd_inl((unsigned int)(reg + i)) == 0))
				i--;
			else
				break;
		else
			if (i && (rtd_inl((unsigned int)(regM + i - 64)) == 0))
				i--;
			else
				break;
	}

	/* Fill BI */
	bi->m_nLength = i + 1;
	j = 1;
	for (; i >= 0; i--) {
		if (i < 64)
			bi->m_ulValue[bi->m_nLength - j] = rtd_inl((unsigned int)(reg + i));
		else
			bi->m_ulValue[bi->m_nLength - j] = rtd_inl((unsigned int)(regM + i - 64));
		j++;
	}

	return;
}
//-----------------------------------------------------------------------------
static void hwsu_set_data (void *addr, BI *bi, UINT8 type)
{
	int i;//, max_idx;
	unsigned int *reg = (unsigned int *) addr;
	unsigned int *regM = (unsigned int *) (addr + 0x800);

	//if (type == TYPE_HW_MONT)
	//	max_idx = 63; //Max: 64 * 32 = 2048 bit
	//else
	//	max_idx = 12; //Max: 13 * 32 = 416 bit
	for (i = 0; i < bi->m_nLength; i++) {
		if (i < 64)
			rtd_outl((unsigned int)(reg + i), bi->m_ulValue[i]);
		else
			rtd_outl((unsigned int)(regM + i - 64), bi->m_ulValue[i]);
	}
	/* Handle rest bytes (clear to 0) */
	//for (; i <= max_idx; i++)
	//	rtd_outl((unsigned int)(reg + i), 0x0);

	return;
}
//-----------------------------------------------------------------------------
static void hwsu_set_bitnum (unsigned int bsize)
{
	rtd_outl((REG_RSA_NUM), (bsize >> 2) - 1);
}
//-----------------------------------------------------------------------------
static void hwsu_chk_mont_engine_ready (void)
{
	while (rtd_inl(REG_RSA_CTRL) & RSA_go);
}
//-----------------------------------------------------------------------------
static void hwsu_mont_engine_go (void)
{
	rtd_outl((REG_RSA_CTRL), RSA_go);
}
//-----------------------------------------------------------------------------
static void hwsu_mont (BI *resl, BI *a, BI *b, BI *m, unsigned int bsize)
{

	hwsu_chk_mont_engine_ready();

    rtd_outl(REG_RSA_CLEAR, RSA_CLEAR_ALL);

	hwsu_set_data((void *)REG_RSA_X, a, TYPE_HW_MONT);
	hwsu_set_data((void *)REG_RSA_Y, b, TYPE_HW_MONT);
	hwsu_set_data((void *)REG_RSA_M, m, TYPE_HW_MONT);

	hwsu_set_bitnum(bsize);

	hwsu_mont_engine_go();

	hwsu_chk_mont_engine_ready();


	hwsu_get_data((void *)REG_RSA_A, resl, TYPE_HW_MONT);

	return;
}
#if 0
extern void mcp_dump_mem(
    unsigned char*      data,
    unsigned int            len );

static unsigned char data[512];
void mcp_dump_bi(BI* bi, char* msg)
{
    //return;
    memset(data, 0, 512);
    printf("%s len = %d\n", msg, bi->m_nLength);
    OutPutToAddr(bi,data,1);
    mcp_dump_mem(data, bi->m_nLength * 4);
}

#define DUMP_BI(x) mcp_dump_bi(x, #x)
#endif
//-----------------------------------------------------------------------------
#if 0
static void hwsu_mul_mod (BI *resl, BI *r2, BI *x, BI *y, BI *m, UINT32 bsize)
{
	hwsu_mont (resl, x, y, m, bsize);
	hwsu_mont (resl, resl, r2, m, bsize);

	return;
}
#endif
//resl = base ^ exp % mod
void hwsu_mont_exp(BI *resl, BI *r2, BI *base, BI *exp, BI *mod, UINT32 bsize)
{
    int i = 0;
    BI * temp_1 = move_p(1);
    //resl = move_p(1);
    
    hwsu_mont(resl,resl,r2,mod,bsize);
    hwsu_mont(base,base,r2,mod,bsize);

    for ( i = BI_chkmsbbit (exp); i >= 0; i-- )
    {
        hwsu_mont(resl,resl,resl,mod,bsize);
        if ( BI_is_bit_set (exp, i) == 1 )
        {
            hwsu_mont(resl,resl,base,mod,bsize);
        }
    }
    hwsu_mont(resl,resl,temp_1,mod,bsize);
}

#ifdef RSA_HW_FAST_MODE
#define HW_SET_RSA_X(bi)  {hwsu_chk_mont_engine_ready();hwsu_set_data((void *)REG_RSA_X,bi,TYPE_HW_MONT);}
#define HW_SET_RSA_Y(bi)  {hwsu_chk_mont_engine_ready();hwsu_set_data((void *)REG_RSA_Y,bi,TYPE_HW_MONT);}
#define HW_SET_RSA_MOD(bi)  {hwsu_chk_mont_engine_ready();hwsu_set_data((void *)REG_RSA_M,bi,TYPE_HW_MONT);}
#define HW_GET_RSA_A(bi)  {hwsu_chk_mont_engine_ready();hwsu_get_data((void *)REG_RSA_A,bi,TYPE_HW_MONT);}


static inline void hwsu_mont_XX_X_go(void)
{
    rtd_outl((REG_RSA_CTRL), RSA_go | RSA_A2X | RSA_X2Y);
    hwsu_chk_mont_engine_ready();
}


static inline void hwsu_mont_XY_X_go(void)
{
    rtd_outl((REG_RSA_CTRL), RSA_go | RSA_A2X);
    hwsu_chk_mont_engine_ready();
}

static inline void hwsu_mont_XY_A_go(void)
{
    rtd_outl((REG_RSA_CTRL), RSA_go);
    hwsu_chk_mont_engine_ready();
}

void hwsu_mont_exp_fast(
    BI *resl, BI *r2, BI *base,
    BI *exp, BI *mod, unsigned int bsize)
{
    int i = 0;
    BI *temp_1 = move_p(1);
    //hwsu_mont(base,base,r2,mod,bsize);
    rtd_outl(REG_RSA_CLEAR, RSA_CLEAR_ALL);
    hwsu_mont(base, base, r2, mod, bsize);
    //hwsu_mont(resl,resl,r2,mod,bsize);
    rtd_outl(REG_RSA_CLEAR, RSA_CLEAR_X);
    HW_SET_RSA_X(resl);
    hwsu_mont_XY_A_go();
    rtd_outl(REG_RSA_CLEAR, RSA_CLEAR_Y);
    HW_SET_RSA_Y(base);
    for (i = BI_chkmsbbit(exp); i >= 0; i--)
    {
        //hwsu_mont(resl,resl,resl,mod,bsize);
        hwsu_mont_XX_X_go();
        if (BI_is_bit_set(exp, i) == 1)
        {
            //hwsu_mont(resl,resl,base,mod,bsize);
            hwsu_mont_XY_X_go();
        }
    }
    //hwsu_mont(resl,resl,temp_1,mod,bsize);
    rtd_outl(REG_RSA_CLEAR, RSA_CLEAR_Y);
    HW_SET_RSA_Y(temp_1);
    hwsu_mont_XY_X_go();
    HW_GET_RSA_A(resl);
}
#endif
//-----------------------------------------------------------------------------
//End of HW RSA utility funciton=========================================

