/*
 *  This is a driver for the eMMC controller found in Realtek
 *  SoCs.
 *
 *  Copyright (C) 2013 Realtek Semiconductors, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <common.h>
#include "watchdog_api.h"
#include <drivers/mmc/rtkmmc.h>
#include <drivers/otp/rtk_otp_region_api.h>

#define __RTKMMC_C__

/************************************************************************
 *  Local define
 ************************************************************************/
#define DRIVER_NAME "rtkemmc_dw"
#define CACHE_HANDLE_INNER
#define RE_INITIAL_IDMAC
#define USE_RBUS_RNG

#define cr_maskl(reg,msk,sht,val)  \
		cr_writel((cr_readl(reg) & ~(msk<<sht))|(val<<sht), reg)

#define BIT(x)			(1 << (x))
#define PAD_SET_CNT 	(7)

/************************************************************************
 *  External function prototype
 ************************************************************************/

/************************************************************************
 *  Local function prototype
 ************************************************************************/
unsigned int dw_em_set_bits(unsigned int bus_width);
static void dw_em_set_scrip_base(unsigned int scrip_base);
static int dw_em_set_PLL_clk(unsigned int type,unsigned int clk_div,u32 init);
static void dw_em_set_div(unsigned int em_div);
static int mmc_send_data_cmd(u32 hc_cmd,u32 cmd_arg, u32 blk_cnt,unsigned char *buffer);
static int dw_em_reinit_idmac_pre(struct rtk_cmd_info * cmd_info);
static int dw_em_get_cpu_occupy(struct rtk_cmd_info *cmd_info);
static int dw_em_release_cpu_occupy(struct rtk_cmd_info *cmd_info);
static int dw_em_wait_status(e_device_type* sdport, unsigned int* rsp_buffer, unsigned int max_try);
static void dw_em_set_ds_delay(int ds_delay);
void dw_em_send_dummy_clk(unsigned ms_time);
int rtk_eMMC_xfer_counter(e_device_type * card,int byte_cnt);
int dw_em_dis_sd_tune(void);

/************************************************************************
 *  External variable & struct
 ************************************************************************/
extern int mmc_had_been_initialized;
extern int mmc_ready_to_use;

unsigned int mmc_cmd23_alert;

/*******************************************************
 * Local variable & struct
 *******************************************************/
#if 0
static const char *const bit_tlb[4] = {
	"1bit",
	"4bits",
	"8bits",
	"unknow"
};
#endif
static const char *const dw_clk_tlb[23] = {
	"200Khz",
	"10Mhz",
	"15Mhz",
	"20Mhz",
	"25Mhz",
	"30Mhz",
	"40Mhz",
	"50Mhz",
	"60Mhz",
	"70Mhz",
	"80Mhz",
	"90Mhz",
	"100Mhz",
	"110Mhz",
	"120Mhz",
	"130Mhz",
	"140Mhz",
	"150Mhz",
	"160Mhz",
	"170Mhz",
	"180Mhz",
	"190Mhz",
	"200Mhz"
};

#define STA_EVENT_LEN 11
static const char *const sta_evevt_tlb[STA_EVENT_LEN] = {
	"IDLE",
	"READY",
	"IDENT",
	"STBY",
	"TRAN",
	"DATA",
	"RCV",
	"PRG",
	"DIS",
	"BTST",
	"SLP"
};

/* mmc spec definition */
const unsigned int tran_exp[] = {
	10000, 100000, 1000000, 10000000,
	0,     0,      0,       0
};

const unsigned char tran_mant[] = {
	0,  10, 12, 13, 15, 20, 25, 30,
	35, 40, 45, 50, 55, 60, 70, 80,
};

const unsigned int tacc_exp[] = {
	1, 10, 100, 1000, 10000, 100000, 1000000, 10000000,
};

const unsigned int tacc_mant[] = {
	0,  10, 12, 13, 15, 20, 25, 30,
	35, 40, 45, 50, 55, 60, 70, 80,
};

e_device_type emmc_card;

//unsigned char * ext_csd;
unsigned int  * dw_em_scrip;

#ifdef THIS_IS_FLASH_WRITE_U_ENV
static unsigned int rtkmmc_debug_msg;
static unsigned int rtkmmc_off_error_msg_in_init_flow;
#else
unsigned int rtkmmc_debug_msg;
unsigned int rtkmmc_off_error_msg_in_init_flow;
#endif

#ifdef CONFIG_SYS_SUPPORT_MULTICORE
#define RTK_EMMC_SUPPORT_MULTICORE
#endif

#ifdef RTK_EMMC_SUPPORT_MULTICORE
//#include "thread_info.h"
bool g_mmc_protect_enable = false;
#endif

/* #define ELAPSE_TIME_CACULATE */
#ifdef ELAPSE_TIME_CACULATE
unsigned int  mmc_C_xfer_us;
unsigned int  mmc_R_xfer_us;
unsigned int  mmc_W_xfer_us;
unsigned int  mmc_R_xfer_blk;
unsigned int  mmc_W_xfer_blk;
#endif

//#define RECORD_CMD_FLOW
#ifdef RECORD_CMD_FLOW
#define MAX_CMD_CNT	20
struct cmd_data
{
	unsigned int cmd;
	unsigned int arg;
};

struct cmd_data cmd_flow[MAX_CMD_CNT]={0};
#endif

/*******************************************************
 * Function body
 *******************************************************/

/************************************************************************
 *
 ************************************************************************/
#define MMC_IP_RESET_OTP
#ifdef MMC_IP_RESET_OTP
extern int otp_read(unsigned long ofst, unsigned long *pval);
extern int otp_write(unsigned long ofst, unsigned long val);

#define OTP_MMC_REG1 0x138
#define OTP_MMC_REG2 0x13c
#define OTP_MMC_VAL1 0x00004244
#define OTP_MMC_VAL2 0x00070000
void rtkmmc_otp(void)
{
    unsigned long reginfo1;
    unsigned long reginfo2;

    otp_read(OTP_MMC_REG1,&reginfo1);
    otp_read(OTP_MMC_REG2,&reginfo2);

    if(!reginfo1 && !reginfo2 ){
        //printf("There are no mmc OTP solution, apply it!!!\n");
        otp_write(OTP_MMC_REG1, OTP_MMC_VAL1);
        otp_write(OTP_MMC_REG2, OTP_MMC_VAL2);
    }else{
        //printf("There are mmc OTP solution.\n");
    }
}
#else
#define rtkmmc_otp()
#endif

/************************************************************************
 *
 ************************************************************************/
#ifdef RECORD_CMD_FLOW
static
void rtk_record_cmd_flow(unsigned int cmd,unsigned int arg)
{
	int i;

	i = MAX_CMD_CNT;
	while(--i)
	{
		cmd_flow[i].cmd = cmd_flow[i-1].cmd;
		cmd_flow[i].arg = cmd_flow[i-1].arg;
	}
	cmd_flow[0].cmd = cmd;
	cmd_flow[0].arg = arg;

}
#else
#define rtk_record_cmd_flow(x,y)
#endif

/************************************************************************
 *
 ************************************************************************/
#ifdef RECORD_CMD_FLOW
static
void rtk_display_cmd_flow(void)
{
	int i;

	i = MAX_CMD_CNT;
	printf("cmd flow\n");
	while(i--){
		printf("[cmd%02d][0x%08x]\n",cmd_flow[i].cmd,cmd_flow[i].arg);
	}
}
#else
#define rtk_display_cmd_flow()
#endif

/************************************************************************
 * Notice to reduce speed when the error occurs,
   the current speed is HS200 or HS400
 ************************************************************************/
static
void notice_reduce_speed(unsigned int cmd_idx)
{
	u32 timing;
	e_device_type* sdport = &emmc_card;
	static unsigned int err_cnt = 0;
	unsigned int err_tolerance = 15;

	if(++err_cnt >= err_tolerance){
		timing = sdport->mmc->timing;
		if((timing == MMC_TIMING_MMC_HS200) || (timing == MMC_TIMING_MMC_HS400)){
				UPRINTF("\n===========================================================\n"
						"        Error occurred\n"
						"        Please try to change the eMMC speed to HS50\n"
						"        Modify configure file, rebuild uboot\n"
						"        bootcode/bin/XXX/RTDXXXX.inc\n"
						"        ##Boot Flash Type is [ HS400 | HS200 | NORMAL ]\n"
						"        Config_emmc_timing_is = NORMAL\n"
						"===========================================================\n");
		}
	}
}

/************************************************************************
 *
 ************************************************************************/
#ifdef  USE_RBUS_RNG
#define RNG_OUT_RDY     0xb800182C	//RNG_READY_REG
#define RNG_RAN_OUT     0xb8001838	//RNG_RANDOM_OUT_REG
#define OUT_RDY         (0x01UL)
unsigned int rtk_get_rng(void)
{
    unsigned int limit = 0x10000;
    unsigned int rtn;

    while(((cr_readl(RNG_OUT_RDY) & OUT_RDY) != OUT_RDY) && limit ){
            udelay(100);
            limit--;
    }
    if(limit == 0){
        UPRINTF("random generator time out!!\n");
        rtn = 0xffffffff;
    }else{
        rtn = cr_readl(RNG_RAN_OUT);
    }

    return rtn;
}
#else

#define rtk_get_rng() 0

#endif

/************************************************************************
 *
 ************************************************************************/
static
void mmc_print_ext_csd(char *ext_csd)
{
/* #define RTKEMMC_SHOW_EXTCSD */
#ifdef RTKEMMC_SHOW_EXTCSD
	printf("====== EXT_CSD TABLE ======\n");
	printf("[ 32]FLUSH_CACHE               = 0x%02x\n",ext_csd[EXT_CSD_FLUSH_CACHE]);
	printf("[ 33]CACHE_CTRL                = 0x%02x\n",ext_csd[EXT_CSD_CACHE_CTRL]);
	printf("[ 34]POWER_OFF_NOTIFICATION    = 0x%02x\n",ext_csd[EXT_CSD_POWER_OFF_NOTIFICATION]);
	printf("[ 54]EXP_EVENTS_STATUS         = 0x%02x\n",ext_csd[EXT_CSD_EXP_EVENTS_STATUS]);
	printf("[ 61]DATA_SECTOR_SIZE          = 0x%02x\n",ext_csd[EXT_CSD_DATA_SECTOR_SIZE]);
	printf("[136]ENH_START_ADDR[0]         = 0x%02x\n",ext_csd[EXT_CSD_ENH_START_ADDR]);
	printf("[137]ENH_START_ADDR[1]         = 0x%02x\n",ext_csd[EXT_CSD_ENH_START_ADDR+1]);
	printf("[138]ENH_START_ADDR[2]         = 0x%02x\n",ext_csd[EXT_CSD_ENH_START_ADDR+2]);
	printf("[139]ENH_START_ADDR[3]         = 0x%02x\n",ext_csd[EXT_CSD_ENH_START_ADDR+3]);
	printf("[140]ENH_SIZE_MULT[0]          = 0x%02x\n",ext_csd[EXT_CSD_ENH_SIZE_MULT+0]);
	printf("[141]ENH_SIZE_MULT[1]          = 0x%02x\n",ext_csd[EXT_CSD_ENH_SIZE_MULT+1]);
	printf("[142]ENH_SIZE_MULT[2]          = 0x%02x\n",ext_csd[EXT_CSD_ENH_SIZE_MULT+2]);
	printf("[143]GP_SIZE_MULT_1[0]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+0]);
	printf("[144]GP_SIZE_MULT_1[1]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+1]);
	printf("[145]GP_SIZE_MULT_1[2]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+2]);
	printf("[146]GP_SIZE_MULT_2[0]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+3]);
	printf("[147]GP_SIZE_MULT_2[1]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+4]);
	printf("[148]GP_SIZE_MULT_2[2]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+5]);
	printf("[149]GP_SIZE_MULT_3[0]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+6]);
	printf("[150]GP_SIZE_MULT_3[1]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+7]);
	printf("[151]GP_SIZE_MULT_3[2]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+8]);
	printf("[152]GP_SIZE_MULT_4[0]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+9]);
	printf("[153]GP_SIZE_MULT_4[1]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+10]);
	printf("[154]GP_SIZE_MULT_4[2]         = 0x%02x\n",ext_csd[EXT_CSD_GP_SIZE_MULT+11]);
	printf("[155]PARTITION_SETTING_COMP    = 0x%02x\n",ext_csd[EXT_CSD_PARTITION_SETTING_COMP]);
	printf("[156]PARTITION_ATTRIBUTE       = 0x%02x\n",ext_csd[EXT_CSD_PARTITION_ATTRIBUTE]);
	printf("[157]MAX_ENH_SIZE_MULT[0]      = 0x%02x\n",ext_csd[EXT_CSD_MAX_ENH_SIZE_MULT]);
	printf("[158]MAX_ENH_SIZE_MULT[1]      = 0x%02x\n",ext_csd[EXT_CSD_MAX_ENH_SIZE_MULT+1]);
	printf("[159]MAX_ENH_SIZE_MULT[2]      = 0x%02x\n",ext_csd[EXT_CSD_MAX_ENH_SIZE_MULT+2]);
	printf("[160]PARTITION_SUPPORT         = 0x%02x\n",ext_csd[EXT_CSD_PARTITION_SUPPORT]);
	printf("[161]HPI_MGMT                  = 0x%02x\n",ext_csd[EXT_CSD_HPI_MGMT]);
	printf("[162]RST_N_FUNCTION            = 0x%02x\n",ext_csd[EXT_CSD_RST_N_FUNCTION]);
	printf("[163]BKOPS_EN                  = 0x%02x\n",ext_csd[EXT_CSD_BKOPS_EN]);
	printf("[164]BKOPS_START               = 0x%02x\n",ext_csd[EXT_CSD_BKOPS_START]);
	printf("[165]SANITIZE_START            = 0x%02x\n",ext_csd[EXT_CSD_SANITIZE_START          ]);
	printf("[166]WR_REL_PARAM              = 0x%02x\n",ext_csd[EXT_CSD_WR_REL_PARAM]);
	printf("[167]WR_REL_SET                = 0x%02x\n",ext_csd[EXT_CSD_WR_REL_SET]);
	printf("[173]BOOT_WP                   = 0x%02x\n",ext_csd[EXT_CSD_BOOT_WP                 ]);
	printf("[175]ERASE_GROUP_DEF           = 0x%02x\n",ext_csd[EXT_CSD_ERASE_GROUP_DEF         ]);
	printf("[179]PART_CONFIG               = 0x%02x\n",ext_csd[EXT_CSD_PART_CONFIG             ]);
	printf("[181]ERASED_MEM_CONT           = 0x%02x\n",ext_csd[EXT_CSD_ERASED_MEM_CONT         ]);
	printf("[183]BUS_WIDTH                 = 0x%02x\n",ext_csd[EXT_CSD_BUS_WIDTH               ]);
	printf("[185]HS_TIMING                 = 0x%02x\n",ext_csd[EXT_CSD_HS_TIMING               ]);
	printf("[187]POWER_CLASS               = 0x%02x\n",ext_csd[EXT_CSD_POWER_CLASS             ]);
	printf("[191]CMD_SET                   = 0x%02x\n",ext_csd[EXT_CSD_CMD_SET]);
	printf("[192]REV                       = 0x%02x\n",ext_csd[EXT_CSD_REV]);
	printf("[194]STRUCTURE                 = 0x%02x\n",ext_csd[EXT_CSD_STRUCTURE               ]);
	printf("[196]CARD_TYPE                 = 0x%02x\n",ext_csd[EXT_CSD_CARD_TYPE               ]);
	printf("[198]OUT_OF_INTERRUPT_TIME     = 0x%02x\n",ext_csd[EXT_CSD_OUT_OF_INTERRUPT_TIME   ]);
	printf("[199]PART_SWITCH_TIME          = 0x%02x\n",ext_csd[EXT_CSD_PART_SWITCH_TIME        ]);
	printf("[200]PWR_CL_52_195             = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_52_195           ]);
	printf("[201]PWR_CL_26_195             = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_26_195           ]);
	printf("[202]PWR_CL_52_360             = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_52_360           ]);
	printf("[203]PWR_CL_26_360             = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_26_360           ]);
	printf("[212]SEC_CNT[0]                = 0x%02x\n",ext_csd[EXT_CSD_SEC_CNT]);
	printf("[213]SEC_CNT[1]                = 0x%02x\n",ext_csd[EXT_CSD_SEC_CNT+1]);
	printf("[214]SEC_CNT[2]                = 0x%02x\n",ext_csd[EXT_CSD_SEC_CNT+2]);
	printf("[215]SEC_CNT[3]                = 0x%02x\n",ext_csd[EXT_CSD_SEC_CNT+3]);
	printf("[217]S_A_TIMEOUT               = 0x%02x\n",ext_csd[EXT_CSD_S_A_TIMEOUT             ]);
	printf("[221]HC_WP_GRP_SIZE            = 0x%02x\n",ext_csd[EXT_CSD_HC_WP_GRP_SIZE          ]);
	printf("[222]REL_WR_SEC_C              = 0x%02x\n",ext_csd[EXT_CSD_REL_WR_SEC_C            ]);
	printf("[223]ERASE_TIMEOUT_MULT        = 0x%02x\n",ext_csd[EXT_CSD_ERASE_TIMEOUT_MULT      ]);
	printf("[224]HC_ERASE_GRP_SIZE         = 0x%02x\n",ext_csd[EXT_CSD_HC_ERASE_GRP_SIZE       ]);
	printf("[225]ACC_SIZE                  = 0x%02x\n",ext_csd[EXT_CSD_ACC_SIZE                ]);
	printf("[226]BOOT_MULT                 = 0x%02x\n",ext_csd[EXT_CSD_BOOT_MULT               ]);
	printf("[228]BOOT_INFO                 = 0x%02x\n",ext_csd[EXT_CSD_BOOT_INFO               ]);
	printf("[229]SEC_TRIM_MULT             = 0x%02x\n",ext_csd[EXT_CSD_SEC_TRIM_MULT           ]);
	printf("[230]SEC_ERASE_MULT            = 0x%02x\n",ext_csd[EXT_CSD_SEC_ERASE_MULT          ]);
	printf("[231]SEC_FEATURE_SUPPORT       = 0x%02x\n",ext_csd[EXT_CSD_SEC_FEATURE_SUPPORT     ]);
	printf("[232]TRIM_MULT                 = 0x%02x\n",ext_csd[EXT_CSD_TRIM_MULT               ]);
	printf("[234]MIN_PERF_DDR_R_8_52       = 0x%02x\n",ext_csd[EXT_CSD_MIN_PERF_DDR_R_8_52     ]);
	printf("[235]MIN_PERF_DDR_W_8_52       = 0x%02x\n",ext_csd[EXT_CSD_MIN_PERF_DDR_W_8_52     ]);
	printf("[236]PWR_CL_200_195            = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_200_195          ]);
	printf("[237]PWR_CL_200_360            = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_200_360          ]);
	printf("[238]PWR_CL_DDR_52_195         = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_DDR_52_195       ]);
	printf("[239]PWR_CL_DDR_52_360         = 0x%02x\n",ext_csd[EXT_CSD_PWR_CL_DDR_52_360       ]);
	printf("[241]INI_TIMEOUT_AP            = 0x%02x\n",ext_csd[EXT_CSD_INI_TIMEOUT_AP          ]);
	printf("[242]COR_PRG_SECTORS_NUM[0]    = 0x%02x\n",ext_csd[EXT_CSD_CORRECTLY_PRG_SECTORS_NUM]);
	printf("[243]COR_PRG_SECTORS_NUM[1]    = 0x%02x\n",ext_csd[EXT_CSD_CORRECTLY_PRG_SECTORS_NUM+1]);
	printf("[244]COR_PRG_SECTORS_NUM[2]    = 0x%02x\n",ext_csd[EXT_CSD_CORRECTLY_PRG_SECTORS_NUM+2]);
	printf("[245]COR_PRG_SECTORS_NUM[3]    = 0x%02x\n",ext_csd[EXT_CSD_CORRECTLY_PRG_SECTORS_NUM+3]);
	printf("[246]BKOPS_STATUS              = 0x%02x\n",ext_csd[EXT_CSD_BKOPS_STATUS            ]);
	printf("[247]POWER_OFF_LONG_TIME       = 0x%02x\n",ext_csd[EXT_CSD_POWER_OFF_LONG_TIME     ]);
	printf("[248]GENERIC_CMD6_TIME         = 0x%02x\n",ext_csd[EXT_CSD_GENERIC_CMD6_TIME       ]);
	printf("[249]CACHE_SIZE[0]             = 0x%02x\n",ext_csd[EXT_CSD_CACHE_SIZE              ]);
	printf("[250]CACHE_SIZE[1]             = 0x%02x\n",ext_csd[EXT_CSD_CACHE_SIZE+1]);
	printf("[251]CACHE_SIZE[2]             = 0x%02x\n",ext_csd[EXT_CSD_CACHE_SIZE+2]);
	printf("[252]CACHE_SIZE[3]             = 0x%02x\n",ext_csd[EXT_CSD_CACHE_SIZE+3]);
	printf("[494]EXT_SUPPORT               = 0x%02x\n",ext_csd[EXT_CSD_EXT_SUPPORT             ]);
	printf("[495]LARGE_UNIT_SIZE_M1        = 0x%02x\n",ext_csd[EXT_CSD_LARGE_UNIT_SIZE_M1      ]);
	printf("[496]CONTEXT_CAPABILITIES      = 0x%02x\n",ext_csd[EXT_CSD_CONTEXT_CAPABILITIES    ]);
	printf("[497]TAG_RES_SIZE              = 0x%02x\n",ext_csd[EXT_CSD_TAG_RES_SIZE            ]);
	printf("[498]TAG_UNIT_SIZE             = 0x%02x\n",ext_csd[EXT_CSD_TAG_UNIT_SIZE           ]);
	printf("[499]DATA_TAG_SUPPORT          = 0x%02x\n",ext_csd[EXT_CSD_DATA_TAG_SUPPORT        ]);
	printf("[500]MAX_PACKED_WRITES         = 0x%02x\n",ext_csd[EXT_CSD_MAX_PACKED_WRITES       ]);
	printf("[501]MAX_PACKED_READS          = 0x%02x\n",ext_csd[EXT_CSD_MAX_PACKED_READS        ]);
	printf("[502]BKOPS_SUPPORT             = 0x%02x\n",ext_csd[EXT_CSD_BKOPS_SUPPORT           ]);
	printf("[503]HPI_FEATURES              = 0x%02x\n",ext_csd[EXT_CSD_HPI_FEATURES            ]);
	printf("[504]S_CMD_SET                 = 0x%02x\n",ext_csd[EXT_CSD_S_CMD_SET               ]);
	printf("[505]EXT_SECURITY_ERR          = 0x%02x\n",ext_csd[EXT_CSD_EXT_SECURITY_ERR        ]);
#endif
}

void export_mmc_print_ext_csd(char *ext_csd){
	mmc_print_ext_csd(ext_csd);
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_get_cpu_occupy(struct rtk_cmd_info *cmd_info)
{
	//e_device_type* sdport = &emmc_card;
#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	unsigned int relock = 0; //[CID-58148]Logically dead code
#endif

#if defined(Config_Process_DTE_KBoot_TRUE)
	get_flash_shared_semaphore();
#endif

#if defined(RTK_EMMC_SUPPORT_MULTICORE)

	unsigned int flags;
	spin_lock_t g_mmc_lock = INIT_SPIN_LOCK;

RE_LOCK:
	spin_lock_save(&g_mmc_lock, flags);

#endif

	if(cr_readl(EM_SCPU_SEL) & PS_SCPU_SEL){
#if defined(RTK_EMMC_SUPPORT_MULTICORE)
		relock = 1;
#else
		printf("%s(%d)Warning call  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n",
			__func__,__LINE__);
#endif
	}else{
		cr_writel( PS_SCPU_SEL, EM_SCPU_SEL );
	}

#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	spin_unlock_restore(&g_mmc_lock, flags);

	//[CID-58148]Logically dead code: relock=0
	if(relock){
		relock = 0;
		udelay(5);
		mmcinfo("%s(%d)waitting <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n",
			__func__,__LINE__);
		goto RE_LOCK;
	}
#endif
	return 0;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_release_cpu_occupy(struct rtk_cmd_info *cmd_info)
{
	//e_device_type* sdport = &emmc_card;

#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	unsigned int flags;
	spin_lock_t g_mmc_lock = INIT_SPIN_LOCK;

	spin_lock_save(&g_mmc_lock, flags);
#endif

	if(cr_readl(EM_SCPU_SEL) & PS_SCPU_SEL){
		cr_writel(0 , EM_SCPU_SEL );	/* release SCPU */
	}else{
		/* if crt reset by error occur,the scpu_sel clear as well.*/
		//printf("%s(%d)Warning call >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n",
		//	__func__,__LINE__);
	}

#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	spin_unlock_restore(&g_mmc_lock, flags);
#endif

#if defined(Config_Process_DTE_KBoot_TRUE)
	release_flash_shared_semaphore();
#endif
	return 0;

}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_read_rsp( unsigned int * rsp, unsigned int reg_count )
{
	if ( reg_count == 16 ){
		rsp[3] = cr_readl(EM_DWC_RESP0);
		rsp[2] = cr_readl(EM_DWC_RESP1);
		rsp[1] = cr_readl(EM_DWC_RESP2);
		rsp[0] = cr_readl(EM_DWC_RESP3);
#ifdef SHOW_RESP
		UPRINTF("rsp[0]=0x%08x\n", rsp[0]);
		UPRINTF("rsp[1]=0x%08x\n", rsp[1]);
		UPRINTF("rsp[2]=0x%08x\n", rsp[2]);
		UPRINTF("rsp[3]=0x%08x\n", rsp[3]);
#endif
	}else{
		rsp[0] = cr_readl(EM_DWC_RESP0);
#ifdef SHOW_RESP
		UPRINTF("rsp[0]=0x%08x\n", rsp[0]);
#endif
		if ( reg_count == 2 ){
			/*
			When CIU sends auto-stop command,
			then response is saved in Response 0 register.
			Response for previous command sent by host is
			still preserved in Response 0 register.
			*/

			rsp[1] = cr_readl(EM_DWC_RESP1);
#ifdef SHOW_RESP
			UPRINTF("rsp[0]=0x%08x\n",rsp[0]);
#endif
		}
	}
}

/************************************************************************
 *
 ************************************************************************/
/* all error bit define at "Device Status". */
#define RESP_ERR_BIT     (  _BIT31|_BIT30|_BIT29|_BIT28|   \
							_BIT27|_BIT26|_BIT24|_BIT23|   \
							_BIT22|_BIT21|_BIT20|_BIT19|   \
							_BIT16|_BIT15|_BIT13|_BIT7  )

/* fatal error, if these bits alert the command should skip retry. not figure out yet. */
#define RESP_ERR_FATAL   (  _BIT31|_BIT30|_BIT29|_BIT28|   \
							_BIT27|_BIT26|_BIT24|_BIT23|   \
							_BIT22|_BIT21|_BIT20|_BIT19|   \
							_BIT16|_BIT15|_BIT13|_BIT7  )

/* X mode error */
#define RESP_X_MODD   ( _BIT31|_BIT30|_BIT27|_BIT26|_BIT24|_BIT23|	\
			_BIT22|_BIT21|_BIT19|_BIT16|_BIT15|_BIT7 )

static const char *const sta_err_bit_tlb[32] = {
	"","","","","","","",           //0~6
	"SWITCH_ERROR[X] ",             //7
	"","","","","",                 //8~12
	"ERASE_RESET ",                 //13
	"",                             //14
	"WP_ERASE_SKIP[X] ",            //15
	"CID/CSD_OVERWRITE[X] ",        //16
	"","",                          //17~18
	"ERROR[X] ",                    //19
	"CC_ERROR ",                    //20
	"DEVICE_ECC_FAILED[X] ",        //21
	"ILLEGAL_COMMAND[Rx] ",         //22
	"COM_CRC_ERROR[Rx] ",           //23
	"LOCK_UNLOCK_FAILED[X] ",       //24
	"",                             //25
	"WP_VIOLATION[X] ",             //26
	"ERASE_PARAM[X] ",              //27
	"ERASE_SEQ_ERROR ",             //28
	"BLOCK_LEN_ERROR ",             //29
	"ADDRESS_MISALIGN[X] ",         //30
	"ADDRESS_OUT_OF_RANGE[X] "      //31
};

/************************************************************************
 * Note:
 *   [X] denote this error is X mode error bit.
 *   It's detected at pervious cmd.
 ************************************************************************/
static
int dw_em_show_rsp_err_reason(u32 err_reason)
{
	unsigned int err_bit;
	unsigned int i;

	err_bit = (err_reason & RESP_ERR_BIT);
	if(err_bit){
		UPRINTF("mmc device status : ");
		for(i=0; i<32; i++){
			if((err_bit >> i) & 0x01U){
				UPRINTF("%s",sta_err_bit_tlb[i]);
			}
		}
		UPRINTF("\n");
	}

	return 0;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_chk_rsp(u32 * rsp, u32 cmd_flag, struct rtk_cmd_info * cmd_info)
{
	e_device_type* sdport = &emmc_card;
	int ret_err = 0;
	u32 cmd_idx = 0;

	cmd_idx = SDMMC_CMD_INDX(cmd_flag);

	if(cmd_flag & SDMMC_CMD_RESP_LONG){
		dw_em_read_rsp(rsp, 16);
		/* only resp == 6 is possible have card status */
	}else{
		if(cmd_flag & SDMMC_CMD_SEND_STOP)
			dw_em_read_rsp(rsp, 2);
		else
			dw_em_read_rsp(rsp, 6);

		if((cmd_flag & SDMMC_CMD_RESP_CRC)){
			if( (cmd_idx != MMC_GO_IRQ_STATE)){
				/*
				 Only R1/R1b should check Card_Status.
				 It can't diff R1/R1b/R5/R6/R7.
				 but only CMD40 have definded, so check it here.
				*/
				sdport->dev_status = rsp[0];
				if(rsp[0] & RESP_ERR_BIT)
					ret_err = _BIT0;

				if(cmd_flag & SDMMC_CMD_SEND_STOP){
					sdport->pre_status = rsp[1];
					if(rsp[1] & RESP_ERR_BIT)
						ret_err |= _BIT1;
				}
			}
		}
	}
	if(ret_err){
		/* response will be check by high-level, print information just for debug. */
		UPRINTF("rsp error !!![cmd%d]\n",cmd_idx);
		UPRINTF("rsp[0]=0x%08x\n",rsp[0]);
		dw_em_show_rsp_err_reason(rsp[0]);
		if(cmd_flag & SDMMC_CMD_SEND_STOP){
			UPRINTF("rsp[1]=0x%08x\n",rsp[1]);
			dw_em_show_rsp_err_reason(rsp[1]);
		}

	}

	return ret_err;
}

/************************************************************************
 *
 ************************************************************************/
/* default phase */
#define DEFAULT_R_PHASE     (8)
#define DEFAULT_C_PHASE     (18)
#define DEFAULT_W_PHASE     (18)

/* TV001 */
#define DEFAULT_C_PHASE_200 (18)
#define DEFAULT_W_PHASE_200 (13)
#define DEFAULT_W_PHASE_400 (10)
#define DEFAULT_DSTUNE      (12)

static
int dw_em_init_phase(u32 w_phase,u32 r_phase,u32 c_phase)
{
	e_device_type* sdport = &emmc_card;
	mmcinfo("w_phase=%u r_phase=%u\n",w_phase,r_phase);

	if(w_phase != 0xff){
		sdport->cur_w_phase = w_phase;
		cr_maskl(EMMC_PLLPHASE_CTRL,0x03fUL, 0 ,w_phase);
	}

	if(r_phase != 0xff){
		sdport->cur_r_phase = r_phase;
		cr_maskl(EMMC_PLLPHASE_CTRL,0x03fUL,10 ,r_phase);
	}

	if(c_phase != 0xff){
		sdport->cur_c_phase = c_phase;
		cr_maskl(EMMC_PLL_PHASE_INTERPOLATION,0x03fUL,10 ,c_phase);
	}

	mmcinfo("PHASE_REG(0x%08x)=0x%08x\n",
		EMMC_PLLPHASE_CTRL, cr_readl(EMMC_PLLPHASE_CTRL));


	return 0;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_set_PLL_SSC(u32 ncode ,u32 fcode,u32 rang)
{
	int ret;
	unsigned int loop;
	//e_device_type* sdport = &emmc_card;

	ret = -1;

	UPRINTF("MMC: Going to enable SSC!!!\n");
	cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,0);	/* PLLEMMC_OC_EN = 0 */

	cr_maskl(EMMC_PLL_SSC3,0x0FFUL,16,ncode);	/* Ncode_ssc = veriable */
	cr_maskl(EMMC_PLL_SSC3,0x7FFUL,0 ,fcode);	/* Fcode_ssc = veriable */
	cr_maskl(EMMC_PLL_SSC5,0x7UL,24,4);		/* dot_gran=4 */
	cr_maskl(EMMC_PLL_SSC5,0x7ffffUL,4,rang);		/* Gran_est=49166(30k) */

	cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,1);	/* PLLEMMC_OC_EN = 1 */

	loop = 200000;
	while(loop--){  /* polling PLLEMMC_Oc_done flag */
		if(cr_readl(EMMC_PLL_SSC6) & _BIT4){
			break;
		}
	}

	cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,0);	/* PLLEMMC_OC_EN = 0 */
	if(loop){
		cr_maskl(EMMC_PLL_SSC5,0x001UL,0 ,1);	/* En_ssc =1 */
		ret = 0;
	}else{
		UPRINTF("SSC enable fail!!!\n");
	}

	return ret;

}

/************************************************************************

 ************************************************************************/
void dw_em_set_clk(unsigned int em_clock)
{
	e_device_type* sdport = &emmc_card;
	unsigned int base_clk = 0;
	unsigned int clk_div = 0;
	unsigned int seted_clk =0;
	unsigned int idx = 0;
	u32 mid = sdport->mid;

	if(em_clock > CLK_200Mhz){
		UPRINTF("emmc: unknow clock target %u, force to 25Mhz\n",
			   em_clock);
		em_clock = CLK_25Mhz;
	}

	if(em_clock <=CLK_400Khz ){
		/* select 200KHz */
		idx = 0;
		seted_clk = CLK_200Khz;
		base_clk = CLK_50Mhz;
		clk_div = 250;
	}else if(em_clock < CLK_15Mhz){
		/* 10Mhz */
		idx = 1;
		seted_clk = CLK_10Mhz;
		base_clk = CLK_60Mhz;
		clk_div = 6;
	}else if(em_clock < CLK_20Mhz){
		/* 15Mhz */
		idx = 2;
		seted_clk = CLK_15Mhz;
		base_clk = CLK_60Mhz;
		clk_div = 4;
	}else if(em_clock < CLK_25Mhz){
		/* 20Mhz */
		idx = 3;
		seted_clk = CLK_20Mhz;
		base_clk = CLK_80Mhz;
		clk_div  = 4;
	} else if (em_clock < CLK_30Mhz) {
		/* 25Mhz */
		idx = 4;
		seted_clk = CLK_25Mhz;
		base_clk = CLK_50Mhz;
		clk_div = 2;
	} else if (em_clock < CLK_40Mhz) {
		/* 30Mhz */
		idx = 5;
		seted_clk = CLK_30Mhz;
		base_clk = CLK_60Mhz;
		clk_div  = 2;
	} else if (em_clock < CLK_50Mhz) {
		/* 40Mhz */
		idx = 6;
		seted_clk = CLK_40Mhz;
		base_clk = CLK_80Mhz;
		clk_div = 2;
	} else if (em_clock < CLK_60Mhz) {
		/* 50Mhz */
		idx = 7;
		seted_clk = CLK_50Mhz;
		base_clk = CLK_50Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_70Mhz) {
		/* 60Mhz */
		idx = 8;
		seted_clk = CLK_60Mhz;
		base_clk = CLK_60Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_80Mhz) {
		/* 70Mhz */
		idx = 9;
		seted_clk = CLK_70Mhz;
		base_clk = CLK_70Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_90Mhz) {
		/* 80Mhz */
		idx = 10;
		seted_clk = CLK_80Mhz;
		base_clk = CLK_80Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_100Mhz) {
		/* 90Mhz */
		idx = 11;
		seted_clk = CLK_90Mhz;
		base_clk = CLK_90Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_110Mhz) {
		/* 100Mhz */
		idx = 12;
		seted_clk = CLK_100Mhz;
		base_clk  = CLK_100Mhz;
		clk_div   = 0;
	} else if (em_clock < CLK_120Mhz) {
		/* 110Mhz */
		idx = 13;
		seted_clk = CLK_110Mhz;
		base_clk = CLK_110Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_130Mhz) {
		/* 120Mhz */
		idx = 14;
		seted_clk = CLK_120Mhz;
		base_clk  = CLK_120Mhz;
		clk_div   = 0;
	} else if (em_clock < CLK_140Mhz) {
		/* 130Mhz */
		idx = 15;
		seted_clk = CLK_130Mhz;
		base_clk = CLK_130Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_150Mhz) {
		/* 140Mhz */
		idx = 16;
		seted_clk = CLK_140Mhz;
		base_clk  = CLK_140Mhz;
		clk_div   = 0;
	} else if (em_clock < CLK_160Mhz) {
		/* 150Mhz */
		idx = 17;
		seted_clk = CLK_150Mhz;
		base_clk = CLK_150Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_170Mhz) {
		/* 160Mhz */
		idx = 18;
		seted_clk = CLK_160Mhz;
		base_clk  = CLK_160Mhz;
		clk_div   = 0;
	} else if (em_clock < CLK_180Mhz) {
		/* 170Mhz */
		idx = 19;
		seted_clk = CLK_170Mhz;
		base_clk = CLK_170Mhz;
		clk_div = 0;
	} else if (em_clock < CLK_190Mhz) {
		/* 180Mhz */
		idx = 20;
		seted_clk = CLK_180Mhz;
		base_clk  = CLK_180Mhz;
		clk_div   = 0;
	} else if (em_clock < CLK_200Mhz) {
		/* 190Mhz */
		idx = 21;
		seted_clk = CLK_190Mhz;
		base_clk = CLK_190Mhz;
		clk_div = 0;
	}else{
		/*
		 if device have tuned prior, It can support 200Mhz.
		 otherwise down speed.
		 */
		/* 200Mhz */
		idx = 22;
		seted_clk = CLK_200Mhz;
		base_clk  = CLK_200Mhz;
		clk_div   = 0;
	}

	if(sdport->cur_clock == seted_clk){
		/* do nothing */
	}else{
		UPRINTF("%s: clk change to %s(%u)[mid:0x%x]\n",DRIVER_NAME, dw_clk_tlb[idx], idx,mid);

		mmcinfo("%s(%d)seted_clk=%d seted_clk=%d clk_div=%d\n",
			__func__,__LINE__,
			seted_clk,base_clk,clk_div);

		dw_em_set_PLL_clk(base_clk,clk_div,0);
		sdport->cur_clock = seted_clk;

	}
}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_send_cmd(u32 cmd, u32 sd_arg)
{
	unsigned int cmd_status = 0;
	unsigned int loop = 0;

	cr_writel(sd_arg, EM_DWC_CMDARG);
	cr_writel(SDMMC_CMD_START | cmd, EM_DWC_CMD);

	loop = 5000;
	while (loop--) {
		cmd_status = cr_readl(EM_DWC_CMD);
		if (!(cmd_status & SDMMC_CMD_START))
			return;
	}

	UPRINTF("Timeout sending command (cmd %#x arg %#x status %#x)\n",
		   cmd, sd_arg, cmd_status);
}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_set_div(unsigned int em_div)
{
	e_device_type* sdport = &emmc_card;
	unsigned int row_div;

	if(em_div)
		row_div = (em_div>>1);
	else
		row_div = 0;

	/* reset div every time change clock */

	/* disable clock */
	cr_writel(0, EM_DWC_CLKENA);
	cr_writel(0, EM_DWC_CLKSRC);

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);

	/* set clock to desired speed */
	cr_writel( row_div, EM_DWC_CLKDIV );

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);

	/* enable clock */
#if 0
	cr_writel(_BIT0, EM_DWC_CLKENA);
#else
	/* only low power if no SDIO */
	cr_writel((_BIT16 | _BIT0), EM_DWC_CLKENA);
#endif

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);

	sdport->cur_div = em_div;

	mmcinfo("CLK divide=%d\n",(row_div<<1));
}

#define DISABLE_LOW_POWER_MODE
#ifdef DISABLE_LOW_POWER_MODE
/************************************************************************
 *
 ************************************************************************/
#define LOW_POWER_ON	0
#define LOW_POWER_OFF	1
static
void dw_em_low_power_ctl(unsigned ctl)
{
	unsigned int row_div;

	row_div = cr_readl( EM_DWC_CLKDIV );

	/* reset div every time change clock */

	/* disable clock */
	cr_writel(0, EM_DWC_CLKENA);
	cr_writel(0, EM_DWC_CLKSRC);

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);

	/* set clock to desired speed */
	cr_writel( row_div, EM_DWC_CLKDIV );

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);

	/* enable clock */
	if(ctl == LOW_POWER_OFF ){
		cr_writel(_BIT0, EM_DWC_CLKENA);
	}else{
		/* only low power if no SDIO */
		cr_writel((_BIT16 | _BIT0), EM_DWC_CLKENA);
	}

	/* inform CIU */
	dw_em_send_cmd(SDMMC_CMD_UPD_CLK | SDMMC_CMD_PRV_DAT_WAIT, 0);
}

/************************************************************************
 *
 ************************************************************************/
void dw_em_send_dummy_clk(unsigned ms_time)
{
	dw_em_low_power_ctl(LOW_POWER_OFF);
	mdelay(ms_time);
	dw_em_low_power_ctl(LOW_POWER_ON);
}

#endif


/************************************************************************
 *
 ************************************************************************/
static
void dw_em_clear_int_sta(unsigned int hold_int){

	if(hold_int)
		cr_writel( 0x0, EM_DWC_INTMASK );           //disable int.

	cr_writel( 0xffffffff, EM_DWC_RINTSTS );    //Clear all int result.
}

/************************************************************************
 *
 ************************************************************************/
#if 0
static
void dw_em_enable_int_sta(unsigned int_event)
{
#if 1
	cr_writel( 0xffffffff, EM_DWC_INTMASK );  //enable int.
#else
	cr_writel( int_event, EM_DWC_INTMASK );  //enable int.
#endif
	/* enable global int. */
	cr_writel( cr_readl(EM_DWC_CTRL)|SDMMC_CTRL_INT_ENABLE, EM_DWC_CTRL );
}
#endif


/************************************************************************
 *
 ************************************************************************/
#if 0
static
void dw_em_clear_int_sta_IDMA(unsigned int hold_int){

	if(hold_int)
		cr_writel( 0x0, EM_DWC_IDINTEN );           //disable int.

	/* Clear all int result. */
	cr_writel( 0xffffffff, EM_DWC_IDSTS );
}
#endif

/************************************************************************
 *
 ************************************************************************/
 #if 0
static
void dw_em_enable_int_sta_IDMA(unsigned int_event)
{
#if 1
	cr_writel( 0xffffffff, EM_DWC_IDINTEN );  //enable int.
#else
	cr_writel( int_event, EM_DWC_IDINTEN );  //enable int.
#endif
	/* enable global int. */
	cr_writel( cr_readl(EM_DWC_CTRL)|SDMMC_CTRL_INT_ENABLE, EM_DWC_CTRL );
}
#endif

/************************************************************************
 *  Y:\nfs\kernel\macarthur3\linux\linux-3.10.0\drivers\mmc\host\dw_mmc.c
 *  static void dw_mci_translate_sglist(struct dw_mci *host, struct mmc_data *data,
 ************************************************************************/
#define SHOW_SCRIPT_DATA        0
static
void dw_em_make_script(struct rtk_cmd_info * cmd_info)
{
	unsigned int *des_base;
	unsigned int  mem_addr;
	unsigned int  blk_cnt;
	unsigned int *des_base2;
	unsigned int  mem_addr2;
	unsigned int  blk_cnt2;
	unsigned int  remain_blk_cnt;
	unsigned int  byte_cnt;
	unsigned int  byte_ctl;
	unsigned int  tmp_val;

	des_base = dw_em_scrip;
	mem_addr = (unsigned int)cmd_info->data_buffer;;
	blk_cnt  = cmd_info->block_count;
	byte_cnt = cmd_info->byte_count;

	if(byte_cnt < 512 )
		byte_ctl = 1;
	else
		byte_ctl = 0;

	des_base2       = des_base;
	mem_addr2       = mem_addr;
	remain_blk_cnt  = blk_cnt;

#if SHOW_SCRIPT_DATA
	mmcinfo("des_base=0x%x mem_addr=0x%x blk_cnt=0x%x byte_cnt0x%x[bc=%x]\n",
			(unsigned int)des_base2, (unsigned int)mem_addr,
			(unsigned int)blk_cnt, (unsigned int)byte_cnt, byte_ctl);
#endif

	while(remain_blk_cnt)
	{
#if SHOW_SCRIPT_DATA
		mmcinfo("des_base2=0x%x\n", (unsigned int)des_base2);
#endif
		/* setting des0; transfer parameter  */
		tmp_val = IDMAC_DES0_OWN
				| IDMAC_DES0_DIC | IDMAC_DES0_CH;

		if(remain_blk_cnt == blk_cnt)
			tmp_val |= IDMAC_DES0_FD;

		if(remain_blk_cnt <= EMMC_MAX_SCRIPT_BLK)
			tmp_val |= IDMAC_DES0_LD;

		des_base2[0] = tmp_val;

		/* setting des1; buffer size in byte */
		if(remain_blk_cnt > EMMC_MAX_SCRIPT_BLK){
			blk_cnt2 = EMMC_MAX_SCRIPT_BLK;
		}else{
			blk_cnt2 = remain_blk_cnt;
		}
		des_base2[1] = (blk_cnt2<<9);

		/* setting des2; Physical address to DMA to/from */
		des_base2[2] = (mem_addr2 );

		/* setting des3; next descrpter entry */
		des_base2[3] = ((unsigned int)(des_base2+4));
		if(byte_ctl){
			des_base2[0] |= IDMAC_DES0_LD;
			des_base2[1] = byte_cnt;
			remain_blk_cnt = 0;
#if SHOW_SCRIPT_DATA
			/*
			  the value of des_base[x] and des_base2[x] is different.
			  because point des_base2 is increase 4 per loop
			*/
			mmcinfo("\n"
					"des0=0x%08x des1=0x%08x des2=0x%08x des3=0x%08x\n",
					des_base2[0],des_base2[1],des_base2[2],des_base2[3]);
#endif
		}else{
#if SHOW_SCRIPT_DATA
			/*
			  the value of des_base[x] and des_base2[x] is different.
			  because point des_base2 is increase 4 per loop
			  If want to see value of des_base2[x], It must be here,
			  because  des_base2 will be increased later.
			*/
			mmcinfo("\n"
					"des0=0x%08x des1=0x%08x des2=0x%08x des3=0x%08x\n",
					des_base2[0],des_base2[1],des_base2[2],des_base2[3]);
#endif
			mem_addr2 = mem_addr2+(blk_cnt2<<9);
			remain_blk_cnt -= blk_cnt2;
			des_base2 += 4;
		}
	}
	flush_cache((unsigned long)dw_em_scrip, 16*EMMC_MAX_SCRIPT_LINE);
}

/************************************************************************
 * IP configure
 ************************************************************************/
static
void dw_em_IP_config(void)
{
	unsigned int reginfo;

	cr_writel( _BIT0, EM_DWC_PWREN );           //enable power of device 0.
	cr_writel( 0x0, EM_DWC_INTMASK );           //disable int.
	cr_writel( 0xffffffff, EM_DWC_RINTSTS );    //Clear all int result.

#if 0
	cr_writel( 0xffffffff, EM_DWC_INTMASK );  //enable int.
	cr_writel( cr_readl(EM_DWC_CTRL)|SDMMC_CTRL_INT_ENABLE, EM_DWC_CTRL );    //enable global int.
#endif

	cr_writel( 0xffffff40,EM_DWC_TMOUT );               //set timeout of data and response as default.

#if 1
	cr_writel( (0x05UL<<SDMMC_MSIZE_SHT)        /* set DW_DMA_Multiple_Transaction_Size as 64(0x05) */
		  |(0x3fUL<<SDMMC_RX_WMARK_SHT)     /* RX_WMARK=0x3f */
		  |(0x40UL<<SDMMC_TX_WMARK_SHT) ,   /* TX_WMARK=0x40 */
			EM_DWC_FIFOTH );
#else
	cr_writel( (0x05UL<<SDMMC_MSIZE_SHT)        /* set DW_DMA_Multiple_Transaction_Size as 64(0x05) */
			  |(0x7fUL<<SDMMC_RX_WMARK_SHT)     /* RX_WMARK=0x7f */
			  |(0x80UL<<SDMMC_TX_WMARK_SHT) ,   /* TX_WMARK=0x80 */
				EM_DWC_FIFOTH );
#endif

#if 1
	cr_writel( SDMMC_DE|SDMMC_FB,EM_DWC_BMOD );
#else
	cr_writel( SDMMC_DE,EM_DWC_BMOD );
#endif

	cr_writel( 0,EM_DWC_IDINTEN );

#if 0	/* bootcode have not interrupt mode */
	reginfo = (SDMMC_CTRL_USE_IDMAC     // use_internal_dmac
		  |SDMMC_CTRL_INT_ENABLE);  // int_en
#else
	reginfo = (SDMMC_CTRL_USE_IDMAC);   //use_internal_dmac
#endif

	cr_writel( reginfo,EM_DWC_CTRL );

	/* enable write/read threshold as 512 bytes */
	cr_writel((0x0200 << 16) | _BIT2 | _BIT0, EM_DWC_CardThrCtl);

	cr_writel( cr_readl(EM_WARP_STATUS0)|BIT(18) ,EM_WARP_STATUS0 );

}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_set_scrip_base(unsigned int scrip_base)
{
	/* set Descriptor List Base Address Register */
	cr_writel( scrip_base,EM_DWC_DBADDR );
}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_reset_card(void)
{
	/* card reset should be implement here. This reg is GPIO control only.  */
	cr_writel(~_BIT0,EM_DWC_RST_n);
	udelay(2000);
	cr_writel(_BIT0,EM_DWC_RST_n);

	/* Toggle 74 clock here :
	 *  The toggle clock will be send out before cmd0 by DW IP.
	 *  To reference unsigned int dw_em_prepare_command()
	 *  cmd0 set SDMMC_CMD_INIT bit.
	 */
}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_reset_IP(void)
{
	int ret;
	u32 loop;

   /*
	* To reset controller/FIFO/DMA.
	* This reset is not to effect warper of RTK.
	*/
	cr_writel(cr_readl(EM_DWC_CTRL)|(0x07), EM_DWC_CTRL);

	ret = -1;
	loop = 1000;
	while(loop--){
		if((cr_readl(EM_DWC_CTRL)&(0x07)) == 0){
			ret = 0;
			break;
		}
		udelay(1);
}

	if(ret){
		UPRINTF("%s: IP reset fail!!!\n",DRIVER_NAME);
	}
	return;
}

/************************************************************************
 *Notice : PLL and IO_pad_driving will be reset after IP reset

 ************************************************************************/
static
void dw_em_reset_host_IP(u8 save_en)
{
	u32 config1;
	u32 config2;
	u32 config3;
	u32 config4;
	u32 config5;
	u32 config6;

	if(save_en){
		/* store currently IP setting */
		config1 = cr_readl(EM_DWC_PWREN);
		config2 = cr_readl(EM_DWC_CLKDIV);
		config3 = cr_readl(EM_DWC_CTYPE);
		config4 = cr_readl(EM_DWC_UHS_REG);
		config5 = cr_readl(EM_DWC_DBADDR);
		config6 = cr_readl(EM_DWC_EMMC_DDR_REG);
	}

	/* reset synopsys eMMC IP */
	dw_em_reset_IP();
	udelay(10);

	dw_em_IP_config();

	if(save_en){
		/* restore previously IP setting */
		cr_writel(config1,EM_DWC_PWREN);
		dw_em_set_div((config2<<1));
		cr_writel(config3,EM_DWC_CTYPE);
		cr_writel(config4,EM_DWC_UHS_REG);
		cr_writel(config5,EM_DWC_DBADDR);
		cr_writel(config6,EM_DWC_EMMC_DDR_REG);
	}

}

/************************************************************************
 *
 ************************************************************************/
#define RTK_IOPAD_CFG1	0x180108A0
#define RTK_IOPAD_CFG2	0x180108A8
#define RTK_IOPAD_CFG3	0x180108AC
#define RTK_IOPAD_SET1	0x180108A4
#define RTK_IOPAD_SET2	0x180108B0
#define RTK_IOPAD_SET3	0x180108B4
#define RTK_IOPAD_SET4	0x180108B8

static
int dw_em_save_pad(u32* pad_arry,int cnt)
{
	if(cnt != PAD_SET_CNT)
		return -1;
	pad_arry[0] = cr_readl(RTK_IOPAD_CFG1);
	pad_arry[1] = cr_readl(RTK_IOPAD_CFG2);
	pad_arry[2] = cr_readl(RTK_IOPAD_CFG3);
	pad_arry[3] = cr_readl(RTK_IOPAD_SET1);
	pad_arry[4] = cr_readl(RTK_IOPAD_SET2);
	pad_arry[5] = cr_readl(RTK_IOPAD_SET3);
	pad_arry[6] = cr_readl(RTK_IOPAD_SET4);

	return 0;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_restore_pad(u32* pad_arry)
{
	cr_writel(pad_arry[0] ,RTK_IOPAD_CFG1);   //rst,clk,cmd,
	cr_writel(pad_arry[1] ,RTK_IOPAD_CFG2);   //ds,d5,d3,d4,d0,d1,d2,d7
	cr_writel(pad_arry[2] ,RTK_IOPAD_CFG3);   //d6

	cr_writel(pad_arry[3] , RTK_IOPAD_SET1);   //rst/clk/cmd
	cr_writel(pad_arry[4] , RTK_IOPAD_SET2);   //ds/d5/d3
	cr_writel(pad_arry[5] , RTK_IOPAD_SET3);   //d4/d0/d1
	cr_writel(pad_arry[6] , RTK_IOPAD_SET4);   //d2/d7/d6

	return 0;
}


/************************************************************************
 eMMC Pad driving:
 DRIVE_LEVEL is means driving strength,
 As the level number is bigger, the strength is greater

 ************************************************************************/

#define DRIVE_LEVEL7      (0x3fUL)
#define DRIVE_LEVEL6      (0x1bUL)
#define DRIVE_LEVEL5      (0x2dUL)
#define DRIVE_LEVEL4      (0x09UL)
#define DRIVE_LEVEL4_2    (0x36UL)
#define DRIVE_LEVEL3      (0x12UL)
#define DRIVE_LEVEL2      (0x24UL)
#define DRIVE_LEVEL1      (0x0UL)

#define IO_PAD_SET1_HS400 ((DRIVE_LEVEL4<<26)|(DRIVE_LEVEL4<<17)|(DRIVE_LEVEL4<<8))   //rst/clk/cmd
#define IO_PAD_SET2_HS400 ((DRIVE_LEVEL4<<26)|(DRIVE_LEVEL4<<17)|(DRIVE_LEVEL4<<8))   //ds/d5/d3
#define IO_PAD_SET3_HS400 ((DRIVE_LEVEL4<<26)|(DRIVE_LEVEL4<<17)|(DRIVE_LEVEL4<<8))   //d4/d0/d1
#define IO_PAD_SET4_HS400 ((DRIVE_LEVEL4<<26)|(DRIVE_LEVEL4<<17)|(DRIVE_LEVEL4<<8))   //d2/d7/d6
#define IO_PAD_SET5_HS400 0

#define IO_PAD_SET1_HS200 IO_PAD_SET1_HS400
#define IO_PAD_SET2_HS200 IO_PAD_SET2_HS400
#define IO_PAD_SET3_HS200 IO_PAD_SET3_HS400
#define IO_PAD_SET4_HS200 IO_PAD_SET4_HS400
#define IO_PAD_SET5_HS200 IO_PAD_SET5_HS400

/*
 with device driving:HS50 : 0x0(x1)
 */
#define IO_PAD_SET1_HS50 ((DRIVE_LEVEL1<<26)|(DRIVE_LEVEL1<<17)|(DRIVE_LEVEL1<<8))
#define IO_PAD_SET2_HS50 ((DRIVE_LEVEL1<<26)|(DRIVE_LEVEL1<<17)|(DRIVE_LEVEL1<<8))
#define IO_PAD_SET3_HS50 ((DRIVE_LEVEL1<<26)|(DRIVE_LEVEL1<<17)|(DRIVE_LEVEL1<<8))
#define IO_PAD_SET4_HS50 ((DRIVE_LEVEL1<<26)|(DRIVE_LEVEL1<<17)|(DRIVE_LEVEL1<<8))
#define IO_PAD_SET5_HS50 ((DRIVE_LEVEL1<<26)|(DRIVE_LEVEL1<<17)|(DRIVE_LEVEL1<<8))

void dw_em_set_IO_driving(unsigned int type_sel)
{
	//e_device_type* sdport = &emmc_card;
	//u32 mid = sdport->mid;
	u32 io_pad_setting1;
	u32 io_pad_setting2;
	u32 io_pad_setting3;
	u32 io_pad_setting4;
	u32 w_phase;
	u32 c_phase;

	/*
	 Pad config:
	 set emmc pull up/pull down/slew rate/smt trigger
	 Note: maybe to set it at card_insert()
	*/
	cr_writel(0x91900000,RTK_IOPAD_CFG1);	//rst,clk,cmd,
	cr_writel(0x59999999,RTK_IOPAD_CFG2);	//ds,d5,d3,d4,d0,d1,d2,d7
	cr_writel(0x90000000,RTK_IOPAD_CFG3);	//d6

	if(type_sel == DRIVING_HS400){
		w_phase = DEFAULT_W_PHASE_400;
		c_phase = DEFAULT_C_PHASE_200;
		io_pad_setting1 = IO_PAD_SET1_HS400;
		io_pad_setting2 = IO_PAD_SET2_HS400;
		io_pad_setting3 = IO_PAD_SET3_HS400;
		io_pad_setting4 = IO_PAD_SET4_HS400;
	}else if(type_sel == DRIVING_HS200){
		w_phase = DEFAULT_W_PHASE_200;
		c_phase = DEFAULT_C_PHASE_200;
		io_pad_setting1 = IO_PAD_SET1_HS200;
		io_pad_setting2 = IO_PAD_SET2_HS200;
		io_pad_setting3 = IO_PAD_SET3_HS200;
		io_pad_setting4 = IO_PAD_SET4_HS200;
	}else{
		w_phase = DEFAULT_W_PHASE;
		c_phase = DEFAULT_C_PHASE;
		io_pad_setting1 = IO_PAD_SET1_HS50;
		io_pad_setting2 = IO_PAD_SET2_HS50;
		io_pad_setting3 = IO_PAD_SET3_HS50;
		io_pad_setting4 = IO_PAD_SET4_HS50;
	}

	/*
	 eMMC Pad driving/duty cycle:
	 Pad driving set to 0.75,duty cycle shift 0ns
	*/
	cr_writel(io_pad_setting1,RTK_IOPAD_SET1); //rst/clk/cmd
	cr_writel(io_pad_setting2,RTK_IOPAD_SET2); //ds/d5/d3
	cr_writel(io_pad_setting3,RTK_IOPAD_SET3); //d4/d0/d1
	cr_writel(io_pad_setting4,RTK_IOPAD_SET4); //d2/d7/d6

	dw_em_init_phase(w_phase,0xff,c_phase);
}


/************************************************************************
 *
 ************************************************************************/
static
void dw_em_reset_host(unsigned int save_en)
{
	e_device_type* sdport = &emmc_card;
	u32 cpu_sel;
	u32 reginfo;
	u32 config1;
	u32 config2;
	u32 config3;
	u32 config4;
	u32 config5;
	u32 config6;
	u32 config7;
	u32 config8;
	unsigned int pad_set[PAD_SET_CNT];
#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	spin_lock_t g_mmc_lock = INIT_SPIN_LOCK;
	unsigned int flags;
#endif

	if(save_en){
		/* store currently IP setting */
		config1 = cr_readl(EM_DWC_PWREN);
		config2 = cr_readl(EM_DWC_CLKDIV);
		config3 = cr_readl(EM_DWC_CTYPE);
		config4 = cr_readl(EM_DWC_UHS_REG);
		config5 = cr_readl(EM_DWC_DBADDR);
		config6 = cr_readl(EM_DWC_EMMC_DDR_REG);
		config7 = cr_readl(EM_HALF_CYCLE_CAL_EN);
		dw_em_dis_sd_tune();
	}
	config8 = cr_readl(EM_WARP_STATUS0);
	dw_em_save_pad(pad_set,PAD_SET_CNT);

#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	spin_lock_save(&g_mmc_lock, flags);
#endif
	cpu_sel = cr_readl(EM_SCPU_SEL);    //save ststus of CPU_SEL
	reginfo = CLKEN_EM_DW;
	cr_writel(reginfo,RTK_EM_DW_CLKEN);

	reginfo = RSTN_EM_DW;
	cr_writel(reginfo,RTK_EM_DW_SRST);

	udelay(50);

	reginfo = RSTN_EM_DW | 0x01UL;
	cr_writel(reginfo,RTK_EM_DW_SRST);

	reginfo = CLKEN_EM_DW | 0x01UL;
	cr_writel(reginfo,RTK_EM_DW_CLKEN);
	cr_writel( cpu_sel, EM_SCPU_SEL );  //restore CPU_SEL

	dw_em_restore_pad(pad_set);

#if defined(RTK_EMMC_SUPPORT_MULTICORE)
	spin_unlock_restore(&g_mmc_lock, flags);
#endif
	/* reset synopsys eMMC IP */
	dw_em_IP_config();

	cr_writel(config8,EM_WARP_STATUS0);
	if(save_en){
		/* restore previously IP setting */
		cr_writel(config1,EM_DWC_PWREN);
		dw_em_set_div((config2<<1));
		cr_writel(config3,EM_DWC_CTYPE);
		cr_writel(config4,EM_DWC_UHS_REG);
		cr_writel(config5,EM_DWC_DBADDR);
		cr_writel(config6,EM_DWC_EMMC_DDR_REG);
		if(config7 & _BIT0){
			dw_em_set_ds_delay(sdport->cur_ds_tune);
		}
	}

}

/************************************************************************
 *
 typedef struct{
	u32                 ctl;
	u8                  ncode_t;
	u8                  r_phase;
	u8                  w_phase;
	u8                  icp;
	u16                 ncode;
	u16                 fcode;
	u32                 range;
}  mmc_param_t;
 ************************************************************************/
mmc_param_t mmc_defaul_tparam[16]= {
/*  ctl/ncode_t/sdiv2/r_phase/w_phase/icp/pi_isel/ncode_ssc/fcode_ssc/range_ssc              */
    { 1,    40,    0,      13,   0xff,  2,     3,    0x27,    0X48F,   0x2E5C }, /* 193.5Mhz */
    { 0,    39,    0,      20,   0xff,  2,     3,       0,        0,        0 }, /* 189Mhz   */
    { 0,    37,    0,      51,   0xff,  2,     3,       0,        0,        0 }, /* 180Mhz   */
    { 0,    35,    0,      48,   0xff,  1,     3,       0,        0,        0 }, /* 171Mhz   */
    { 0,    33,    0,      45,   0xff,  1,     3,       0,        0,        0 }, /* 162Mhz   */
    { 0,    30,    0,      43,   0xff,  1,     2,       0,        0,        0 }, /* 148.5Mhz */
    { 0,    28,    0,      41,   0xff,  1,     2,       0,        0,        0 }, /* 139.5Mhz */
    { 0,    26,    0,      37,   0xff,  1,     2,       0,        0,        0 }, /* 130.5Mhz */
    { 0,    24,    0,      34,   0xff,  1,     2,       0,        0,        0 }, /* 121.5Mhz */
    { 0,    21,    0,      35,   0xff,  1,     2,       0,        0,        0 }, /* 108Mhz   */
    { 0,    41,    1,      37,   0xff,  2,     3,       0,        0,        0 }, /* 99Mhz    */
    { 0,    37,    1,      37,   0xff,  2,     3,       0,        0,        0 }, /* 90Mhz    */
    { 0,    33,    1,      37,   0xff,  1,     3,       0,        0,        0 }, /* 81Mhz    */
    { 0,    29,    1,      37,   0xff,  1,     2,       0,        0,        0 }, /* 72Mhz    */
    { 0,    23,    1,      37,   0xff,  1,     2,       0,        0,        0 }, /* 58.5Mhz  */
    { 0,    19,    1,      12,   0xff,  0,     1,       0,        0,        0 }  /* 49.5Mhz include under */
};

/************************************************************************
 *
 * PLL Clock Formula:
 *
 * PLLEMMC_NCODE_T = 0x1800_0720[23:16]
 * PLLSN           = 0x1800_0700[10:8] <==1(currently)
 *   Note : max PLLSN is 4
 * PLLSDIV2        = 0x1800_0700[17]   <==0(currently)
 *   Fout = 27 *( PLLEMMC_NCODE_T +3)/ (PLLSN+2)/( PLLSDIV2+1)/2
 *        = 27 *( PLLEMMC_NCODE_T +3)/ 6
 *   PLLEMMC_NCODE_T = (((6 * Fout )/27)-3)
 *
 * if PLLSN is 4
 *   PLLEMMC_NCODE_T = (((12 * Fout )/27)-3)
 *
 *                      PLLSN=  [1]    [4]
 * 200Mhz : 0x1800_0720[23:16] = 41 	85
 * 190Mhz : 0x1800_0720[23:16] = 39	81
 * 180Mhz : 0x1800_0720[23:16] = 37	77
 * 170Mhz : 0x1800_0720[23:16] = 35	72
 * 160Mhz : 0x1800_0720[23:16] = 32	68
 * 150Mhz : 0x1800_0720[23:16] = 30	63
 * 140Mhz : 0x1800_0720[23:16] = 28	59
 * 130Mhz : 0x1800_0720[23:16] = 26	54
 * 120Mhz : 0x1800_0720[23:16] = 24	50
 * 110Mhz : 0x1800_0720[23:16] = 21	45
 * 100Mhz : 0x1800_0720[23:16] = 19	41
 *  90Mhz : 0x1800_0720[23:16] = 17	37
 *  80Mhz : 0x1800_0720[23:16] = 15	32
 *  70Mhz : 0x1800_0720[23:16] = 12	28
 *  60Mhz : 0x1800_0720[23:16] = 10	23
 *  50Mhz : 0x1800_0720[23:16] =  8	19
 *
 * ICP setting :
 * under 50Mhz =>                 => ICP=? => 0x18000700[16:12]=0x00
 * 50,60,70Mhz => VCO=100,120,140 => ICP=5 => 0x18000700[16:12]=0x01
 * 80,90,100,110, 120,130,140Mhz
 *             => VCO=160,180,200,220,240,260,280
 *                                => ICP=10 => 0x18000700[16:12]=0x03
 * 150,160,170,180,190,200Mhz
 *             => VCO=300,320,340,360,380,400
 *                                => ICP=20 => 0x18000700[16:12]=0x07
 *
 ************************************************************************/
static
int dw_em_set_PLL_clk(unsigned int targ_clk,unsigned int clk_div,u32 init)
{
	e_device_type* sdport = &emmc_card;
	int err = 0;
	unsigned int pllemmc_ncode_t = 0;
	unsigned int loop = 200000;
	unsigned int icp = 0;
	unsigned int sdiv2 = 0;
	unsigned int sdiv2_old = 0;
	unsigned int pi_isel = 0;
	unsigned int r_phase = 0;
	unsigned int pll_sta = 0;
	unsigned int ncode_ssc = 0;
	unsigned int fcode_ssc = 0;
	unsigned int ssc_rang = 0;
	unsigned char idx;
	unsigned int pad_set[PAD_SET_CNT];
	unsigned int ldo_trim_info_otp = 0;
	bool ret = 0;
	u32 config1;
	u32 config3;
	u32 config4;
	u32 config5;
	u32 config6;
	u32 config7;
	u32 config8;
	mmc_param_t *tmp_param_ptr;

#define SSC_EN _BIT0

	if (sdport->cur_PLL == targ_clk) {
		dw_em_set_div(clk_div);
		return 0;
	}

	if (targ_clk >= CLK_200Mhz){
		idx = 0;
	}else if (targ_clk >= CLK_190Mhz){
		idx = 1;
	}else if (targ_clk >= CLK_180Mhz){
		idx = 2;
	}else if (targ_clk >= CLK_170Mhz){
		idx = 3;
	}else if (targ_clk >= CLK_160Mhz){
		idx = 4;
	}else if (targ_clk >= CLK_150Mhz){
		idx = 5;
	}else if (targ_clk >= CLK_140Mhz){
		idx = 6;
	}else if (targ_clk >= CLK_130Mhz){
		idx = 7;
	}else if (targ_clk >= CLK_120Mhz){
		idx = 8;
	}else if (targ_clk >= CLK_110Mhz){
		idx = 9;
	}else if (targ_clk >= CLK_100Mhz){
		idx = 10;
	}else if (targ_clk >= CLK_90Mhz){
		idx = 11;
	}else if (targ_clk >= CLK_80Mhz){
		idx = 12;
	}else if (targ_clk >= CLK_70Mhz){
		idx = 13;
	}else if (targ_clk >= CLK_60Mhz){
		idx = 14;
	}else {
		idx = 15;
	}

	tmp_param_ptr = mmc_defaul_tparam;

	pllemmc_ncode_t = tmp_param_ptr[idx].ncode_t;
	sdiv2           = tmp_param_ptr[idx].sdiv2;
	r_phase         = tmp_param_ptr[idx].r_phase;
	icp             = tmp_param_ptr[idx].icp;
	pi_isel         = tmp_param_ptr[idx].pi_isel;

	ncode_ssc = tmp_param_ptr[idx].ncode_ssc;
	fcode_ssc = tmp_param_ptr[idx].fcode_ssc;
	ssc_rang  = tmp_param_ptr[idx].range_ssc;

	if(tmp_param_ptr[idx].ctl & MMC_EN_SSCG)
		pll_sta |= SSC_EN;

	/* UPRINTF("pllemmc_ncode_t=0x%x\n",pllemmc_ncode_t); */

	/* backup IP status first */
	config1 = cr_readl(EM_DWC_PWREN);
	config3 = cr_readl(EM_DWC_CTYPE);
	config4 = cr_readl(EM_DWC_UHS_REG);
	config5 = cr_readl(EM_DWC_DBADDR);
	config6 = cr_readl(EM_DWC_EMMC_DDR_REG);
	config7 = cr_readl(EM_HALF_CYCLE_CAL_EN);
	config8 = cr_readl(EM_WARP_STATUS0);
	dw_em_dis_sd_tune();

	if(dw_em_save_pad(pad_set,PAD_SET_CNT))
		printf("iopad array size mismatch!\n");

	/* reset IP only */
	dw_em_reset_IP();
	udelay(10);

	/* gating digital clock */
	cr_writel(CLKEN_EM_DW,RTK_EM_DW_CLKEN);

	if(clk_div){
		/*
		 * if clk_div is true, meaning bus clock lower ther 50Mhz,
		 * use default phase
		 */
		r_phase = DEFAULT_R_PHASE;
	}else{
		/* If clock get from PLL directly ,use new phase setting clock. */
		if(targ_clk == sdport->tud_r_clock)
			r_phase = sdport->tud_r_phase;
	}

	/* w_phase[15:10];r_phase [5:0];c_phase [25:20]; */
	dw_em_init_phase(0xff,r_phase,0xff);

	if(init){
		cr_maskl(EMMC_PLL_SET,0x001UL,0 ,0);	/* PLLALLRSTB = 0 */
		cr_maskl(EMMC_PLL_SET,0x001UL,30 ,0);	/* PLL_EMMC_CKSSC_INV = 0 */
		cr_maskl(EMMC_PLL_SSC0,0x2FFUL,4 ,1);	/* PLLEMMC_OC_STEP_SET = 1 */
		cr_maskl(EMMC_PLL_CTRL,0x001UL,24 ,1);	/* PLLEMMC_FREEZE */
		cr_maskl(EMMC_PLL_CTRL,0x001UL,8 ,0);   /* LDO RSVD[0]=0; RSVD[0]=0x18000704[8] */
		cr_maskl(EMMC_PLL_CTRL,0x003UL,12 ,1);  /* LDO RSVD[5:4]=1; RSVD[5:4]=0x18000704[13:12] */
		ret = rtk_otp_field_read(OTP_FIELD_EMMC_LDO_TRIM_INFO, (unsigned char *)&ldo_trim_info_otp, 4);
		if(!ret)
			printf("Read otp OTP_FIELD_EMMC_LDO_TRIM_INFO fail\n");
		switch(ldo_trim_info_otp){
			case 4:
				cr_maskl(EMMC_PLL_CTRL, 0x003UL, 12, 0);
				break;
			case 5:
				cr_maskl(EMMC_PLL_CTRL, 0x003UL, 12, 1);
				break;
			case 6:
				cr_maskl(EMMC_PLL_CTRL, 0x003UL, 12, 2);
				break;
			case 7:
				cr_maskl(EMMC_PLL_CTRL, 0x003UL, 12, 3);
				break;
			default:
				break;
		}
		cr_maskl(EMMC_PLL_CTRL,0x003UL,0 ,2);	/* LDO power on, PLLEMMC power on */
		cr_maskl(EMMC_PLL_SSC2,0x001UL,2 ,1);   /* En_pi_debug, relative PI_CUR_SEL*/
		cr_maskl(EMMC_PLL_SSC4,0x0FFUL,16,pllemmc_ncode_t);	/* To setting PLLEMMC_NCODE_T */
		cr_maskl(EMMC_PLL_SSC5,0x001UL,0 ,0);	/* En_ssc =0; disable SSC */

		udelay(5);

		cr_maskl(EMMC_PLL_SET,0x007UL,8 ,1);	/* PLLSN = 1, */
		cr_maskl(EMMC_PLL_SET,0x001UL,29,0);    /* CP */
		cr_maskl(EMMC_PLL_SET,0x003UL,27,3);    /* CS */
		cr_maskl(EMMC_PLL_SET,0x01fUL,12,icp);	/* PLL Charger Pump Current IchDpll[16:12] */
		cr_maskl(EMMC_PLL_SET,0x001UL,17,sdiv2);    /* PLLSDIV2 = 0, 27x(19+3)/3/2=99MHz */
		cr_maskl(EMMC_PLL_SSC1,0x03UL,10,pi_isel);  /* PI_CUR_SEL */
		udelay(5);

		cr_maskl(EMMC_PLL_SET,0x001UL,0 ,1);	/* PLLALLRSTB = 1 */
		cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,1);	/* PLLEMMC_OC_EN = 1 */
	}else{
		sdiv2_old = (cr_readl(EMMC_PLL_SET)>>17) & 0x1UL;
		cr_maskl(EMMC_PLL_CTRL,0x001UL,24 ,1);  /* PLLEMMC_FREEZE */
		cr_maskl(EMMC_PLL_SSC5,0x001UL,0 ,0);	/* En_ssc =0; disable SSC */
		cr_maskl(EMMC_PLL_SSC0,0x2FFUL,4 ,0x200);	/* PLLEMMC_OC_STEP_SET = 0x200 */
		cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,0);	/* PLLEMMC_OC_EN = 0 */
		cr_maskl(EMMC_PLL_SET,0x007UL,8 ,1);	/* PLLSN = 1, */
		cr_maskl(EMMC_PLL_SET,0x01fUL,12,icp);  /* PLL Charger Pump Current IchDpll[16:12] */
		cr_maskl(EMMC_PLL_SSC1,0x03UL,10,pi_isel);  /* PI_CUR_SEL */
		if (sdiv2 > sdiv2_old) {
			/* high --> low */
			cr_maskl(EMMC_PLL_SET,0x001UL,17,sdiv2);    /* PLLSDIV2 = 0, 27x(19+3)/3/2=99MHz */
			cr_maskl(EMMC_PLL_SSC4,0x0FFUL,16,pllemmc_ncode_t); /* To setting PLLEMMC_NCODE_T */
		}else{
			/* low --> high */
			cr_maskl(EMMC_PLL_SSC4,0x0FFUL,16,pllemmc_ncode_t); /* To setting PLLEMMC_NCODE_T */
			cr_maskl(EMMC_PLL_SET,0x001UL,17,sdiv2);    /* PLLSDIV2 = 0, 27x(19+3)/3/2=99MHz */
		}
		cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,1);	/* PLLEMMC_OC_EN = 1 */
	}

	while(loop--){  /* polling PLLEMMC_Oc_done flag */
		if(cr_readl(EMMC_PLL_SSC6) & _BIT4){
			break;
		}
	}

	cr_maskl(EMMC_PLL_SSC0,0x001UL,0 ,0);	/* PLLEMMC_OC_EN = 0 */
	udelay(150);
	cr_maskl(EMMC_PLL_CTRL,0x001UL,24 ,0);	/* release PLLEMMC_FREEZE */

	if(!loop){
		err = -1;
		goto OUT;
	}

	/* To enable SSC, just care about PLL clock, no matter divider or else. */
	if(pll_sta & SSC_EN){
		dw_em_set_PLL_SSC(ncode_ssc, fcode_ssc ,ssc_rang);
	}

	sdport->cur_PLL = targ_clk;

OUT:
	//crt sync reset
	cr_writel(RSTN_EM_DW,RTK_EM_DW_SRST);
	cr_writel((CLKEN_EM_DW | 0x01U),RTK_EM_DW_CLKEN);
	udelay(10);
	cr_writel(CLKEN_EM_DW,RTK_EM_DW_CLKEN);
	cr_writel((RSTN_EM_DW | 0x01UL),RTK_EM_DW_SRST);
	cr_writel((CLKEN_EM_DW | 0x01U),RTK_EM_DW_CLKEN);

	/* reset IP only */
	dw_em_reset_IP();
	udelay(10);

	/*config IP */
	dw_em_IP_config();

	dw_em_restore_pad(pad_set);

	/* restore previously IP setting */
	cr_writel(config1,EM_DWC_PWREN);
	dw_em_set_div(clk_div);
	cr_writel(config3,EM_DWC_CTYPE);
	cr_writel(config4,EM_DWC_UHS_REG);
	cr_writel(config5,EM_DWC_DBADDR);
	cr_writel(config6,EM_DWC_EMMC_DDR_REG);
	if(config7 & _BIT0){
		dw_em_set_ds_delay(sdport->cur_ds_tune);
	}
	cr_writel(config8,EM_WARP_STATUS0);

	return err;

}

/************************************************************************
 * usage: force sdport->cur_ds_tune = 0 to make sure 1st tune on ds-tune.
 ************************************************************************/
#if 0
int dw_em_auto_sd_tune(void)
{
	e_device_type* sdport = &emmc_card;
	u32 ds_tune;

	ds_tune = sdport->cur_ds_tune;
	mmcinfo("%s(%d)ds_tune=0x%08x\n",__func__,__LINE__,ds_tune);
	/*
	 enable DS-tune
	 Enable of half-cycle calibration
	 */
	cr_maskl(EM_HALF_CYCLE_CAL_EN,1,18 ,0); 	/* fw_mode is trigged by rising */
	udelay(5);
	cr_maskl(EM_DS_TUNE_CTRL,0x1f,15 ,ds_tune);  	/* fill result read from  0xb8010890 */

	/* pre-setting fine-tune value *** */
	/* half_cycle_res_mod = 2; "1=>minus";"2=>plus" */
	if(sdport->mid == 0x15){
		/* set half_cycle_res_thr = -1 for Samsung*/
		cr_maskl(EM_HALF_CYCLE_CAL_EN,3,1 ,1);
		cr_maskl(EM_HALF_CYCLE_CAL_EN,0x1f,3 ,1);
	}else{
		/* set half_cycle_res_thr = +3 */
		cr_maskl(EM_HALF_CYCLE_CAL_EN,3,1 ,2);
		cr_maskl(EM_HALF_CYCLE_CAL_EN,0x1f,3 ,3);
	}

	/* enable auto ds tune */
	cr_maskl(EM_DS_TUNE_CTRL,1,20 ,1);	/* bypass SW mode */
	cr_maskl(EM_HALF_CYCLE_CAL_EN,1,18 ,1);	/*enable half cycle calibrate mode*/
	cr_maskl(EM_HALF_CYCLE_CAL_EN,1,0 ,1);

	udelay(5);
	cr_maskl(EM_HALF_CYCLE_CAL_EN,1,18 ,0); /* fw_mode is trigged by rising */

	mmcinfo("%s(%d)ds_tune=0x%08x\n",
		 __func__,__LINE__,sdport->cur_ds_tune);
	mmcinfo("0x%08x=0x%08x 0x%08x=0x%08x\n",
		0xb801088c,cr_readl(0xb801088c),
		0xb8010890,cr_readl(0xb8010890));

	return 0;
}
#endif

/************************************************************************
 *
 ************************************************************************/
int dw_em_dis_sd_tune(void)
{
	e_device_type* sdport = &emmc_card;
	u32 reginfo;

	reginfo = cr_readl(EM_HALF_CYCLE_CAL_EN);

	if(reginfo & _BIT0){
		/* record ds-tune value */
		sdport->cur_ds_tune = cr_readl(EM_HALF_CYCLE_CAL_RESULT) & 0x1f;

		cr_maskl(EM_HALF_CYCLE_CAL_EN,1,0 ,0);	/* disable DS-tune */
		mmcinfo("%s(%d)ds_tune=0x%08x\n",__func__,__LINE__,sdport->cur_ds_tune);
		mmcinfo("0x%08x=0x%08x 0x%08x=0x%08x\n",
			0xb801088c,cr_readl(0xb801088c),
			0xb8010890,cr_readl(0xb8010890));
	}else{
		mmcinfo("%s(%d)skip\n",__func__,__LINE__);
	}
	return 0;
}

/************************************************************************
 *
 ************************************************************************/
#define USE_FIXED_W_PHASE
int dw_em_set_DDR400(u32 ddr_mode)
{
	u32 reginfo;
	u32 ddr_targ;

	/* non-DDR mode, SW 1.8V */
	cr_writel(_BIT0, EM_DWC_UHS_REG);

	reginfo = cr_readl(EM_DWC_EMMC_DDR_REG);

	if (ddr_mode) {
		ddr_targ = _BIT31;
	} else {
		ddr_targ = 0;
	}

	if ((reginfo & (_BIT31)) != ddr_targ) {
		reginfo = 0;

		if (ddr_targ) {
			reginfo |= ddr_targ;
		} else {
			/* Do nothing */
		}
		mmcinfo("set phase 0x%x=0x%08x\n",EMMC_PLLPHASE_CTRL,cr_readl(EMMC_PLLPHASE_CTRL));
		cr_writel(reginfo, EM_DWC_EMMC_DDR_REG);
	}else{
		/* nothing to do */
	}

	return 0;

}

/************************************************************************
 *
 ************************************************************************/
int dw_em_set_DDR(u32 ddr_mode)
{
	u32 reginfo;
	u32 ddr_targ;

	/*
	 * If DDR400 enable last time, disable it.
	 */
	cr_writel(0, EM_DWC_EMMC_DDR_REG);

	reginfo = cr_readl(EM_DWC_UHS_REG);

	if (ddr_mode) {
		ddr_targ = (_BIT16 | _BIT0);
	} else {
		ddr_targ = 0;
	}

	if ((reginfo & (_BIT16|_BIT0)) == ddr_targ) {
		/* do nothing */
	} else {
		reginfo &= ~(_BIT16 | _BIT0);

		if (ddr_targ) {
			reginfo |= ddr_targ;
		} else {
			/* Do nothing */
		}
		cr_writel(reginfo, EM_DWC_UHS_REG);
	}

	return 0;

}

/************************************************************************
 *
 ************************************************************************/
unsigned int dw_em_set_bits(u32 bus_width)
{
	e_device_type* sdport = &emmc_card;
	u32 reginfo = 0;
	u32 bit_targ;

	reginfo = cr_readl(EM_DWC_CTYPE);

	if(bus_width & BUS_WIDTH_8){
		sdport->cur_width = 8;
		if (reginfo & _BIT16) {
			goto skip_out;
		} else {
			UPRINTF("%s: 8 bits mode\n",DRIVER_NAME);
			bit_targ = _BIT16;
		}
	}else if(bus_width & BUS_WIDTH_4){
		sdport->cur_width = 4;
		if (reginfo & _BIT0) {
			goto skip_out;
		}else{
			UPRINTF("%s: 4 bits mode\n",DRIVER_NAME);
			bit_targ = _BIT0;

		}
	}else{
		sdport->cur_width = 1;
		if (reginfo & (_BIT16 | _BIT0)) {
			UPRINTF("%s: 1 bits mode\n",DRIVER_NAME);
			bit_targ = 0;
		} else {
			goto skip_out;
		}
	}
	reginfo &= ~(_BIT16 | _BIT0);
	reginfo |= bit_targ;

	cr_writel( reginfo,EM_DWC_CTYPE);

skip_out:

	return 0;
}

/************************************************************************
 * Y:\nfs\kernel\macarthur3\linux\linux-3.10.0\drivers\mmc\host\dw_mmc.c
 * static u32 dw_mci_prepare_command(struct mmc_host *mmc, struct mmc_command *cmd)
 ************************************************************************/
static
unsigned int dw_em_prepare_command(struct rtk_cmd_info * cmd_info)
{
	u32 cmd_idx  = 0;
	u32 cmd_flag = 0;
	u32 rsp_typ  = 0;
	u32 cmd_typ  = 0;

	cmd_idx = (u32)cmd_info->cmd->opcode;
	rsp_typ = (u32)cmd_info->rsp_para;
	cmd_typ = (u32)cmd_info->xfer_flag;
//#define SHOW_PREPARE_CMD
#ifdef SHOW_PREPARE_CMD
	mmcinfo("cmd_idx=0x%08x rsp_typ=0x%08x cmd_typ=0x%08x\n",
		 cmd_idx,rsp_typ,cmd_typ);
#endif
	cmd_flag = cmd_idx;

#if 1
	if (cmd_idx == MMC_STOP_TRANSMISSION)
		cmd_flag |= SDMMC_CMD_STOP;
	else{
		cmd_flag |= SDMMC_CMD_PRV_DAT_WAIT;
	}
#else
	if (cmd_idx == MMC_STOP_TRANSMISSION)
		cmd_flag |= SDMMC_CMD_STOP;

	if (cmd_idx != MMC_SEND_STATUS)
		cmd_flag |= SDMMC_CMD_PRV_DAT_WAIT;
#endif
	/* before idle cmd, send 80 clk plus first */
	if(cmd_idx == MMC_GO_IDLE_STATE)
		cmd_flag |= SDMMC_CMD_INIT;

	/* check resp type */
	if(rsp_typ & MMC_RSP_PRESENT){

		cmd_flag |= (SDMMC_CMD_RESP_EXP);

		if(rsp_typ & MMC_RSP_136)
			cmd_flag |= (SDMMC_CMD_RESP_LONG);

	}

	/*
	 Cmd13 skip resp CRC check.
	 reference JESD84-B50-1.pdf; 6.6.2 High-speed modes selection
	 */
#if 0
	/* Because hard to judge what time is proper disable CRC time,
	 Skip all time here */
	if( (rsp_typ & MMC_RSP_CRC)
	 && (cmd_idx != MMC_SEND_STATUS) )
#else
	/* add judge flag to check enable CRC or not */
	if(rsp_typ & MMC_RSP_CRC)
#endif
	{
		cmd_flag |= (SDMMC_CMD_RESP_CRC);
	}

	/* check cmd type ADTC? */
	if(cmd_typ )
	{
		cmd_flag |= SDMMC_CMD_DAT_EXP;

		/* check data write/read */
		if(cmd_typ == RTK_MMC_DATA_WRITE)
			cmd_flag |= SDMMC_CMD_DAT_WR;

	}

#ifdef AUTO_SEND_STOP
	/* auto send stop cmd */
	if((cmd_idx == MMC_READ_MULTIPLE_BLOCK)
	  |(cmd_idx == MMC_WRITE_MULTIPLE_BLOCK))
	{
		cmd_flag |= SDMMC_CMD_SEND_STOP;
	}
#endif

	cmd_flag |= SDMMC_CMD_USE_HREG;
	cmd_info->cmd_flag = cmd_flag;

#ifdef SHOW_PREPARE_CMD
	 mmcinfo("cmd_flag=0x%08x\n", cmd_flag);
#endif
	return cmd_flag;
}

/************************************************************************
 * for elapse time caculate:
 *	tick = get_boot_time_us();
 *	elapsed = get_boot_time_us() - tick;
 * Call this func at any place you want to know how many time emmc consumption
 ************************************************************************/
#ifdef ELAPSE_TIME_CACULATE
void display_emmc_xfer_info(void)
{
	printf("\n==========EMMC Xfer info=========================================\n");
	printf("mmc_R_xfer_us=%d mmc_W_xfer_us=%d mmc_C_xfer_us=%d\n",
		mmc_R_xfer_us,mmc_W_xfer_us,mmc_C_xfer_us);
	printf("Total_xfer_us=%d \n",(mmc_R_xfer_us+mmc_W_xfer_us+mmc_C_xfer_us));
	printf("mmc_R_xfer_blk=%d mmc_W_xfer_blk=%d\n",
		mmc_R_xfer_blk,mmc_W_xfer_blk);
	printf("=================================================================\n\n");
}
#endif

/************************************************************************
 *
 ************************************************************************/
#define IDMAC_NORMAL_INT    (SDMMC_IDMAC_INT_NI|SDMMC_IDMAC_INT_RI|SDMMC_IDMAC_INT_TI)
#define IDMAC_INT_ERR	    (SDMMC_IDMAC_INT_AI|SDMMC_IDMAC_INT_CES| \
			     SDMMC_IDMAC_INT_DU|SDMMC_IDMAC_INT_FBE)
static
int dw_em_TriggerXferCmd(unsigned int cmd_flag, struct rtk_cmd_info *cmd_info)
{
	e_device_type* sdport   = &emmc_card;
	int ret_err;
	unsigned int    cmd_idx;
	unsigned int    cmd_arg;
	unsigned int    loops;
	unsigned int 	int_status = 0;
	unsigned int 	sd_trans = 0;
	unsigned int 	sd_status = 0;
	unsigned int 	wrap_sta = 0;
	unsigned int    wrap_sta0 = 0;
	unsigned int    wrap_sta1 = 0;
	unsigned int 	*rsp;
#ifdef ELAPSE_TIME_CACULATE
	unsigned int tick, elapsed;
#endif

#ifdef ELAPSE_TIME_CACULATE
	tick = get_boot_time_us();
#endif

	cmd_idx = SDMMC_CMD_INDX(cmd_flag);
	cmd_arg = cr_readl(EM_DWC_CMDARG);
	rtk_record_cmd_flow(cmd_idx,cmd_arg);
	rsp = cmd_info->rsp;

	cr_writel(cr_readl(EM_INT_STATUS)|(_BIT1|_BIT2), EM_INT_STATUS);

	/* disable and clear int */
	dw_em_clear_int_sta(1);

#ifdef CACHE_HANDLE_INNER
	//flush cache before start xfer
	if(cmd_info->xfer_flag == RTK_MMC_DATA_READ){
		flush_cache((unsigned long)(cmd_info->data_buffer), cmd_info->byte_count*cmd_info->block_count);
	}else if(cmd_info->xfer_flag == RTK_MMC_DATA_WRITE){
		flush_cache((unsigned long)(cmd_info->data_buffer), cmd_info->byte_count*cmd_info->block_count);
	}
#endif
	cr_writel(cmd_flag|SDMMC_CMD_START, EM_DWC_CMD);

	ret_err = -19; // TIMEOUT ( defined in include/mmc.h )

	if(cmd_info->timeout)
		loops = cmd_info->timeout;
	else
		loops = 3000;    /* wait 3s */

	loops = loops*100;
	while( loops ){
		int_status	= cr_readl(EM_DWC_RINTSTS);
		sd_trans	= cr_readl(EM_DWC_IDSTS);
		sd_status	= cr_readl(EM_DWC_STATUS);
		wrap_sta	= cr_readl(EM_INT_STATUS);
		wrap_sta0       = cr_readl(EM_WARP_STATUS0);
		wrap_sta1       = cr_readl(EM_WARP_STATUS1);

		if(cr_readl(EM_DWC_CMD) & _BIT31){
			goto TRY_LOOP;
		}

		if((int_status & SDMMC_INT_ERROR)) {
			if( (cmd_idx == MMC_SEND_OP_COND)
			 && (int_status & SDMMC_INT_RESP_ERR))
			{
			 	/* cmd1 skip response error */
			}
			else{
				ret_err = -17;
				break;
			}
		}

		if(cmd_flag & SDMMC_CMD_DAT_EXP)
		{
			if(wrap_sta & 0x10){
				if(rtkmmc_debug_msg){
					/*
					  for secure issue,
					  we don't want user to know which region was protected.
					  so skip log print out.
					*/
					printf("protect region hit!!!\n");
				}
				ret_err = 0;
				break;
			}

			if(sd_trans & IDMAC_INT_ERR ){
				ret_err = -18;
				break;
			}
			if(int_status & SDMMC_INT_DATA_OVER){
				if (cmd_flag & SDMMC_CMD_DAT_WR){
					if( !(sd_status & SDMMC_DAT0_BUSY)
					 && !(sd_trans & (0xfUL << 13))
					 && !(wrap_sta0 & _BIT1))
					{
						ret_err = 0;
						break;
					}
				}else{
					if( (wrap_sta & PS_DMA_WR_DONE_S)
					 && !(sd_trans & (0xfUL << 13))
					 && !(wrap_sta0 & _BIT0)
					 && !(wrap_sta0 & _BIT1))
					 {
						/* At read case, waitting inner DDR done.
						   Read action is writting data to DDR.
						   that is why to use PS_DMA_WR_DONE_S */
						ret_err = 0;
						break;
					}
				}
			}
		}
		else
		{
			if(int_status & SDMMC_INT_CMD_DONE){
				ret_err = 0;
				break;
			}
		}
TRY_LOOP:
		udelay(10);
		loops--;
	}

	cr_writel(int_status,EM_DWC_RINTSTS); /* clear int status */
	cr_writel(sd_trans,EM_DWC_IDSTS);
	cr_writel(sd_status,EM_DWC_STATUS);
	cr_writel(wrap_sta,EM_INT_STATUS);

#define RESP_ERROR	(SDMMC_INT_RESP_ERR|SDMMC_INT_RCRC|SDMMC_INT_RTO)

	if(int_status & RESP_ERROR){
		rsp[0] = 0xffffffff;
	}else{
		int rsp_err = 0;

		rsp_err = dw_em_chk_rsp(rsp, cmd_flag, cmd_info);
		if(rsp_err && cmd_flag & SDMMC_CMD_SEND_STOP){
			/* It's needed to handle error of resp at case to enable auto cmd12  */
			if(rsp_err & _BIT1){
				ret_err |= rsp_err;
			}
		}
	}

	if( ret_err) {
		if(!(sdport->rtflags & RTKCR_TUNING_STATE)) {
			rtk_display_cmd_flow();
			notice_reduce_speed(cmd_idx);
			UPRINTF("Cmd fail!!!\n");
			UPRINTF("%s(%d)cmd_idx=%d cmd_arg=0x%08x cmd_flag=0x%x loop_cnt=0x%x\n"
				"        RINTSTS=0x%08x IDSTS  =0x%08x\n"
				"        STATUS =0x%08x INT_STATUS =0x%08x\n"
				"        WARP_STATUS0=0x%08x  WARP_STATUS1=0x%08x\n",
				__func__,__LINE__,
				cmd_idx,cmd_arg,cmd_flag,cmd_info->timeout,
				int_status,sd_trans,sd_status,wrap_sta,
				wrap_sta0,wrap_sta1);
		}

#define GET_DBUS_SMA(value)	((value>>16)&0x07)
#define GET_AHB_SMA(value)	(value & 0x7FFF)

		/* The RESET is must be. Don't mix with message print.  */
		if((cmd_flag & SDMMC_CMD_DAT_EXP)
		 &&((GET_DBUS_SMA(wrap_sta1)!=1) || (GET_AHB_SMA(wrap_sta1)!=1)))
		 {
			/* if wrap state machine not idle, reset wrap */
			dw_em_reset_host(1);
		}else{
			dw_em_reset_host_IP(1);
		}
	}else{
		if(sdport->rtflags & RTKCR_TUNING_STATE){
			dw_em_reset_host_IP(1);
		}
	}

#ifdef ELAPSE_TIME_CACULATE
	elapsed = get_boot_time_us() - tick;

	if(cmd_flag & SDMMC_CMD_DAT_EXP){
		if (cmd_flag & SDMMC_CMD_DAT_WR){
			mmc_W_xfer_us+=elapsed;
			mmc_W_xfer_blk+=cmd_info->block_count;
		}else{
			mmc_R_xfer_us+=elapsed;
			mmc_R_xfer_blk+=cmd_info->block_count;
		}
	}else{
		mmc_C_xfer_us+=elapsed;
	}
#endif

	return ret_err;
}

/************************************************************************
 *
 ************************************************************************/
int dw_em_SendCmd( unsigned char  cmd_idx,
				   unsigned int   cmd_arg,
				   unsigned int   rsp_type,
				   unsigned int * rsp)
{
	e_device_type* sdport = &emmc_card;
	int ret_err = 0;
	unsigned int cmd_flag;
	struct rtk_cmd_info cmd_info;
	struct rtk_mmc_command cmd;
	//u32 cpuID;
	//cpuID = get_cpu_id();

	dw_em_get_cpu_occupy(&cmd_info);

	cmd_info.cmd = &cmd;
	cmd.arg      = cmd_arg;
	cmd.opcode   = cmd_idx;

	cmd_info.rsp_para  = rsp_type;
	cmd_info.xfer_flag = 0;
	cmd_info.timeout   = 0;
	cmd_info.rsp       = rsp;

	cmd_flag = dw_em_prepare_command(&cmd_info);
	if(cmd_idx == 23)
		sdport->rtflags |= RTKCR_CMD23_ALERT;

	if(rtkmmc_debug_msg){
		printf( "cmd_idx=%d cmd_arg=0x%08x cmd_flag=0x%08x rsp_type=0x%08x%s\n",
			 cmd_idx,cmd_arg,cmd_flag,rsp_type,
			 (rsp_type &  MMC_RSP_BUSY)?"[R1b]":"");
	}

	cr_writel(cmd_arg,          EM_DWC_CMDARG);


	if(rsp_type &  MMC_RSP_BUSY) {
		/* wait 10s for wait-response-type */
		cmd_info.timeout = 10000;
	} else {
		cmd_info.timeout = 10;
	}

	ret_err = dw_em_TriggerXferCmd(cmd_flag, &cmd_info);

	if( ret_err ){
		/* do nothing now */
		UPRINTF( "MMC: %s fail\n",__func__);
	}
	else{
		/* TO get manufacturer ID here */
		if(cmd_idx == MMC_ALL_SEND_CID ){
			sdport->mid = ((rsp[0]>>24) & 0xffU);
		}
	}

	dw_em_release_cpu_occupy(&cmd_info);

	return ret_err;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_send_stop(e_device_type * card)
{
	unsigned int rsp_buffer[4];
	int ret_err = 0;
	unsigned int rca;

	if(card)
		rca = card->rca;
	else
		rca = 0;

	ret_err = dw_em_SendCmd(MMC_STOP_TRANSMISSION,
			rca,  //if HPI have not set, RCA is unnecessary. "sdport.rca,"
			MMC_RSP_R1b,
			rsp_buffer);

	return ret_err;
}

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_reinit_idmac_pre(struct rtk_cmd_info * cmd_info)
{
	unsigned int ctrl;

	cr_writel(SDMMC_SWR,EM_DWC_BMOD);

	/* Mask out interrupts - get Tx & Rx complete only */
	ctrl = (SDMMC_IDMAC_INT_NI|SDMMC_IDMAC_INT_RI|SDMMC_IDMAC_INT_TI);
	cr_writel(ctrl, EM_DWC_IDINTEN);

	dw_em_set_scrip_base((unsigned int)dw_em_scrip);

	/* Select IDMAC interface */
	ctrl = cr_readl(EM_DWC_CTRL);
	ctrl |= SDMMC_CTRL_USE_IDMAC;
	cr_writel(ctrl, EM_DWC_CTRL);

	/* Enable the IDMAC */
	ctrl = cr_readl(EM_DWC_BMOD);
	ctrl |= SDMMC_IDMAC_ENABLE | SDMMC_IDMAC_FB;
	cr_writel(ctrl, EM_DWC_BMOD);

	/* Start it running */
	cr_writel(1, EM_DWC_PLDMND);

	return 0;
}

/************************************************************************
 *
 ************************************************************************/
#ifdef RE_INITIAL_IDMAC
static
int dw_em_reinit_idmac_post(struct rtk_cmd_info * cmd_info)
{
	u32 temp;

	/* Disable and reset the IDMAC interface */
	temp = cr_readl(EM_DWC_CTRL);
	temp &= ~SDMMC_CTRL_USE_IDMAC;
	temp |= SDMMC_CTRL_DMA_RESET;
	cr_writel(temp, EM_DWC_CTRL);

	/* Stop the IDMAC running */
	temp = cr_readl(EM_DWC_BMOD);
	temp &= ~(SDMMC_IDMAC_ENABLE | SDMMC_IDMAC_FB);
	cr_writel(temp, EM_DWC_BMOD);

	return 0;
}
#endif

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_Stream_Cmd(struct rtk_cmd_info * cmd_info)
{
	e_device_type* sdport   = &emmc_card;
	int ret_err = 0;
	unsigned int cmd_flag;
	//unsigned int cmd_idx    = cmd_info->cmd->opcode;
	unsigned int cmd_arg    = cmd_info->cmd->arg;
	unsigned int * rsp      = cmd_info->cmd->resp;
	unsigned int byte_cnt   = cmd_info->byte_count;
	unsigned int blk_cnt    = cmd_info->block_count;
	//u32 cpuID;
	//cpuID = get_cpu_id();

	dw_em_get_cpu_occupy(cmd_info);

#ifdef  RE_INITIAL_IDMAC
	dw_em_reinit_idmac_pre(cmd_info);
#endif

	if(sdport->rtflags & RTKCR_TUNING_STATE)
		cmd_info->timeout = 5;
	else
		cmd_info->timeout = 2500;

	cmd_info->rsp       = rsp;

	dw_em_make_script(cmd_info);

	cmd_flag = dw_em_prepare_command(cmd_info);

	if(sdport->rtflags & RTKCR_CMD23_ALERT){
		sdport->rtflags &= ~RTKCR_CMD23_ALERT;
		cmd_flag &= ~SDMMC_CMD_SEND_STOP;
	}

	if(rtkmmc_debug_msg){
		printf( "cmd_idx=%u cmd_arg=0x%08x cmd_flag=0x%08x rsp_type=0x%08x\n"
			"        byte_cnt=0x%08x blk_cnt=0x%08x phy_buf_addr=0x%08x\n",
			cmd_info->cmd->opcode, cmd_arg, cmd_flag, cmd_info->rsp_para,
			byte_cnt, blk_cnt, (unsigned int)cmd_info->data_buffer);
	}

	cr_writel(cmd_arg,          EM_DWC_CMDARG);
	cr_writel(byte_cnt,         EM_DWC_BLKSIZ);
	cr_writel(byte_cnt*blk_cnt, EM_DWC_BYTCNT);

	ret_err = dw_em_TriggerXferCmd(cmd_flag, cmd_info);

	if( !ret_err ){
		/* do nothing now */
	}
	else{
		ret_err = -1;
	}

#ifdef  RE_INITIAL_IDMAC
	dw_em_reinit_idmac_post(cmd_info);
#endif

	dw_em_release_cpu_occupy(cmd_info);

	return ret_err;

}

/************************************************************************
 *
 ************************************************************************/
int dw_em_re_tuning( unsigned int ctl)
{
	u32 target_clk;
	u32 timing;
	e_device_type* sdport = &emmc_card;
	unsigned int driving = 0;

	/* decrease speed */
	timing = sdport->mmc->timing;
	target_clk = (sdport->cur_clock - CLK_20Mhz);

	if(timing == MMC_TIMING_MMC_HS400){
		driving = DRIVING_HS400;
		if(target_clk < CLK_100Mhz)
			target_clk = CLK_100Mhz;
	}else if(timing == MMC_TIMING_MMC_HS200){
		driving = DRIVING_HS200;
		if(target_clk < CLK_100Mhz)
			target_clk = CLK_100Mhz;
	}else{
		driving = DRIVING_COMMON;
		if(target_clk < CLK_20Mhz)
			target_clk = CLK_20Mhz;
	}

	dw_em_set_IO_driving(driving);
	dw_em_set_clk(target_clk);

	/* re-tuning phase  */
	return 0;
}

/************************************************************************
 * Note:
 *    The pointer is pervious cmd info.
 *    if any info should be return to previous cmd, return here.
 ************************************************************************/
int dw_em_state_recover(struct rtk_cmd_info* pre_cmd_info)
{
	e_device_type* sdport = &emmc_card;
	unsigned int rsp[4] = {0};
	unsigned int curr_state = 0;

	//dw_em_send_dummy_clk(3);
	if(dw_em_wait_status(sdport,rsp,1) >= 0){
		curr_state = R1_CURRENT_STATE(rsp[0]);
		UPRINTF("%s(%d) curr_state : %s[%d]\n",
			__func__,__LINE__,
			(curr_state >= STA_EVENT_LEN)? "UNKNOW STATE":sta_evevt_tlb[curr_state],
			curr_state);
	}
	if((curr_state == STATE_RCV) || (curr_state == STATE_DATA)){
		dw_em_send_stop(sdport);
		//dw_em_send_dummy_clk(5);
	}else if(curr_state == STATE_PRG){
		if(dw_em_wait_status(sdport,rsp,5))
			UPRINTF("mmc TRAN status fail!\n");
	}
	return 0;
}

/************************************************************************
 *
 ************************************************************************/
int dw_em_Stream(
	struct mmc_cmd * mcmd,
	struct mmc_data * data,
	unsigned int xfer_flag)
{
	e_device_type* sdport = &emmc_card;
	struct rtk_cmd_info cmd_info;
	struct rtk_mmc_command cmd;
	unsigned char cmd_idx = mcmd->cmdidx;
	unsigned int cmd_arg = mcmd->cmdarg;
	unsigned int rsp_type = mcmd->resp_type;
	unsigned int blk_cnt = data->blocks;
	unsigned int blk_size = data->blocksize;
	unsigned int *data_buffer = (unsigned int *)data->dest;
	unsigned int retry = 0;
	int err = -1;

#if (defined(Config_Module_Test_Boot_TRUE) || defined(Config_FW_Type_FT))
#define MAX_RETRY_CNT 0
#else
#define MAX_RETRY_CNT 6
#endif
	retry = MAX_RETRY_CNT;
RETRY:
	cmd_info.cmd = &cmd;
	cmd.arg      = cmd_arg;
	cmd.opcode   = cmd_idx;

	cmd_info.rsp_para    = rsp_type;
	cmd_info.byte_count  = blk_size;
	cmd_info.block_count = blk_cnt;
	cmd_info.data_buffer = (unsigned char *)data_buffer;
	cmd_info.xfer_flag   = xfer_flag;

	err = dw_em_Stream_Cmd(&cmd_info );

	if(err ){
		if(retry){
			UPRINTF("%s fail(retry %d)\n",__func__,retry);
			dw_em_state_recover(&cmd_info);
			retry--;

			if( !(sdport->rtflags & RTKCR_TUNING_STATE))
			{
#if (defined(Config_Module_Test_Boot_TRUE) || defined(Config_FW_Type_FT))
				/* Skip down speed. */
#else
				if(!(retry%2) && (retry != MAX_RETRY_CNT))
					dw_em_re_tuning(0);
#endif
				goto  RETRY;
			}
		}else{
			UPRINTF("%s retry %d fail\n",__func__,retry);
		}
	}

	return err;

}

#define TUNING_ENABLE
#ifdef TUNING_ENABLE
/************************************************************************
 * For reference
 *
 * 4Bits Pattern
 * [000]=ff [001]=0f [002]=ff [003]=00 [004]=ff [005]=cc [006]=c3 [007]=cc
 * [008]=c3 [009]=3c [010]=cc [011]=ff [012]=fe [013]=ff [014]=fe [015]=ef
 * [016]=ff [017]=df [018]=ff [019]=dd [020]=ff [021]=fb [022]=ff [023]=fb
 * [024]=bf [025]=ff [026]=7f [027]=ff [028]=77 [029]=f7 [030]=bd [031]=ef
 * [032]=ff [033]=f0 [034]=ff [035]=f0 [036]=0f [037]=fc [038]=cc [039]=3c
 * [040]=cc [041]=33 [042]=cc [043]=cf [044]=ff [045]=ef [046]=ff [047]=ee
 * [048]=ff [049]=fd [050]=ff [051]=fd [052]=df [053]=ff [054]=bf [055]=ff
 * [056]=bb [057]=ff [058]=f7 [059]=ff [060]=f7 [061]=7f [062]=7b [063]=de
 *
 * =======================================================================
 * 8Bits Pattern
 * [000]=ff [001]=ff [002]=00 [003]=ff [004]=ff [005]=ff [006]=00 [007]=00
 * [008]=ff [009]=ff [010]=cc [011]=cc [012]=cc [013]=33 [014]=cc [015]=cc
 * [016]=cc [017]=33 [018]=33 [019]=cc [020]=cc [021]=cc [022]=ff [023]=ff
 * [024]=ff [025]=ee [026]=ff [027]=ff [028]=ff [029]=ee [030]=ee [031]=ff
 * [032]=ff [033]=ff [034]=dd [035]=ff [036]=ff [037]=ff [038]=dd [039]=dd
 * [040]=ff [041]=ff [042]=ff [043]=bb [044]=ff [045]=ff [046]=ff [047]=bb
 * [048]=bb [049]=ff [050]=ff [051]=ff [052]=77 [053]=ff [054]=ff [055]=ff
 * [056]=77 [057]=77 [058]=ff [059]=77 [060]=bb [061]=dd [062]=ee [063]=ff
 * [064]=ff [065]=ff [066]=ff [067]=00 [068]=ff [069]=ff [070]=ff [071]=00
 * [072]=00 [073]=ff [074]=ff [075]=cc [076]=cc [077]=cc [078]=33 [079]=cc
 * [080]=cc [081]=cc [082]=33 [083]=33 [084]=cc [085]=cc [086]=cc [087]=ff
 * [088]=ff [089]=ff [090]=ee [091]=ff [092]=ff [093]=ff [094]=ee [095]=ee
 * [096]=ff [097]=ff [098]=ff [099]=dd [100]=ff [101]=ff [102]=ff [103]=dd
 * [104]=dd [105]=ff [106]=ff [107]=ff [108]=bb [109]=ff [110]=ff [111]=ff
 * [112]=bb [113]=bb [114]=ff [115]=ff [116]=ff [117]=77 [118]=ff [119]=ff
 * [120]=ff [121]=77 [122]=77 [123]=ff [124]=77 [125]=bb [126]=dd [127]=ee
 *
 ************************************************************************/
#define TUNING_BLK_CNT 		1
#define TUNING_WRITE_FLAG	_BIT31
#define TUNING_SKIP_FLAG	_BIT30
#define TUNING_HS400		_BIT29
#define TUNING_DIR_UP		_BIT28
#define TUNING_SKIP_ALL		_BIT27
#define TUNING_BLK_SHT		6
#define TUNING_CMD_MASK		(0x3fUL)			/* bit0~bit5 */
#define TUNING_BLK_MASK		(0xfUL << TUNING_BLK_SHT)	/* bit6~bit9 */
#define TUNING_TPHASE_SHT	6
#define TUNING_TPHASE_MASK	(0x3fUL << TUNING_TPHASE_SHT)	/* bit6~bit11 */
#define GET_TUNING_CMD(opcode)	(opcode & TUNING_CMD_MASK)
#define GET_TUNING_BLK(opcode)	((opcode & TUNING_BLK_MASK) >> TUNING_BLK_SHT)
#define GET_TPHASE(opcode)	((opcode & TUNING_TPHASE_MASK) >> TUNING_TPHASE_SHT)
#define TUNING_BLK_ADR 	0x80 /* At write case, this address should be careful to erase data on eMMC */

static const unsigned char pattern_blk[64] = {
	0xff, 0x0f, 0xff, 0x00, 0xff, 0xcc, 0xc3, 0xcc,
	0xc3, 0x3c, 0xcc, 0xff, 0xfe, 0xff, 0xfe, 0xef,
	0xff, 0xdf, 0xff, 0xdd, 0xff, 0xfb, 0xff, 0xfb,
	0xbf, 0xff, 0x7f, 0xff, 0x77, 0xf7, 0xbd, 0xef,
	0xff, 0xf0, 0xff, 0xf0, 0x0f, 0xfc, 0xcc, 0x3c,
	0xcc, 0x33, 0xcc, 0xcf, 0xff, 0xef, 0xff, 0xee,
	0xff, 0xfd, 0xff, 0xfd, 0xdf, 0xff, 0xbf, 0xff,
	0xbb, 0xff, 0xf7, 0xff, 0xf7, 0x7f, 0x7b, 0xde,

};

/************************************************************************
 *
 ************************************************************************/
static
int dw_em_chk_pattern(u8 *data)
{
	e_device_type* sdport = &emmc_card;
	int i;
	int err = 0;

	if(sdport->cur_width == 8)
	{
		unsigned char nb1 = 0;
		unsigned char nb2 = 0;

		mmcmsg2("%dbits pattern check\n",sdport->cur_width);
		for(i=0; i<64; i++){
			nb1 = (pattern_blk[i] & 0xf0) >> 4;
			nb1 = (nb1<<4)|nb1;
			nb2 = (pattern_blk[i] & 0xf);
			nb2 = (nb2<<4)|nb2;

			if((data[i*2] != nb1) || (data[(i*2)+1] != nb2)){
				mmcmsg2("nb1=0x%x nb2=0x%x \n",nb1,nb2);
				mmcmsg2("data[%x]=0x%x data[%x]=0x%x \n",
					i,data[i*2],i+1,data[(i*2)+1]);
				err = -1;
				break;
			}
		}

	} else {
		mmcmsg2("%dbits pattern check\n",sdport->cur_width);
		for(i=0; i<64; i++){
			if(data[i] != pattern_blk[i]){
				mmcmsg2("data[%x]=0x%x pattern_blk[%x]=0x%x \n",
					i,data[i],i,pattern_blk[i]);
				err = -1;
				break;
			}
		}
	}

	return err;
}

/************************************************************************
 *
 ************************************************************************/
static
void rtkem_show_tuning_result(e_device_type *sdport, u8* p,char* reason)
{
	UPRINTF("%s\n"
	"        {00-31}[%d%d%d%d %d%d%d%d %d%d%d%d %d%d%d%d% d%d%d%d %d%d%d%d %d%d%d%d %d%d%d%d]\n",
		reason,
		p[ 0],p[ 1],p[ 2],p[ 3],p[ 4],p[ 5],p[ 6],p[ 7],
		p[ 8],p[ 9],p[10],p[11],p[12],p[13],p[14],p[15],
		p[16],p[17],p[18],p[19],p[20],p[21],p[22],p[23],
		p[24],p[25],p[26],p[27],p[28],p[29],p[30],p[31]);
}

/************************************************************************
 *
 ************************************************************************/
static
void dw_em_set_ds_delay(int ds_delay)
{
    e_device_type* sdport = &emmc_card;

    cr_maskl(EM_HALF_CYCLE_CAL_EN, 1, 18,0);
    udelay(5);
    cr_maskl(EM_DS_TUNE_CTRL, 0x1f, 15, sdport->cur_ds_tune);
    cr_maskl(EM_HALF_CYCLE_CAL_EN, 3, 1, 3);
    cr_maskl(EM_HALF_CYCLE_CAL_EN, 0x1f,3 , ds_delay);
    cr_maskl(EM_DS_TUNE_CTRL,1,20 ,1);
    cr_maskl(EM_HALF_CYCLE_CAL_EN,1,18 ,1);
    cr_maskl(EM_HALF_CYCLE_CAL_EN,1,0 ,1);
    udelay(5);
    cr_maskl(EM_HALF_CYCLE_CAL_EN,1,18 ,0);
#if 0
    UPRINTF("%s(%d):", __func__, __LINE__);
    UPRINTF("0x%08x=0x%08x 0x%08x=0x%08x 0x%08x=0x%08x\n",
		0xb801088c,cr_readl(0xb801088c),
		0xb8010890,cr_readl(0xb8010890),
        0xb8010850,cr_readl(0xb8010850));
#endif
}

static
int dw_em_ds_tuning(u32 ctl)
{
	e_device_type *sdport = &emmc_card;
	u16 i=0, j=0;
	int err=0;
	u16 max_range = 0;
	u16 min_ds = 0;
	u16 ds_middle = 0;
	u8 *test_buf;
	u8 result[MAX_DS_TUNE] = {0};

	test_buf = (u8 *)malloc((512 + 0x20));

	if (!test_buf) {
		UPRINTF("can't allocate memeory space\n");
		goto ERR;
	}

	for (i=0; i < MAX_DS_TUNE; i++) {
		dw_em_set_ds_delay(i);
		err = mmc_send_data_cmd(17, 1, 1, test_buf);
		// not care data accuracy

		if (!err)
			result[i] = 1;
	}

	rtkem_show_tuning_result(sdport,result,"DS tuning result: ");

	for (i=0; i < MAX_DS_TUNE; i++) {
		int circum = 0;

		for (j=i; j < i+MAX_DS_TUNE; j++) {
			int convert_j =  (j >= MAX_DS_TUNE)?
				(j-MAX_DS_TUNE) : j;
			if (result[convert_j])
				circum++;
			else
				break;
		}

		if (circum > max_range) {
			max_range = circum;
			min_ds = i;
		}
	}

    ds_middle = min_ds + (max_range)/2;

    if (max_range < 3) {
        UPRINTF("ds capable range %d is small\n", max_range);
        dw_em_set_ds_delay(sdport->cur_ds_tune);
        goto ERR;
    } else {
       dw_em_set_ds_delay(ds_middle);
       sdport->cur_ds_tune = ds_middle;
       UPRINTF("set ds %d\n", ds_middle);
    }

    free(test_buf);
    return 0;
ERR:
    if (test_buf)
        free(test_buf);
    return -1;
}

/************************************************************************
 *
 ************************************************************************/
u8 *emmc_buf = NULL;
#define TUNE_BLK_CNT 1
static
int dw_em_simple_tuning(u32 ctl)
{
	e_device_type* sdport = &emmc_card;

	int i, j;
	u32 tmp_phase = 0;
	u32 cur_phase = 0;
	u32 phase_sht = 0;
	u32 phase_mask = 0;
	u32 min_p = 0;
	u32 max_p = 0;
	u32 phase_diff = 0;
	u32 tune_cmd = 0;
	u8 w_flag = 0;
	u8 p[32] = {0}; //MAX_TUNING_STEP
	int err = 0;

	mmcmsg3( "cur bus width %d\n", sdport->cur_width );

	tune_cmd = GET_TUNING_CMD(ctl);

	if(ctl & TUNING_WRITE_FLAG)
		w_flag = 1;

	if(w_flag) {
		phase_sht = PHASE_W_SHT;
		phase_mask = PHASE_W_MASK;
	} else {
		phase_sht = PHASE_R_SHT;
		phase_mask = PHASE_R_MASK;
	}

	/* get more buffer for test */
	if(!emmc_buf){
		emmc_buf = (u8 *)malloc((TUNE_BLK_CNT*512) + 0x20);
		emmc_buf = (u8 *)((((uint)(emmc_buf))&(~0x1F)) + 0x20);
	}

	if(!emmc_buf){
		UPRINTF( "emmc_buf is NULL\n");
		return -1;
	}

	memset(emmc_buf, 0, TUNE_BLK_CNT*512);
	/* tuining start */
	mmcmsg3("PHASE_REG(0x%08x)=0x%08x\n",
		EMMC_PLLPHASE_CTRL, cr_readl(EMMC_PLLPHASE_CTRL));
	cur_phase = (cr_readl(EMMC_PLLPHASE_CTRL)>>phase_sht) & phase_mask;
	mmcmsg3("initial %s phase=%u(0x%x)\n",(w_flag)?"write":"read",cur_phase, cur_phase);
#define SCAN_LOW_TO_HIGH
#ifdef  SCAN_LOW_TO_HIGH
	for (i=0; i < MAX_TUNING_STEP; i++) {
#else
	i = MAX_TUNING_STEP;
	while(i--){
#endif
		tmp_phase = i;
		mmcmsg3( "tmp %s phase is %u(0x%x)\n",(w_flag)?"write":"read",tmp_phase,tmp_phase);

		if(!(ctl & TUNING_SKIP_FLAG))
			cr_maskl(EMMC_PLLPHASE_CTRL,phase_mask,phase_sht ,tmp_phase);

		sdport->rtflags |= RTKCR_TUNING_STATE;

		if(w_flag)
			mdelay(2);

		if(tune_cmd == 21) {
			err = mmc_send_data_cmd(21,0, 1, emmc_buf);
		} else {

			err = mmc_send_data_cmd(tune_cmd,
					TUNING_BLK_ADR,
					TUNING_BLK_CNT, emmc_buf);

			if(w_flag){
				mdelay(2);
			}
		}

		sdport->rtflags &= ~RTKCR_TUNING_STATE;

		if(err){
			mmcmsg3("tuning cmd fail!(phase %d)\n",i);
		}else{
			if(tune_cmd == 21) {
/* #define  EN_SHOW_DATA */
#ifdef  EN_SHOW_DATA
				show_data(emmc_buf,(sdport->cur_width == 8)? 128 : 64);
#endif
				err = dw_em_chk_pattern(emmc_buf);
				if(err){
					mmcmsg3("pattern check fail!(phase %d)\n",i);
				}
			}
		}

		if (!err){
			p[i] = true;
		}
	}

	if(ctl & TUNING_SKIP_FLAG){
		return 0;
	}else{
		UPRINTF("vaild phase [0~%d]\n",MAX_TUNING_STEP-1);
		rtkem_show_tuning_result(sdport,p,"tuning result: ");
		/*rtkem_show_tuning_result(sdport,p,"odd fix result: ");*/
	}

	/* find longest good phase circumference */
	for (i=0; i<MAX_TUNING_STEP; i++) {
		int circum = 0;

		for (j=i; j < i+MAX_TUNING_STEP; j++) {
			int convert_j =  (j >= MAX_TUNING_STEP)?
				(j-MAX_TUNING_STEP) : j;
			if (p[convert_j])
				circum++;
			else
				break;
		}

		if (circum > phase_diff) {
			phase_diff = circum;
			min_p = i;
		}
	}

	if (phase_diff < 8) {
		err = -EIO;
		UPRINTF("good range too small,roll back to original phase %d(0x%x)\n",
			cur_phase,cur_phase);

		tmp_phase = cur_phase;
		cr_maskl(EMMC_PLLPHASE_CTRL,phase_mask,phase_sht ,tmp_phase);
	}else{
		err = 0;

		if (ctl & TUNING_SKIP_FLAG) {
			UPRINTF("skip tuning!!!,keep original phase %d(0x%x)\n",
				cur_phase,cur_phase);

			/* keep sdport->cur_w_phase;cur_w_ph400;cur_r_phase same */
			//tmp_phase = cur_phase;
			//cr_maskl(EMMC_PLLPHASE_CTRL,phase_mask,phase_sht ,tmp_phase);
		}else{
			max_p = min_p + phase_diff;
			if(max_p >= MAX_TUNING_STEP)
				max_p -= MAX_TUNING_STEP;

			tmp_phase = min_p + phase_diff/2;

			if(tmp_phase >= MAX_TUNING_STEP)
				tmp_phase -= MAX_TUNING_STEP;


			cr_maskl(EMMC_PLLPHASE_CTRL,phase_mask,phase_sht ,tmp_phase);

			UPRINTF("%s: max_p=%u min_p=%u diff=%u\n",
				DRIVER_NAME,max_p,min_p,phase_diff);
			UPRINTF("============================================\n");
			UPRINTF("%s: final PHASE(%s) is %u; 0x%08x = 0x%08x\n",
				DRIVER_NAME,(w_flag)?"W":"R",tmp_phase,
				EMMC_PLLPHASE_CTRL,cr_readl(EMMC_PLLPHASE_CTRL));
			UPRINTF("ds_tune=%u(show only)\n",
				(cr_readl(EM_HALF_CYCLE_CAL_RESULT) & 0x1f));
			UPRINTF("============================================\n");

			if(w_flag){
				sdport->cur_w_phase = tmp_phase;
				sdport->cur_w_ph400 = min_p + phase_diff/4;
				if(sdport->cur_w_ph400 >= MAX_TUNING_STEP)
					sdport->cur_w_ph400 -= MAX_TUNING_STEP;
				mmcmsg3("cur_w_phase = 0x%08x\n",sdport->cur_w_phase);
				mmcmsg3("cur_w_ph400 = 0x%08x\n",sdport->cur_w_ph400);
			} else {

				sdport->cur_r_phase = tmp_phase;
				sdport->tud_r_phase = sdport->cur_r_phase;
				sdport->tud_r_clock = sdport->cur_clock;
				mmcmsg3("cur_r_phase = 0x%08x\n",sdport->cur_r_phase);
			}
		}
	}

	return err;
}

/************************************************************************
 *
 ************************************************************************/
int dw_em_execute_tuning(u32 opcode)
{
	e_device_type* sdport = &emmc_card;
	int err = 0;
	u32 tune_ctl = 0;
	u32 pre_ctl = 0;

	if( (opcode == 21)
	 || (opcode == 18) )
	{
		/*
		 opcode 21 is HS200 tuning state
		 opcode 18 is HS400 tuning state
		 */

		/* enable DS-tune for HS400 */
		tune_ctl = pre_ctl;
		if(tune_ctl & TUNING_SKIP_ALL){
#if 0 // [CID-313153] Logically dead code
			UPRINTF("%s(%d) per_ds_tune=%d\n",
				__func__,__LINE__,sdport->cur_ds_tune);
#endif
		}

		if(tune_ctl & TUNING_SKIP_ALL){
#if 0 // [CID-313153] Logically dead code
			UPRINTF("%s(%d)restore r_phase=%d\n",
				__func__,__LINE__,sdport->cur_r_phase);
			dw_em_init_phase(0xff,(u32)sdport->cur_r_phase,0xff); /* w_phase;r_phase */
			err = 0;
			goto ERR_OUT;
#endif
		}

		/* read tuning first at HS200 */
		if(opcode == 18){
			tune_ctl |= 17;
			tune_ctl |= (TUNING_SKIP_FLAG|TUNING_HS400);

			/* block R/W teset use 1 block for stability */
			tune_ctl |= (TUNE_BLK_CNT << TUNING_BLK_SHT);
			/* dw_em_simple_tuning( tune_ctl); */
			sdport->rtflags |= RTKCR_TUNING_STATE;
			dw_em_ds_tuning(tune_ctl);
			sdport->rtflags &= ~RTKCR_TUNING_STATE;
			err = 0; /* skip tuning at HS400 mode */
		}else{
			/* cmd21 only 1 block */
			tune_ctl |= 21;
			tune_ctl |= (TUNE_BLK_CNT << TUNING_BLK_SHT);
			err = dw_em_simple_tuning( tune_ctl);
		}

		if(err){
			goto ERR_OUT;
		}

#ifndef USE_FIXED_W_PHASE

		/* write tuning  */
		tune_ctl = pre_ctl;
		tune_ctl |= TUNING_WRITE_FLAG;
		if(opcode == 18){
			tune_ctl |= TUNING_HS400;
			tune_ctl |= 24;	// tuning cmd
			/* block R/W teset use 1 block for stability */
			tune_ctl |= (TUNE_BLK_CNT << TUNING_BLK_SHT);
			return 0;
		}else{
			/* cmd21 only 1 block */
			tune_ctl |= 21;	// tuning cmd
			tune_ctl |= (TUNE_BLK_CNT << TUNING_BLK_SHT);
		}

		err = dw_em_simple_tuning(tune_ctl);
		if(err){
			goto ERR_OUT;
		}
#endif
	}else{
		err = -1;
	}
ERR_OUT:
	return err;

}


/*******************************************************
 *
 *******************************************************/
static
int mmc_send_data_cmd(u32 hc_cmd,
	u32 cmd_arg, u32 blk_cnt,
	unsigned char *buffer)
{
	e_device_type* sdport = &emmc_card;
	struct rtk_cmd_info cmd_info = {0};
	struct rtk_mmc_command  cmd  = {0};
	int ret_err;

	mmcinfo("\n");

	cmd_info.cmd = &cmd;

	cmd.opcode   = hc_cmd;
	cmd.arg      = cmd_arg;

	cmd_info.rsp_para    = MMC_RSP_R1;
	cmd_info.block_count = blk_cnt;
	cmd_info.data_buffer = buffer;

	if( (hc_cmd == 21)
	 || (hc_cmd == 18)
	 || (hc_cmd == 17))
	{
		cmd_info.xfer_flag   = RTK_MMC_DATA_READ;
	}else
	if ((hc_cmd == 25)
	 || (hc_cmd == 24))
	{
		cmd_info.xfer_flag   = RTK_MMC_DATA_WRITE;
	}else{
		UPRINTF("cmd not support yet!!!\n");
		ret_err = -1;
		goto ERR_OUT;
	}

	if(hc_cmd == 21){
		if(sdport->cur_width == 8){
			/* 8bits 128bytes */
			cmd_info.byte_count = 128;
		}else if(sdport->cur_width ==4){
			/* 4bits 64bytes */
			cmd_info.byte_count = 64;
		}else{
			mmcmsg2( "not supported parameter");
			ret_err = -1;
			goto ERR_OUT;
		}

	}else{
		cmd_info.byte_count = 512;
	}

	ret_err = dw_em_Stream_Cmd( &cmd_info);


#ifndef AUTO_SEND_STOP
	if((hc_cmd == 18)
	|| (hc_cmd == 25))
	{
		dw_em_send_stop(sdport);
	}
#endif

ERR_OUT:
	return ret_err;
}


#endif

/************************************************************************
 * Note:
 * return -1 : send_status cmd fail, curr_state return 0xff.
 * return  0 : have not got STATE_TRAN status until max_try, have curr_state return.
 * return  1 : STATE_TRAN status got.
 *
 * Note:
 *    be ware!! pointer of rsp_buffer must point to 16 bytes buffer.
 ************************************************************************/
static
int dw_em_wait_status(e_device_type* sdport,
	unsigned int* rsp_buffer,
	unsigned int max_try)
{
	unsigned int curr_state;
	int ret_err;
	int rtn;
	unsigned int retry_cnt;

	curr_state =0xff;
	ret_err = 0;
	rtn = -1;
	retry_cnt = 0;
	while(1){
		retry_cnt++;
		ret_err = dw_em_SendCmd(MMC_SEND_STATUS, sdport->rca, MMC_RSP_R1, rsp_buffer);
		if( !ret_err ) {
			curr_state = R1_CURRENT_STATE(rsp_buffer[0]);
			if( curr_state == STATE_TRAN /*4*/ ) {
				if(rsp_buffer[0] & R1_READY_FOR_DATA){
					rtn = 1;
					break;
				}
			}
			else {
				if(retry_cnt >= max_try){
					//UPRINTF("%s(%d) curr_state : %s[%d]\n",
					//	__func__,__LINE__,
					//	(curr_state >= STA_EVENT_LEN)? "UNKNOW STATE":sta_evevt_tlb[curr_state],
					//	curr_state);
					rtn = 0;
					break;
				}
			}
		}
		else {
			//UPRINTF("%s(%d)send ststus fail!\n",__func__,__LINE__);
			rtn = -1;
			break;
		}
		udelay(5);

	};

	return rtn;
}

/*******************************************************
 *
 *******************************************************/
int mmc_show_ext_csd( e_device_type * card, unsigned char * pext_csd )
{
	unsigned int sec_cnt;
	unsigned int boot_size_mult;
#ifdef EMMC_SHOW_EXTCSD
	int i,j,k;
	UPRINTF("emmc:EXT CSD\n");
	k = 0;
	for( i = 0; i < 32; i++ ) {
		UPRINTF("    : %03d - ", i<<4);
		for( j = 0; j < 16; j++ ) {
			UPRINTF(" %02x", pext_csd[k++]);
		}
		UPRINTF("\n");
	}
	UPRINTF("    :k=%d\n",k);
#endif
	boot_size_mult = pext_csd[226];
	UPRINTF("emmc:BOOT PART %u Kbytes(0x%02x)\n", (boot_size_mult<<7), boot_size_mult);
	sec_cnt = pext_csd[215];
	sec_cnt = (sec_cnt<<8) | pext_csd[214];
	sec_cnt = (sec_cnt<<8) | pext_csd[213];
	sec_cnt = (sec_cnt<<8) | pext_csd[212];
	UPRINTF("emmc:SEC_COUNT %u(0x%08x)\n", sec_cnt, sec_cnt);

	return 0;
}

/*******************************************************
 *
 *******************************************************/
static
int rtkem_clr_sta(void)
{
	emmc_card.cur_clock	= 0;
	emmc_card.cur_width	= 0;
	emmc_card.cur_PLL	= 0;
	emmc_card.cur_div	= 0;
	emmc_card.cur_w_phase	= 0;
	emmc_card.cur_w_ph400	= 0;
	emmc_card.cur_r_phase	= 0;
	emmc_card.cur_ds_tune	= DEFAULT_DSTUNE;

	return 0;
}

/*******************************************************
 *
 *******************************************************/
void set_emmc_pin_mux(void)
{
	unsigned int reginfo;

	// 0x180008BC[0]=0x1
	reginfo = cr_readl(0x180008bc);
	cr_writel(reginfo|0x00000001,0x180008bc);

	//  PAD_EMMC_RST_N      0x180008B0[31:28] = 0x1
	//  PAD_EMMC_CLK        0x180008B0[23:20] = 0x1
	//  PAD_EMMC_CMD        0x180008B0[15:12] = 0x1
	reginfo = cr_readl(0x180008b0) & 0x0f0f0fff;
	cr_writel(reginfo|0x10101000,0x180008b0);

	//  PAD_EMMC_DS         0x180008B4[31:28] = 0x1
	//  PAD_EMMC_D5         0x180008B4[23:20] = 0x1
	//  PAD_EMMC_D3         0x180008B4[15:12] = 0x1
	//  PAD_EMMC_D4         0x180008B4[7:4]   = 0x1
	reginfo = cr_readl(0x180008b4) & 0x0f0f0f0f;
	cr_writel(reginfo|0x10101010,0x180008b4);

	//  PAD_EMMC_D0         0x180008B8[31:28] = 0x1
	//  PAD_EMMC_D1         0x180008B8[23:20] = 0x1
	//  PAD_EMMC_D2         0x180008B8[15:12] = 0x1
	//  PAD_EMMC_D7         0x180008B8[7:4]   = 0x1
	reginfo = cr_readl(0x180008b8) & 0x0f0f0f0f;
	cr_writel(reginfo|0x10101010,0x180008b8);

	//  PAD_EMMC_D6         0x180008BC[31:28] = 0x1
	reginfo = cr_readl(0x180008bc) & 0x0fffffff;
	cr_writel(reginfo|0x10000000,0x180008bc);
}

/*******************************************************
 *
 *******************************************************/
int dw_em_init_setup(void)
{
	/*
	 * rca value should sync with setting at rtkmmc_generic.c
	 * int mmc_data_sync( struct mmc * mmc );
	 */
	emmc_card.rca = 1<<16;
	emmc_card.sector_addressing = 0;

	rtkem_clr_sta();

	/* enable MMC pin_mux first */
	set_emmc_pin_mux();

	/* enable PLL of eMMC */
	dw_em_set_PLL_clk(CLK_50Mhz,0,1);

	dw_em_reset_host(0);
	dw_em_set_IO_driving(DRIVING_COMMON);
	dw_em_set_clk(CLK_200Khz);
	dw_em_init_phase(DEFAULT_W_PHASE,DEFAULT_R_PHASE,DEFAULT_C_PHASE); /* w_phase;r_phase */

	dw_em_set_bits(BUS_WIDTH_1);
	dw_em_set_scrip_base((unsigned int)dw_em_scrip);
	return 0;
}

int mmc_initial( int reset_only )
{
#ifndef CONFIG_LESS_CONSOLE
	UPRINTF("\nemmc:%s(%d) %s %s\n", __FUNCTION__, reset_only, __DATE__, __TIME__);
#endif

	rtkmmc_otp();

#ifdef ELAPSE_TIME_CACULATE
	mmc_C_xfer_us	= 0;
	mmc_R_xfer_us	= 0;
	mmc_W_xfer_us	= 0;
	mmc_R_xfer_blk	= 0;
	mmc_W_xfer_blk	= 0;
#endif

#ifdef CONFIG_NONCACHE_ALLOCATION
	dw_em_scrip = malloc_noncache_align( (16*EMMC_MAX_SCRIPT_LINE), 64);
#else
	dw_em_scrip = NULL;

	dw_em_scrip = (u8 *)malloc((16*EMMC_MAX_SCRIPT_LINE) + 0x20);
	if(!dw_em_scrip){
		UPRINTF("%s(%d)malloc memory fail\n",__func__,__LINE__);
		return -1;
	}
	dw_em_scrip = (u8 *)((((uint)(dw_em_scrip))&(~0x1F)) + 0x20);
#endif

	/* reset mmc ip */
	dw_em_init_setup();

	/* control reset pin, high -> low -> high */
	dw_em_reset_card();

#ifndef CONFIG_LESS_CONSOLE
	UPRINTF("reset finish...\n");
#endif
	return 0;

}


/******************************************************************************

 ******************************************************************************/
static
int rtk_emmc_chk_max_xfer_blk(struct mmc * mmc, bool show_msg)
{
	e_device_type *card = &emmc_card;
	unsigned int bus_width = 0;
	unsigned int mxa_xfer_blk;

	if(mmc != card->mmc){
		UPRINTF("%s(%d)unmatch device !!!\n",__func__,__LINE__);
		return 0;
	}

	bus_width = card->cur_width;
	if (bus_width == 8){
		if(mmc->timing == MMC_TIMING_MMC_HS400){
			mxa_xfer_blk = EMMC_MAX_XFER_BLK;
		}else if(mmc->timing == MMC_TIMING_MMC_HS200){
			mxa_xfer_blk = EMMC_MAX_XFER_BLK>>1;
		}else{
			mxa_xfer_blk = 600;
		}
	}else if (bus_width == 4){
		mxa_xfer_blk = 300;
	}else{
		mxa_xfer_blk = 60;
	}

	mmc->b_max = mxa_xfer_blk;

	if(show_msg)
		UPRINTF("Current max xfer block is %d !!!\n",mxa_xfer_blk);

	return 0;
}

/******************************************************************************

 ******************************************************************************/
int export_rtk_emmc_chk_max_xfer_blk(struct mmc * mmc, bool show_msg)
{
	return rtk_emmc_chk_max_xfer_blk(mmc,show_msg);
}


/******************************************************************************
u64 rtk_eMMC_write_byte( u64 byte_addr_str, unsigned int data_size, unsigned int * buffer );

unsigned int byte_addr		target want to access of eMMC data byte address
unsigned int data_size		access size int bytes
unsigned int * buffer		buffer address

return value : trabsfer length in byte, if zero transfer fail
*******************************************************************************/
u64 rtk_eMMC_write_byte(
		u64  byte_addr_str,
		unsigned int data_size,
		unsigned int * buffer )
{
	struct mmc * mmc = NULL;
	u64 ret;
	u32 cur_dev_num;

	mmc = emmc_card.mmc;
	cur_dev_num = mmc->block_dev.dev;

	ret = mmc->block_dev.byte_write(cur_dev_num,
				(u64)byte_addr_str,(unsigned long)data_size,(void*)buffer);

	if(ret != data_size){
		UPRINTF("emmc: transfer error!!! byte(xfer=0x%08x; targ=0x%08x\n",
				(unsigned int)ret,data_size);
	}

	return ret;

}

/******************************************************************************
u64 rtk_eMMC_read_byte ( u64 byte_addr_str, unsigned int data_size, unsigned int * buffer );

unsigned int byte_addr		target want to access of eMMC data byte address
unsigned int data_size		access size int bytes
unsigned int * buffer		buffer address

return value : trabsfer length in byte, if zero transfer fail
*******************************************************************************/
u64 rtk_eMMC_read_byte(
		u64 byte_addr_str,   /*unsigned long long byte_addr,*/
		unsigned int data_size,
		unsigned int * buffer )
{
	struct mmc * mmc = NULL;
	u64 ret;
	u32 cur_dev_num;

	mmc = emmc_card.mmc;
	cur_dev_num = mmc->block_dev.dev;

	ret = mmc->block_dev.byte_read(cur_dev_num,
				(u64)byte_addr_str,(unsigned long)data_size,(void*)buffer);

	if(ret != data_size){
		UPRINTF("emmc: transfer error!!! byte(xfer=0x%08x; targ=0x%08x\n",
				(unsigned int)ret,data_size);
	}

	return ret;

}

/******************************************************************************
	blk_addr: start block address in 512 bytes write to eMMC.
	data_size : how many bytes want to be writed to eMMC .
	buffer : pointer of data buffer

	return value: byte count of successful reading.
 ******************************************************************************/
unsigned int rtk_eMMC_write( unsigned int blk_addr, unsigned int data_size, unsigned int * buffer )
{
	/* Func rtk_eMMC_xxxx_byte() can handle request data length small then 512.
	 It's recommand to use.
	 */
	u64 xfer_bytes;
	u32 blk_cnt;

	xfer_bytes = rtk_eMMC_write_byte((((u64)blk_addr)<<9),data_size,buffer);
	blk_cnt = (u32)(xfer_bytes>>9);
	if( xfer_bytes & 0x1ffUL ) {
		blk_cnt+=1;
	}
	return blk_cnt;

}

/******************************************************************************
	blk_addr: start block address in 512 bytes read form eMMC.
	data_size : how many bytes want to be read from eMMC.
	buffer : pointer of data buffer

	return value: byte count of successful reading.

 ******************************************************************************/
unsigned int rtk_eMMC_read( unsigned int blk_addr, unsigned int data_size, unsigned int * buffer )
{
	/* Func rtk_eMMC_xxxx_byte() can handle request data length small then 512.
	 It's recommand to use.
	 */
	u64 xfer_bytes;
	u32 blk_cnt;

	xfer_bytes = rtk_eMMC_read_byte((((u64)blk_addr)<<9),data_size,buffer);
	blk_cnt = (u32)(xfer_bytes>>9);

	if( xfer_bytes & 0x1ffUL ) {
		blk_cnt+=1;
	}
	return blk_cnt;

}

int rtk_eMMC_xfer_counter(e_device_type * card,int byte_cnt)
{
	int err = 0;
	if(card->call_back_byte_cnt){
		card->call_back_byte_cnt(0, byte_cnt);
	}else{
		//UPRINTF("The callback func %s have not registed!\n",__func__);
		err = -1;
	}

	return err;

}

void eMMC_register_callback(int (*emmc_user_callback)(int, int))
{
    emmc_card.call_back_byte_cnt = emmc_user_callback;
}

void eMMC_show_speed(void)
{
	struct mmc * mmc = emmc_card.mmc;

	if(mmc){
		UPRINTF("eMMC(%s, mid=0x%x) %s (%dhz)[%dbit]\n",
			(emmc_card.mid==0x11)?"Toshiba":
			(emmc_card.mid==0x15)?"Samsung":
			(emmc_card.mid==0x13)?"Micron":"unknown",
			emmc_card.mid,
			(mmc->timing==MMC_TIMING_MMC_HS400)?"MMC_HS400":
			(mmc->timing==MMC_TIMING_MMC_HS200)?"MMC_HS200":
			(mmc->timing==MMC_TIMING_MMC_HS)?"MMC_HS50":"MMC_LEGACY",
			emmc_card.cur_clock,emmc_card.cur_width);
	}else{
		UPRINTF("emmc not ready for use!!!\n");
	}
}

int do_rtkmmc (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = CMD_RET_USAGE;
	unsigned int reg[5];

	if( strncmp( argv[1], "hs400", 5 ) == 0 ){
		eMMC_show_speed();
		reg[0] = cr_readl(EMMC_PLLPHASE_CTRL);
		reg[1] = cr_readl(EMMC_PLL_PHASE_INTERPOLATION);
		reg[2] = cr_readl(0xb801088c);
		reg[3] = cr_readl(0xb8010890);
		reg[4] = cr_readl(0xb8010850);
		printf( "PHASE_REG :\n"
				"    0x%08x=0x%08x 0x%08x=0x%08x\n",
				EMMC_PLLPHASE_CTRL, reg[0],
				EMMC_PLL_PHASE_INTERPOLATION,reg[1]);
		printf( "DS_TUNE_REG :\n"
    			"    0x%08x=0x%08x 0x%08x=0x%08x 0x%08x=0x%08x\n",
				0xb801088c,reg[2],
				0xb8010890,reg[3],
        		0xb8010850,reg[4]);
		printf( "W_Phase=%d; R_Phase=%d; C_Phase=%d\n"
				"DS mode=%d; Thr=%d\n"
				"Half_cycle_cal_result=%d\n"
				"Bypass_ds_ph_sel=%d\n",
				(reg[0]&0x1f),((reg[0]>>10)&0x1f),((reg[1]>>10)&0x1f),
				((reg[2]>>1)&0x03),((reg[2]>>3)&0x1f),
				(reg[3]&0x1f),((reg[4]>>15)&0x1f));

		return 0;
	}

	return  ret;
}

U_BOOT_CMD(
	rtkmmc, 7, 0, do_rtkmmc,
	"RTK MMC direct function",
	": all argument in HEX tpye.\n"
	"rtkmmc hs400: show hs400 enable or not\n"
	"\n"
);

