#include <common.h>
#include <drivers/gpio/rtk_gpio.h>
#include <drivers/pwm/pwm.h>
#include <asm/arch/io.h>
#include <sys_reg_reg.h>
#include <rbus/stb_reg.h>
#include <platform_lib/board/pcb_mgr.h>
#include <panel_config_parameter.h>

int PcbPwmInit = FALSE;
#define MAX_PWM_COUNT 10
static RTK_PWM_INFO PwmInfoList[MAX_PWM_COUNT];

extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);

int pwm_totalcnt = PWM_DUTY_MAX;

static char GPIO_Group_List[6][10] = {
    "ISO_GPIO",
    "MIS_GPIO",
    "ISO_GPI",
    "ISO_GPO",
    "MIS_GPI",
    "MIS_GPO",
};

int rtk_pwm_clk_sel(int type,int clk_sel){

    if(type  == PCB_PIN_TYPE_PWM){

        //{reg_pwm_clksel, reg_pwm_pll_clksel}
        //=2!|b00 pwm clock source is pllaud_ck196m
        //=2!|b01 pwm clock source is dpll clock div 4
        //=2!|b10 pwm clock source is xtal27
        //=2!|b11 pwm clock source is xtal27

        if(clk_sel == CLK_27MHZ){
            rtd_setbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm0_clksel_mask);
            rtd_setbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm_pll_clksel_mask);
        }
        else if(clk_sel == CLK_196MHZ){
            rtd_clearbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm0_clksel_mask);
            rtd_clearbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm_pll_clksel_mask);
        }
        else if(clk_sel == D_CLK_DIV4){
            rtd_clearbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm0_clksel_mask);
            rtd_setbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm_pll_clksel_mask);
        }
        else{
            rtd_setbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm0_clksel_mask);
            rtd_setbits(SYS_REG_SYS_CLKDIV_reg,SYS_REG_SYS_CLKDIV_pwm_pll_clksel_mask);
        }
    }
    else if (type  == PCB_PIN_TYPE_ISO_PWM){
        if(clk_sel == CLK_147MHZ)
            rtd_setbits(STB_ST_CLKMUX_reg,STB_ST_CLKMUX_iso_pwm_clk_sel_mask);
        else
            rtd_clearbits(STB_ST_CLKMUX_reg,STB_ST_CLKMUX_iso_pwm_clk_sel_mask);
    }
    return 0;
}

#if defined(CONFIG_SYS_PANEL_PARAMETER)
unsigned long Get_DISPLAY_CLOCK_TYPICAL(void)
{
    return default_panel_parameter.iCONFIG_DISPLAY_CLOCK_TYPICAL;
}
#endif

int rtk_pwm_totalcnt(void)
{    return  pwm_totalcnt;    }

char* rtk_get_clock_mode(int mode)
{
    switch(mode){
        case CLK_27MHZ:
            return "CLK_27MHZ";
        case CLK_196MHZ:
            return "CLK_196MHZ";
        case D_CLK_DIV4:
            return "D_CLK_DIV4";
        case CLK_147MHZ:
            return "CLK_147MHZ";
        default:
            return "CLK_27MHZ";
    }
}

int rtk_get_clock_src(int type,int index)
{
    unsigned int regval;

    if(type ==  PWM_UNKNOW){
        printf(" PWM_UNKNOW no DClk Source\n");
        return CLK_27MHZ;
    }else if(type ==  PWM_ISO ){

        regval = rtd_inl(STB_ST_CLKMUX_reg);

        if((regval & STB_ST_CLKMUX_iso_pwm_clk_sel_mask) != 0 )
            return CLK_147MHZ;
        else
            return CLK_27MHZ;

    }else{

        regval = rtd_inl(SYS_REG_SYS_CLKDIV_reg);

        if((regval & (SYS_REG_SYS_CLKDIV_pwm0_clksel_mask) ) != 0)
            return CLK_27MHZ;
        else{
            if((regval & (SYS_REG_SYS_CLKDIV_pwm_pll_clksel_mask) ) == 0)
                return CLK_196MHZ;
            else
                return D_CLK_DIV4;

        }
    }
}


unsigned long rtk_hw_clock_get(int mode)
{
#if defined(CONFIG_SYS_PANEL_PARAMETER)
    unsigned long dclk = 0;
#endif
    unsigned long clock_freq = 0;
    switch(mode){
        case CLK_27MHZ:
            clock_freq = CONFIG_CPU_XTAL_FREQ_27MHZ;
            break;

        case CLK_196MHZ:
            clock_freq = CONFIG_CPU_PLL_FREQ_196MHZ;
            break;
        case CLK_147MHZ:
            clock_freq = CONFIG_CPU_PLL_FREQ_147MHZ;
            break;

#if defined(CONFIG_SYS_PANEL_PARAMETER)
        case D_CLK_DIV4:
            dclk = Get_DISPLAY_CLOCK_TYPICAL();
            clock_freq = (unsigned long) dclk >> 2; //Divide 4
            break;
#endif
        default:
            clock_freq = CONFIG_CPU_XTAL_FREQ_27MHZ;
    }
    return clock_freq;
}

int rtk_get_pos_thr(int m_pos, unsigned long M, unsigned long N,int duty_max,int totalcnt)
{
    unsigned long phase_us= 0;
    unsigned int pos_start_clk =0;
    /*get v delay*/
    if(m_pos > 0) {
        pos_start_clk = (1 << M)*(N+1)*m_pos;

        if(pos_start_clk >=0x1fffff) {
            printf("[pwm] %s vsync overflow\n",__func__);
            printf("pos_start_clk = %d, phase_us = %ld\n",pos_start_clk, phase_us);
            pos_start_clk=0;
        }
    }else{
        pos_start_clk=0;
    }
    return pos_start_clk;
}

unsigned long rtk_cpu_ns_get(int mode)
{
#if defined(CONFIG_SYS_PANEL_PARAMETER)
    unsigned long dclk = 0;
#endif
    unsigned long cpu_ns;
    switch(mode){
        case CLK_27MHZ:
            cpu_ns = CONFIG_CPU_XTAL_NS_27MHZ;
            break;

        case CLK_196MHZ:
            cpu_ns = CONFIG_CPU_PLL_NS_196MHZ;
            break;
        case CLK_147MHZ:
            cpu_ns = CONFIG_CPU_PLL_NS_147MHZ;
            break;
#if defined(CONFIG_SYS_PANEL_PARAMETER)
        case D_CLK_DIV4:
            dclk = Get_DISPLAY_CLOCK_TYPICAL();
            dclk = (dclk >> 2);   
            cpu_ns = (unsigned long) (10000000000/dclk); //Divide 4
            break;
#endif
        default:
            cpu_ns = CONFIG_CPU_XTAL_NS_27MHZ;
    }

    return cpu_ns;
}

#define M_PWM_W_DB0_MODE_W(num,reg,val) reg.pwm##num##_w_db0_mode = val
#define M_PWM_DB0_RD_SEL_W(num,reg,val) reg.pwm##num##_db0_rd_sel = val
#define M_PWM_W_DB0_EN_W(num,reg,val) reg.pwm##num##_w_db0_en = val
void rtk_pwm_db_set(RTK_PWM_TYPE type,int index,int vsync)
{
    pwm_db0_ctrl_RBUS dbReg;

    if(type == PWM_MISC){
        dbReg.regValue = rtd_inl(MIS_PWM_DB0_CTRL_VADDR);
        switch(index){
            case PWM0_DB0:
                M_PWM_W_DB0_MODE_W(0,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(0,dbReg,0);
                M_PWM_W_DB0_EN_W(0,dbReg,1);
                break;
            case PWM1_DB0:
                M_PWM_W_DB0_MODE_W(1,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(1,dbReg,0);
                M_PWM_W_DB0_EN_W(1,dbReg,1);
                break;
            case PWM2_DB0:
                M_PWM_W_DB0_MODE_W(2,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(2,dbReg,0);
                M_PWM_W_DB0_EN_W(2,dbReg,1);
                break;
            case PWM3_DB0:
                M_PWM_W_DB0_MODE_W(3,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(3,dbReg,0);
                M_PWM_W_DB0_EN_W(3,dbReg,1);
                break;
            case PWM4_DB0:
                M_PWM_W_DB0_MODE_W(4,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(4,dbReg,0);
                M_PWM_W_DB0_EN_W(4,dbReg,1);
                break;
            case PWM5_DB0:
                M_PWM_W_DB0_MODE_W(5,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(5,dbReg,0);
                M_PWM_W_DB0_EN_W(5,dbReg,1);
                break;
            case PWM6_DB0:
                M_PWM_W_DB0_MODE_W(6,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(6,dbReg,0);
                M_PWM_W_DB0_EN_W(6,dbReg,1);
                break;
            case PWM7_DB0:
                M_PWM_W_DB0_MODE_W(7,dbReg,vsync);
                M_PWM_DB0_RD_SEL_W(7,dbReg,0);
                M_PWM_W_DB0_EN_W(7,dbReg,1);
                break;
            default:
                break;
        }
        rtd_outl(MIS_PWM_DB0_CTRL_VADDR, dbReg.regValue);
    }
    else if(type == PWM_ISO){
        dbReg.regValue = rtd_inl(ISO_PWM_DB_CTRL_VADDR);
        dbReg.i_pwm_w_db_mode = vsync;
        dbReg.i_pwm_w_db_en   = 1;
        dbReg.i_db_rd_sel     = 0;
        rtd_outl(ISO_PWM_DB_CTRL_VADDR, dbReg.regValue);
    }
}

int rtk_pwm_db_wr(RTK_PWM_TYPE type, int index, int db_wr)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    if (db_wr == 0 || db_wr == 1){
        pwm_ctrl_reg.regValue = rtd_inl(registerAddress);
        if(PWM_ISO == type)
            pwm_ctrl_reg.i_pwm0_w_db_wr = db_wr;
        else
            pwm_ctrl_reg.m_pwm0_w_db0_wr = db_wr;
        rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
    }
    else
        printf("set pwm db_wr=%d invalid!!\n",db_wr);

    return 0;
}

char* rtk_pwm_gpio_group_get(int group)
{
    int index = 0;
    switch(group){
        case PCB_PIN_TYPE_GPIO:
            index = 1;
            break;
        case PCB_PIN_TYPE_ISO_GPIO:
            index = 0;
            break;
        case PCB_PIN_TYPE_GPI:
            index = 4;
            break;
        case PCB_PIN_TYPE_GPO:
            index = 5;
            break;
        case PCB_PIN_TYPE_ISO_GPI:
            index = 2;
            break;
        case PCB_PIN_TYPE_ISO_GPO:
            index = 3;
            break;
        case PCB_PIN_TYPE_PWM:
            return "MISC_PWM";
        case PCB_PIN_TYPE_ISO_PWM:
            return "ISO_PWM";
    }
    return GPIO_Group_List[index];
}

char* rtk_pwm_name_get(int index)
{
    if( index < 0 || index >= MAX_PWM_COUNT)
    {
        printf("index out of range!!\n");
        return NULL;
    }
    return PwmInfoList[index].name;
}

int rtk_pwm_list(void)
{
    int i = 0;
    for(i=0;i<MAX_PWM_COUNT;++i){
        if(strlen(PwmInfoList[i].name) == 0)
            break;

        printf("[%d]:%s\n",i,PwmInfoList[i].name);
    }

    return 0;
}

RTK_PWM_TYPE rtk_pwm_type_get(int i)
{
    RTK_PWM_TYPE type = 0;
    PCB_PIN_TYPE_T pin_type;
    pin_type = GET_PIN_TYPE(PwmInfoList[i].pinInfo.val);
    if(pin_type == PCB_PIN_TYPE_PWM)
        type = PWM_MISC;
    else if(pin_type == PCB_PIN_TYPE_ISO_PWM)
        type = PWM_ISO;
    else{
        type = PWM_UNKNOW;
        printf("Unknow PWM Type!!\n");
    }

    return type;
}

int rtk_pwm_index_get(RTK_PWM_TYPE type,int i)
{
    int index = 0;
    index = GET_PIN_INDEX(PwmInfoList[i].pinInfo.val);

    if(type == PWM_ISO){
        if(index < 0 || index > ISO_PWM_MAX)
            return (-1);
    }else if(type == PWM_MISC){
        if(index < 0 || index > MISC_PWM_MAX)
            return (-1);
    }
    else{
        return (-1);
    }

    return index;
}

int rtk_pwm_info_dump(void)
{
    int i = 0;
    char ch;

    do{
        printf("---------------------------------------------------------------------- \n");
        if(((PCB_PIN_TYPE_T)PwmInfoList[i].pinInfo.pwm_type) == PCB_PIN_TYPE_ISO_PWM)
            ch = 'I';
        else
            ch = 'M';
        printf("=========== PWM Init Info ===================\n");
        printf("pwm-%c-%d pin info:0x%llx \n",ch,PwmInfoList[i].pinInfo.pwm_index,PwmInfoList[i].pinInfo.val);
        printf("pwm  name:%s \n",PwmInfoList[i].name);
        printf("init info : max_duty-%d  ",PwmInfoList[i].pinInfo.duty_max);
        printf("duty-%d ",PwmInfoList[i].pinInfo.init_duty);
        printf("freq-%dHz \n",(PwmInfoList[i].pinInfo.freq_h << 16) | (PwmInfoList[i].pinInfo.freq_l << 8)|(PwmInfoList[i].pinInfo.freq) );
        printf("=========== PWM PCB Info  ===================\n");
        printf("pinInfo.val:%llx\n",PwmInfoList[i].pinInfo.val);
        printf("pinInfo.pwm_type:%s\n",rtk_pwm_gpio_group_get(PwmInfoList[i].pinInfo.pwm_type));
        printf("pinInfo.pwm_index:%d\n",PwmInfoList[i].pinInfo.pwm_index);
        printf("pinInfo.invert:%d\n",PwmInfoList[i].pinInfo.invert);
        printf("pinInfo.freq:%d\n",(PwmInfoList[i].pinInfo.freq_h << 16) | (PwmInfoList[i].pinInfo.freq_l << 8)|(PwmInfoList[i].pinInfo.freq));
        printf("pinInfo.duty_max:%d\n",PwmInfoList[i].pinInfo.duty_max);
        printf("pinInfo.init_duty:%d\n",PwmInfoList[i].pinInfo.init_duty);
        if(0 == PwmInfoList[i].pin_mux_info.val)
            printf("pwm pinmux info is not defined");
        else{
            printf("=========== PWM GPIO Pinmux Info ===========\n");
            printf("pwm-%c-%d pinmux info:0x%llx \n",ch,PwmInfoList[i].pinInfo.pwm_index,PwmInfoList[i].pin_mux_info.val);
            printf("pimux reg :0x%8x[%d:%d] \n",PwmInfoList[i].pin_mux_info.addr,\
                                                PwmInfoList[i].pin_mux_info.end_bit,\
                                                PwmInfoList[i].pin_mux_info.start_bit);
            printf("pwm set-value :0x%1x \n",PwmInfoList[i].pin_mux_info.pwm_value);
            printf("gpio set-value:0x%1x \n",PwmInfoList[i].pin_mux_info.gpio_value);
            printf("gpio attribute:%s-%d \n",rtk_pwm_gpio_group_get(PwmInfoList[i].pin_mux_info.gpio_group),\
                                           PwmInfoList[i].pin_mux_info.gpio_num);
        }
        i++;
    }while(NULL != PwmInfoList[i].name);
#if 0 //[CID-313990] Logically dead code: i must be at least 1.
    if(0 == i){
        printf("no pwm info get\n");
        return -1;
    }
#endif
    printf("---------------------------------------------------------------------- \n");
    return 0;
}

void rtk_pwm_pcb_info_init(void)
{
    PCB_PIN_TYPE_T pin_type;
    int i = 0;
    char pinmux_info_name[64];
    unsigned long long ullPcbPinMuxValue = 0;
    PCB_ENUM_T *ppcb_enum = pcb_enum;
    int iPcbPwmCounter = 0;

    if(PcbPwmInit == TRUE){
        printf("PCB PWM Info Init Already!!\n");
        return;
    }
    /*parse the iso pwm pin*/
    do {
        if((ppcb_enum == NULL) || (ppcb_enum[i].name == NULL) || (ppcb_enum[i].name[0] == 0))
            break;
        if (memcmp(ppcb_enum[i].name, "PIN", 3) == 0) {
            pin_type = GET_PIN_TYPE(ppcb_enum[i].info);
            if((pin_type ==  PCB_PIN_TYPE_ISO_PWM) || (pin_type ==  PCB_PIN_TYPE_PWM)){
                PwmInfoList[iPcbPwmCounter].pinInfo.val = ppcb_enum[i].info;
                PwmInfoList[iPcbPwmCounter].pinInfo.invert = GET_PWM_INVERT(ppcb_enum[i].info);
                PwmInfoList[iPcbPwmCounter].name = ppcb_enum[i].name;
                PwmInfoList[iPcbPwmCounter].mode = PWM_MODE;
                (void)snprintf(pinmux_info_name, sizeof(pinmux_info_name), "%s%s", ppcb_enum[i].name,"_GPIO_INFO");
                if(0 == pcb_mgr_get_enum_info_byname(pinmux_info_name, &ullPcbPinMuxValue))
                {
                    PwmInfoList[iPcbPwmCounter].pin_mux_info.val = ullPcbPinMuxValue;
                    PwmInfoList[iPcbPwmCounter].gid = rtk_gpio_get_gid_by_pintype(GET_PWM_PINMUX_GPIO_PINTYPE(ullPcbPinMuxValue),
                                                                                    GET_PWM_PINMUX_GPIO_NUM(ullPcbPinMuxValue));
                }

                iPcbPwmCounter++;
            }
        }
        i ++;
    } while(1);
    PcbPwmInit = TRUE;
}

RTK_PWM_INFO* rtk_pwm_info_find(char *name)
{
    int index = 0;
    for(index = 0; index < MAX_PWM_COUNT; ++index){
        if(strcmp(name, PwmInfoList[index].name) == 0){
            return &PwmInfoList[index];
        }
    }
    
    return NULL;
}

int rtk_pwm_gpio_mode_check(char *name)
{
    RTK_PWM_INFO *pwm_info = NULL;

    pwm_info = rtk_pwm_info_find(name);

    if(pwm_info->mode == GPIO_MODE){
        return (-1);
    }
    
    return 0;
}

int rtk_get_pwm_duty_base_change(int duty,int duty_max,int totalcnt)
{
    return (duty*(totalcnt+1))/(duty_max+1);
}

int pwm_init(RTK_PWM_TYPE type,int index,int pwm_freq_hz, int duty_max, int duty, int clk_src, int invert)
{
    int totalcnt = 0;
    unsigned short N_best = N_MAX, M_best = M_MAX;
    pwm_ctrl_RBUS         pwm_ctrl_reg;
    pwm_timing_ctrl_RBUS  pwm_timing_ctrl_reg;
    pwm_duty_set_RBUS     pwm_duty_set_reg;
    int regAddr = 0;
    int __attribute__ ((unused)) regval;

    if (type == PWM_MISC)
    {
        if (index >= 8) return -1;
        printf("pwm_freq_hz = %d\n", pwm_freq_hz);
    } else {
        if (index >= 2) return -1;
        printf("iso_pwm_freq_hz = %d\n", pwm_freq_hz);
    }

    if (type == PWM_MISC)
        rtk_pwm_clk_sel(PCB_PIN_TYPE_PWM,clk_src);
    else
        rtk_pwm_clk_sel(PCB_PIN_TYPE_ISO_PWM,clk_src);

    rtk_hw_get_divider(pwm_freq_hz*100, duty_max, &totalcnt, &M_best, &N_best, clk_src);

    printf("M = 0x%x\n", M_best);
    printf("N = 0x%x\n", N_best);
    if (type == PWM_MISC)
    {
        printf("PCB_PIN_TYPE_PWM\n");
        pwm_timing_ctrl_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->MISC_TIMING_CTRL);
        pwm_duty_set_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET);
        pwm_ctrl_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->MISC_CTRL);

        pwm_timing_ctrl_reg.m_pwm0_m = M_best;
        pwm_timing_ctrl_reg.m_pwm0_n = N_best;

        pwm_duty_set_reg.m_pwm0_totalcnt = totalcnt;
        pwm_duty_set_reg.m_pwm0_dut = rtk_get_pwm_duty_base_change(duty,duty_max,totalcnt);

        pwm_ctrl_reg.regValue = 0; //rtd_inl(PWM0_CTRL_reg + index * 12);
        pwm_ctrl_reg.m_pwm0_en = 1;
        pwm_ctrl_reg.m_pwm0_dvs_mux = 1;
        pwm_ctrl_reg.m_pwm0_w_db0_wr = 1;
        pwm_ctrl_reg.m_pwm0l = (invert != 0);

        rtd_outl(pwm_mis_phy[index].p_reg_map->MISC_CTRL,pwm_ctrl_reg.regValue);
        rtd_outl(pwm_mis_phy[index].p_reg_map->MISC_TIMING_CTRL,pwm_timing_ctrl_reg.regValue);
        rtd_outl(pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET,pwm_duty_set_reg.regValue);

        regAddr = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
        regAddr = pwm_mis_phy[index].p_reg_map->MISC_TIMING_CTRL;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
        regAddr = pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);

    } else {
        printf("PCB_PIN_TYPE_ISO_PWM\n");
        pwm_timing_ctrl_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->ISO_TIMING_CTRL);
        pwm_duty_set_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET);
        pwm_ctrl_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->ISO_CTRL);

        pwm_timing_ctrl_reg.i_pwm0_clk_sel = 0;
        pwm_timing_ctrl_reg.i_pwm0_m = M_best;
        pwm_timing_ctrl_reg.i_pwm0_n = N_best;

        pwm_duty_set_reg.i_pwm0_totalcnt = totalcnt;
        pwm_duty_set_reg.i_pwm0_dut = rtk_get_pwm_duty_base_change(duty,duty_max,totalcnt);

        pwm_ctrl_reg.regValue = 0; //rtd_inl(PWM0_CTRL_reg + index * 12);
        pwm_ctrl_reg.i_pwm0_en = 1;
        pwm_ctrl_reg.i_pwm0l = (invert != 0);

        rtd_outl(pwm_iso_phy[index].p_reg_map->ISO_CTRL,pwm_ctrl_reg.regValue);
        rtd_outl(pwm_iso_phy[index].p_reg_map->ISO_TIMING_CTRL,pwm_timing_ctrl_reg.regValue);
        rtd_outl(pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET,pwm_duty_set_reg.regValue);

        regAddr = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
        regAddr = pwm_iso_phy[index].p_reg_map->ISO_TIMING_CTRL;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
        regAddr = pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET;
        regval = rtd_inl(regAddr);
        pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
    }
    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return 0;
}

void rtk_hw_get_divider(int pwm_freq_hz, int duty_max, int *totalcnt,
            unsigned short *M_best, unsigned short *N_best, int clk_src)
{
    int i , j= 0;
    unsigned long ftmp = 0, error = 0, error_best = 0;
    unsigned long N = 0, ftmp_1 = 0, N1 = 0, ftmp_2 = 0;
    unsigned long pwmFreqHz = 0;
    int stop = 0;
    unsigned long hw_clock = rtk_hw_clock_get(clk_src);
    if (pwm_freq_hz == 300) {
        *M_best = 0x3;
        *N_best = 0xfff;
        *totalcnt = duty_max;
    } else if(pwm_freq_hz > 300 && pwm_freq_hz <= 100000){// For freq = 3~1000Hz,total cnt fixed 255
        ftmp = (duty_max + 1) * pwm_freq_hz + 1;
        error_best = ftmp;
        for (i = 0; i <= M_MAX; i++) {
            N1 = (hw_clock / (1 << i))*10;
            N = (N1 /ftmp)*10;
            if (N <= 0)
                N = 1;

            if (N > 0x1000)
                continue;

            ftmp_2 = (hw_clock / (1 << i))*10;
            ftmp_1 = (ftmp_2 / N)*10;
            error = ABS( ftmp_1, ftmp );
            if (error < error_best) {
                error_best = error;
                *N_best = (unsigned short)N;
                *M_best = i;
            }
        }
        *totalcnt = duty_max;
    }
    else { // For freq > 1000Hz, totalcnt is variable
        pwmFreqHz = pwm_freq_hz/100;
        error_best = pwmFreqHz  + 1;
        for (j=duty_max; j<4097; j++) // Start duty_max=255 to find solution
        {
            for (i=0; i<=3; i++)
            {
                N = (hw_clock / (( 1 << i)*j*pwmFreqHz));
                if (N == 0)
                    N=1;

                if (N > 0x1000)
                    continue;

                error = ABS(hw_clock / (( 1 << i)*j*N),pwmFreqHz);

                if(error == 0)
                {
                    error_best = error;
                    *N_best = N-1;
                    *M_best = i;
                    *totalcnt = j-1;
                    stop = 1;
                    break;
                }

                if (error < error_best)
                {
                    error_best = error;
                    *N_best = N-1;
                    *M_best = i;
                    *totalcnt = j-1;
                }
            }
            if (stop == 1)
            {
                break;
            }
        }
    }
    printf("%s CLK_SRC:%s M_best:%d N_best:%d freq:%d totalcnt:%d duty_max:%d\n",__FUNCTION__,rtk_get_clock_mode(clk_src),*M_best,*N_best,pwm_freq_hz,*totalcnt,duty_max);
}


int get_pwm_frequency(RTK_PWM_TYPE type, int index)
{
    int freq = MIN_PWM_FREQ;
    int totalcnt = 0;
    unsigned short N = 0, M = 0;

    unsigned int DutySetAddress = 0;
    unsigned int TimingCtrlAddress = 0;
    unsigned long hw_clock = rtk_hw_clock_get(rtk_get_clock_src(type, index));

    pwm_duty_set_RBUS     pwm_duty_set_reg;
    pwm_timing_ctrl_RBUS  pwm_timing_ctrl_reg;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        DutySetAddress = pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET;
        TimingCtrlAddress = pwm_iso_phy[index].p_reg_map->ISO_TIMING_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        DutySetAddress = pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET;
        TimingCtrlAddress = pwm_mis_phy[index].p_reg_map->MISC_TIMING_CTRL;
    } else {
        printf("%s parameter fail register address 0x%X !\n",  __func__ , TimingCtrlAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    pwm_timing_ctrl_reg.regValue =rtd_inl(TimingCtrlAddress);
    if(PWM_ISO == type){
        M = pwm_timing_ctrl_reg.i_pwm0_m;
        N = pwm_timing_ctrl_reg.i_pwm0_n;
    }else{
        M = pwm_timing_ctrl_reg.m_pwm0_m;
        N = pwm_timing_ctrl_reg.m_pwm0_n;
    }

    pwm_duty_set_reg.regValue = rtd_inl(DutySetAddress);
    if(PWM_ISO == type)
        totalcnt = pwm_duty_set_reg.i_pwm0_totalcnt;
    else
        totalcnt = pwm_duty_set_reg.m_pwm0_totalcnt;

    freq = (hw_clock /(1 << M)) / (N+1)/totalcnt;

    printf("get pwm freq=%d success!!\n", freq);

    return freq;
}

int set_pwm_frequency(RTK_PWM_TYPE type, int index,int freq)
{
    int current_totalcnt = 0;
    int current_duty = 0;
    int duty = 0;
    int totalcnt = 0;
    int duty_max = 255;
    unsigned short N = 0, M = 0;

    unsigned int DutySetAddress = 0;
    unsigned int TimingCtrlAddress = 0;
    pwm_duty_set_RBUS     pwm_duty_set_reg;
    pwm_timing_ctrl_RBUS  pwm_timing_ctrl_reg;

#if 0
    if (freq <= MIN_PWM_FREQ)
        freq = MIN_PWM_FREQ;/* minimal is 3.2, AP won't set this?! */
    else if (freq >= MAX_PWM_FREQ)
        freq = MAX_PWM_FREQ;
#endif
    int clk_src = rtk_get_clock_src(type,index);

    rtk_hw_get_divider(freq*100, duty_max, &totalcnt, &M, &N, clk_src);

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        DutySetAddress = pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET;
        TimingCtrlAddress = pwm_iso_phy[index].p_reg_map->ISO_TIMING_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        DutySetAddress = pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET;
        TimingCtrlAddress = pwm_mis_phy[index].p_reg_map->MISC_TIMING_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , TimingCtrlAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }


    pwm_timing_ctrl_reg.regValue =rtd_inl(TimingCtrlAddress);
    if(PWM_ISO == type){
        pwm_timing_ctrl_reg.i_pwm0_m = M;
        pwm_timing_ctrl_reg.i_pwm0_n = N;
    }
    else{
        pwm_timing_ctrl_reg.m_pwm0_m = M;
        pwm_timing_ctrl_reg.m_pwm0_n = N;
    }

    rtd_outl(TimingCtrlAddress,pwm_timing_ctrl_reg.regValue);

    pwm_duty_set_reg.regValue = rtd_inl(DutySetAddress);
    if(PWM_ISO == type){
        current_duty = pwm_duty_set_reg.i_pwm0_dut;
        current_totalcnt = pwm_duty_set_reg.i_pwm0_totalcnt;
        duty = rtk_get_pwm_duty_base_change(current_duty,current_totalcnt,totalcnt);
        pwm_duty_set_reg.i_pwm0_dut = duty;
        pwm_duty_set_reg.i_pwm0_totalcnt = totalcnt;
    }
    else{
        current_duty = pwm_duty_set_reg.m_pwm0_dut;
        current_totalcnt = pwm_duty_set_reg.m_pwm0_totalcnt;
        duty = rtk_get_pwm_duty_base_change(current_duty,current_totalcnt,totalcnt);
        pwm_duty_set_reg.m_pwm0_dut = duty;
        pwm_duty_set_reg.m_pwm0_totalcnt = totalcnt;
    }
    rtd_outl(DutySetAddress,pwm_duty_set_reg.regValue);

    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return 0;
}


int get_pwm_duty(RTK_PWM_TYPE type, int index)
{
    pwm_duty_set_RBUS pwm_duty_set_reg;
    int duty = 0;
    int duty_max = 255;
    int totalcnt = 0;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    pwm_duty_set_reg.regValue = rtd_inl( registerAddress );
    if(PWM_ISO == type){
        duty = pwm_duty_set_reg.i_pwm0_dut;
        totalcnt = pwm_duty_set_reg.i_pwm0_totalcnt;
    }
    else{
        duty = pwm_duty_set_reg.m_pwm0_dut;
        totalcnt = pwm_duty_set_reg.m_pwm0_totalcnt;
    }

    if( duty > 255 || duty < 0 )
        printf("get pwm duty=%d duty_max=%d invalid!!\n", duty , duty_max );
    else
        printf("get pwm duty=%d duty_max=%d totalcnt=%d success!!\n", duty , duty_max, totalcnt);

    return duty;
}

int set_pwm_duty(RTK_PWM_TYPE type, int index, int duty)
{
    pwm_duty_set_RBUS pwm_duty_set_reg;
    int duty_max = 255;
    int totalcnt = 0;

    unsigned int registerAddress = 0;
    
    if( duty > 255 || duty < 0 ){
        printf("set pwm duty=%d  failed!!\n\n", duty );
        return -1;
    }else
        printf("set pwm duty=%d  success!!\n\n", duty );


    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_DUTY_SET;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_DUTY_SET;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }


    pwm_duty_set_reg.regValue = rtd_inl( registerAddress );
    if(PWM_ISO == type)
        totalcnt = pwm_duty_set_reg.i_pwm0_totalcnt;
    else
        totalcnt = pwm_duty_set_reg.m_pwm0_totalcnt;

    if (duty >= totalcnt) duty = totalcnt;
    if (duty <= 0) duty = 0;

    if(PWM_ISO == type)
        pwm_duty_set_reg.i_pwm0_dut = rtk_get_pwm_duty_base_change(duty,duty_max,totalcnt);
    else
        pwm_duty_set_reg.m_pwm0_dut = rtk_get_pwm_duty_base_change(duty,duty_max,totalcnt);

    rtd_outl( registerAddress , pwm_duty_set_reg.regValue);

    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return 0;
}

int get_pwm_enable(RTK_PWM_TYPE type, int index)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;
    int enable;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    pwm_ctrl_reg.regValue = rtd_inl( registerAddress );
    if(PWM_ISO == type)
        enable = pwm_ctrl_reg.i_pwm0_en;
    else
        enable = pwm_ctrl_reg.m_pwm0_en;

    printf("get pwm enable=%d success!!\n",enable);
    return enable;
}

int set_pwm_enable(RTK_PWM_TYPE type, int index, int enable)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }


    if (enable == 0 || enable == 1){
        pwm_ctrl_reg.regValue = rtd_inl( registerAddress );
        if(PWM_ISO == type)
            pwm_ctrl_reg.i_pwm0_en = enable;
        else
            pwm_ctrl_reg.m_pwm0_en = enable;

        rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
    }
    else
        printf("set pwm enable=%d invalid!!\n",enable);

    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return 0;
}

int get_pwm_polarity(RTK_PWM_TYPE type, int index)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;
    int enable;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    pwm_ctrl_reg.regValue = rtd_inl( registerAddress );
    if(PWM_ISO == type)
        enable = pwm_ctrl_reg.i_pwm0l;
    else
        enable = pwm_ctrl_reg.m_pwm0l;

    printf("get pwm polarity=%d success!!\n",enable);
    return enable;
}

int set_pwm_polarity(RTK_PWM_TYPE type, int index, int polarity)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;

    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }


    if (polarity == 0 || polarity == 1){
        pwm_ctrl_reg.regValue = rtd_inl( registerAddress );
        if(PWM_ISO == type)
            pwm_ctrl_reg.i_pwm0l = polarity;
        else
            pwm_ctrl_reg.m_pwm0l = polarity;
        rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
    }
    else
        printf("set pwm polarity=%d invalid!!\n",polarity);

    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return polarity;
}

int get_pwm_sync(RTK_PWM_TYPE type, int index)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;
    int rtk_vsync;
    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    pwm_ctrl_reg.regValue = rtd_inl(registerAddress);
    if(PWM_ISO == type)
        rtk_vsync = pwm_ctrl_reg.i_pwm0_vs_rst_en;
    else
        rtk_vsync = pwm_ctrl_reg.m_pwm0_vs_rst_en;

    printf("get pwm sync=%d success!!\n",rtk_vsync);

    return rtk_vsync;
}

int set_pwm_sync(RTK_PWM_TYPE type, int index,int rtk_vsync)
{
    pwm_ctrl_RBUS pwm_ctrl_reg;
    unsigned int registerAddress = 0;

    if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
        registerAddress = pwm_iso_phy[index].p_reg_map->ISO_CTRL;
    } else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
        registerAddress = pwm_mis_phy[index].p_reg_map->MISC_CTRL;
    }else{
        printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
        printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
        printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
        return -1;
    }

    if (rtk_vsync == 0 || rtk_vsync == 1){
        pwm_ctrl_reg.regValue = rtd_inl(registerAddress);
        if(PWM_ISO == type)
            pwm_ctrl_reg.i_pwm0_vs_rst_en = rtk_vsync;
        else
            pwm_ctrl_reg.m_pwm0_vs_rst_en = rtk_vsync;

        rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
    }
    else
        printf("set pwm sync=%d invalid!!\n",rtk_vsync);

    //DB Enable & write
    rtk_pwm_db_set(type,index,0);
    rtk_pwm_db_wr(type,index,1);

    return 0;
}

int set_pwm_gpio_mode(char *name,int mode, int value)
{
    int gpio_num = 0 ;
    int group = 0;
    int gpioValue = 0;
    int invert = 0;

    RTK_PWM_INFO *pwm_info = NULL;

    rtk_pwm_pcb_info_init();

    pwm_info = rtk_pwm_info_find(name);

    if(pwm_info == NULL){
        printf("%s find pwm_info faile!\n", __func__ );
        return PWM_FAILE;
    }

    if(mode == PWM_MODE){
        if(pwm_info->mode != mode){
            pwm_info->mode = PWM_MODE;
            rtd_part_outl(pwm_info->pin_mux_info.addr,
                pwm_info->pin_mux_info.end_bit,
                pwm_info->pin_mux_info.start_bit,
                pwm_info->pin_mux_info.pwm_value);
        }
    }else if(mode == GPIO_MODE){
        if(pwm_info->mode != mode){
            pwm_info->mode = GPIO_MODE;
            rtd_part_outl(pwm_info->pin_mux_info.addr,
                pwm_info->pin_mux_info.end_bit,
                pwm_info->pin_mux_info.start_bit,
                pwm_info->pin_mux_info.gpio_value);
        }
        group = gpio_group(pwm_info->gid);
        gpio_num = gpio_idx(pwm_info->gid);
        invert = pwm_info->pinInfo.invert;
        gpioValue = (invert)?(!value):(value);
        rt_gpio_set(gpio_num,gpioValue,group);
        printf("group:%d gpio_num:%d invert:%d value:%d\n",group, gpio_num,invert,value);

    }else{
        printf("set mode invalid!!\n");
        return PWM_FAILE;
    }

    
    return PWM_OK;
}

int get_pwm_gpio_mode(char *name,int *mode, int *value)
{
    int gpio_num = 0 ;
    int group = 0;
    int invert = 0;
    RTK_PWM_INFO *pwm_info = NULL;

    rtk_pwm_pcb_info_init();

    pwm_info = rtk_pwm_info_find(name);

    if(pwm_info == NULL){
        printf("%s find pwm_info faile!\n", __func__ );
        return PWM_FAILE;
    }
    
    if(pwm_info->mode == PWM_MODE){
        *mode = pwm_info->mode;
        *value = 0;
        printf("mode:PWM\n");
    }else if(pwm_info->mode == GPIO_MODE){
        *mode = pwm_info->mode;

        group = gpio_group(pwm_info->gid);
        gpio_num = gpio_idx(pwm_info->gid);
        invert = pwm_info->pinInfo.invert;
        *value = rt_gpio_get(gpio_num,group);
        printf("mode:GPIO\n");
        printf("group:%d gpio_num:%d invert:%d value:%d\n",group, gpio_num, invert, *value);

    }else{
        printf("get mode invalid!!\n");
        return PWM_FAILE;
    }
    
    return PWM_OK;
}

