#ifndef __RTK_CW_UTIL_H__
#define __RTK_CW_UTIL_H__
#include <asm/types.h>
#include <common.h>
#include <linux/list.h>

#define BITS_PER_BYTE   8
#define BITS_TO_LONGS(nr) \
    DIV_ROUND_UP(nr, BITS_PER_BYTE * sizeof(long))
#ifndef BITS_PER_LONG
#define BITS_PER_LONG (BITS_PER_BYTE * sizeof(long))
#endif
typedef unsigned char   UINT8;
typedef signed char     INT8;
typedef unsigned short  UINT16;
typedef signed short    INT16;
typedef unsigned int    UINT32;
typedef signed int      INT32;
typedef unsigned long long  UINT64;
typedef signed long long    INT64;

#define kmalloc(size, flags)    malloc(size)
#define kfree(ptr)      free(ptr)
#define set_bit(index, ptr) (ptr[index /BITS_PER_LONG] = ptr[index /BITS_PER_LONG] | (1<<( index % BITS_PER_LONG)))
#define clear_bit(index, ptr) (ptr[index /BITS_PER_LONG] = ptr[index /BITS_PER_LONG] & (~(1<<( index % BITS_PER_LONG))))

/*************Use rw semaphore to protect*************/
#include <ubi_uboot.h>
typedef struct rw_semaphore MUTEX_LOCKTYPE;
#define INIT_LOCK(lock)         init_rwsem(&(lock))
#define WRITE_LOCK(lock, flags)     up_write(&(lock))
#define WRITE_UNLOCK(lock, flags)   down_write(&(lock))
#define READ_LOCK(lock, flags)      up_read(&(lock))
#define READ_UNLOCK(lock, flags)        down_read(&(lock))

#define GFP_ALLOCK_FLAG     GFP_KERNEL



/***********************Structs define and interfaces define***********************/
#define MODULE_NAME_LEN 26
#define CW_AREA_LEN 128
#define BITMAP_LEN  BITS_TO_LONGS(CW_AREA_LEN)

typedef struct _cw_key_region {
    char module_name[MODULE_NAME_LEN];
    unsigned char cw_use; /*the count of cw used*/
    unsigned long map[BITMAP_LEN];/*the bit map of cw used*/
    struct list_head node;
} RTK_CW_KEY_REGION;

typedef struct _cw_key_region_mm {
    MUTEX_LOCKTYPE cw_mm_lock;
    struct list_head cw_key_region_list;
    unsigned long cw_map[BITMAP_LEN];/*the bit map of cw used*/
} RTK_CW_KEY_REGION_MM;


int rtk_cw_request_cw(char *module_name, UINT8 cw_index);
int rtk_cw_request_cw_array(char *module_name, UINT8 *index_array, int len);
int rtk_cw_request_multi_cws(char *module_name, UINT8 start_index, int len);
int rtk_cw_release_cw(char *module_name, UINT8 cw_index);
int rtk_cw_release_cw_by_module(char *module_name);
int rtk_cw_set_cw(char *module_name, UINT8 start_index, UINT8 *content, int len);
int rtk_cw_get_cw(char *module_name, UINT8 start_index, UINT8 *content, int len);
int rtk_cw_set_nonseq_cw(char *module_name, UINT8 *index_array, int arr_len, UINT8 *content, int len);
int rtk_cw_get_nonseq_cw(char *module_name, UINT8 *index_array, int arr_len, UINT8 *content, int len);
int rtk_cw_query_free_cw(UINT8 *free_index_array, int len);
int rtk_cw_query_used_cw_by_module(char *module_name, UINT8 *index_array, int len);
int rtk_cw_dump_cw_status(char *buf);
int rtk_cw_module_init(void);
void rtk_cw_module_exit(void);

/**********************Debug define***************************/
#ifdef CW_DBG_EN
#define CW_DBG(fmt, args...)        printf("[CW] DBG, " fmt, ##args)
#else
#define CW_DBG(fmt, args...)
#endif
#define CW_INFO(fmt, args...)       printf("[CW]INFO," fmt, ##args)
#define CW_WARNING(fmt, args...)        printf("[CW]WARNING," fmt, ##args)
#define CW_ERR(fmt, args...)        printf("[CW]ERROR," fmt, ##args)


#endif /* 
 */
