/*
 *
 *  file:rtk_mcp_reg.h
 *
 *  Define the register setting of KCPU-MCP or SCPU-MCP
 *
 */

#ifndef __RTK_MCP_REG_H__
#define __RTK_MCP_REG_H__
#include <rbus/cp_reg.h>
#define CP_REG_BASE                          0xb8015000
#include <rbus/tp_reg.h>

#include <asm/arch/system.h>
#define S_MCP_CTRL                          CP_MCP_CTRL_reg
#define S_MCP_STATUS                        CP_MCP_STATUS_reg
#define S_MCP_EN                            CP_MCP_EN_reg
#define S_MCP_CTRL1                         CP_MCP_CTRL1_reg
#define S_MCP_BASE                          CP_MCP_BASE_reg
#define S_MCP_LIMIT                         CP_MCP_LIMIT_reg
#define S_MCP_RDPTR                         CP_MCP_RDPTR_reg
#define S_MCP_WRPTR                         CP_MCP_WRPTR_reg
#define S_MCP_DES_COUNT                     CP_MCP_DES_COUNT_reg
#define S_MCP_DES_COMPARE                   CP_MCP_DES_COMPARE_reg
#define S_MCP_DES_INI_KEY                   CP_MCP_DES_INI_KEY_0_reg
#define S_MCP_AES_INI_KEY                   CP_MCP_AES_INI_KEY_0_reg
#define S_MCP_MULTI2_SYS_KEY                CP_MCP_MULTI2_SYS_KEY_0_reg
#define CP_OTP_LOAD                        CP_CP_OTP_LOAD_reg
#define S_MCP_RSA_SEL_REG                   CP_S_RSA_SEL_reg


#define WRITE_REG32(reg, val)            rtd_outl(reg, val)
#define READ_REG32(reg)                     rtd_inl(reg)


/* MCP descriptor define */
#define MCP_IV_SEL2(x)               (((x) & 0x01) << 14)
#define MCP_IV_SEL_DSCPT             MCP_IV_SEL2(0)
#define MCP_IV_SEL_CW                MCP_IV_SEL2(1)

#define MCP_KEY_SEL(x)                       (((x) & 0x3) << 12)
#define MCP_KEY_FROM_DSCPT            MCP_KEY_SEL(0)
#define MCP_KEY_FROM_OTP                MCP_KEY_SEL(1)
#define MCP_KEY_FROM_CW                 MCP_KEY_SEL(2)
#define MCP_KEY_FROM_DDR                MCP_KEY_SEL(3)

#define MCP_PATTERN(x)               (((x) & 0x3) << 15)
#define MCP_FULL_CRYPT                 MCP_PATTERN(0)
#define MCP_FULL_SKIP                  MCP_PATTERN(1)
#define MCP_CRYPT_SKIP                 MCP_PATTERN(2)

#define MCP_DOUBLE_DADR(x)           (((x) & 0x1) << 10)
#define MCP_ONE_DADR                   MCP_DOUBLE_DADR(0)
#define MCP_TWO_DADR                   MCP_DOUBLE_DADR(1)

#define MCP_BCM_SEL(x)                       (((x) & 0xf) << 6)
#define MCP_BCM_ECB                          MCP_BCM_SEL(0)
#define MCP_BCM_CBC                          MCP_BCM_SEL(1)
#define MCP_BCM_CTR                          MCP_BCM_SEL(2)
#define MCP_BCM_RCBC_CS                  MCP_BCM_SEL(3)

#define MCP_ENCRYPT                          BIT(5)
#define MCP_DECRYPT                           0x0

#define MCP_MODE_SEL(x)                    ((x) & 0x1f)
#define MCP_MODE_DES                       MCP_MODE_SEL(0x0)
#define MCP_MODE_3DES                     MCP_MODE_SEL(0x1)
#define MCP_MODE_MD5                      MCP_MODE_SEL(0x3)
#define MCP_MODE_SHA1                     MCP_MODE_SEL(0x4)
#define MCP_MODE_AES                       MCP_MODE_SEL(0x5)
#define MCP_MODE_AES_G                   MCP_MODE_SEL(0x6)
#define MCP_MODE_AES_H                   MCP_MODE_SEL(0x7)
#define MCP_MODE_CMAC                    MCP_MODE_SEL(0x8)
#define MCP_MODE_CSA                       MCP_MODE_SEL(0x9)
#define MCP_MODE_MULTI2                 MCP_MODE_SEL(0xa)
#define MCP_MODE_SHA256                 MCP_MODE_SEL(0xb)
#define MCP_MODE_CRC                       MCP_MODE_SEL(0xc)
#define MCP_MODE_CRC_SHA256          MCP_MODE_SEL(0xd)
#define MCP_MODE_AES_192                MCP_MODE_SEL(0x10)
#define MCP_MODE_AES_256                MCP_MODE_SEL(0x11)
#define MCP_MODE_CAM                      MCP_MODE_SEL(0x1d)
#define MCP_MODE_CAM_192               MCP_MODE_SEL(0x1e)
#define MCP_MODE_CAM_256               MCP_MODE_SEL(0x1f)
#define MCP_MODE_SHA384                 MCP_MODE_SEL(0b01110)
#define MCP_MODE_SHA512                 MCP_MODE_SEL(0b01111)


#define MCP_CRYPT_BLK_CNT(x)   (((x) & 0xF) << 28)
#define MCP_SKIP_BLK_CNT(x)     (((x) & 0xF) << 24)
#define MCP_BC(x)                           ((x) & 0x00FFFFF0)

#define DMA_WRITE_FINISH(x)            (((x) & 0x1) << 16)

#define REG_KEY_CW_ENTRY(x)           (((x) & 0x7F)<<8)
#define REG_IV_CW_ENTRY(x)           (((x) & 0x7F)<<0)

#endif
